// Copyright (C) 1996-2004 Florin Ghido, all rights reserved.


// OptimFROG.h contains an interface for file OptimFROG.dll
// which works with OptimFROG Lossless/DualStream files

// Version: 1.100, Date: 2004.04.21


#if !defined(__OptimFROG_h)
#define __OptimFROG_h


#include <stdio.h>


typedef signed char sInt8;
typedef unsigned char uInt8;
typedef signed short int sInt16;
typedef unsigned short int uInt16;
typedef signed long int sInt32;
typedef unsigned long int uInt32;
typedef unsigned char condition;
typedef float Float32;
typedef double Float64;
#if defined(_MSC_VER)
typedef signed __int64 sInt64;
typedef unsigned __int64 uInt64;
#endif
#if defined(__GNUC__)
typedef signed long long int sInt64;
typedef unsigned long long int uInt64;
#endif
#if defined(__BORLANDC__)
typedef signed __int64 sInt64;
typedef unsigned __int64 uInt64;
#endif
#define cTrue 1
#define cFalse 0


#if defined(_MSC_VER)
#pragma pack(push)
#pragma pack(1)
#endif

#if defined(__BORLANDC__)
#pragma option push -a1
#endif

typedef struct
{
  condition (*close)(void* instance);
  sInt32 (*read)(void* instance,void* destBuffer,uInt32 count);
  condition (*eof)(void* instance);
  condition (*seekable)(void* instance);
  sInt64 (*length)(void* instance);
  sInt64 (*getPos)(void* instance);
  condition (*seek)(void* instance,sInt64 pos);
}
#if defined(__GNUC__)
__attribute__((packed))
#endif
ReadInterface;


typedef struct
{
  uInt32 keyCount;
  char* keys[64];
  char* values[64];
}
#if defined(__GNUC__)
__attribute__((packed))
#endif
OptimFROG_Tags;

typedef struct
{
  uInt32 channels;
  uInt32 samplerate;
  uInt32 bitspersample;
  uInt32 bitrate;

  uInt32 version;
  char* method;
  char* speedup;

  sInt64 noPoints;
  sInt64 originalSize;
  sInt64 compressedSize;
  sInt64 length_ms;

  char* sampleType;
  char* channelConfig;
}
#if defined(__GNUC__)
__attribute__((packed))
#endif
OptimFROG_Info;

#if defined(_MSC_VER)
#pragma pack(pop)
#endif

#if defined(__BORLANDC__)
#pragma option pop
#endif


typedef void (*OptimFROG_callBack)(void* callBackParam,Float64 percentage);


#if defined(__cplusplus)
extern "C" {
#define DEFAULT_FALSE =cFalse
#define DEFAULT_NULL =NULL
#else
#define DEFAULT_FALSE 
#define DEFAULT_NULL 
#endif


__declspec(dllimport) uInt32    OptimFROG_getVersion();

__declspec(dllimport) void*     OptimFROG_createInstance();
__declspec(dllimport) void      OptimFROG_destroyInstance(void* decoderInstance);

__declspec(dllimport) condition OptimFROG_openExt(void* decoderInstance,ReadInterface* rInt,void* readerInstance,condition readTags DEFAULT_FALSE);

__declspec(dllimport) condition OptimFROG_open(void* decoderInstance,char* fileName,condition readTags DEFAULT_FALSE);
__declspec(dllimport) condition OptimFROG_close(void* decoderInstance);

__declspec(dllimport) sInt32    OptimFROG_readHead(void* decoderInstance,void* headData,uInt32 maxSize);
__declspec(dllimport) sInt32    OptimFROG_readTail(void* decoderInstance,void* tailData,uInt32 maxSize);

__declspec(dllimport) condition OptimFROG_seekable(void* decoderInstance);
__declspec(dllimport) condition OptimFROG_seekPoint(void* decoderInstance,sInt64 samplePoint);
__declspec(dllimport) condition OptimFROG_seekTime(void* decoderInstance,sInt64 milliseconds);
__declspec(dllimport) sInt64    OptimFROG_getPos(void* decoderInstance);
__declspec(dllimport) condition OptimFROG_recoverableErrors(void* decoderInstance);

__declspec(dllimport) sInt32    OptimFROG_read(void* decoderInstance,void* data,uInt32 noPoints,condition max16bit DEFAULT_FALSE);

__declspec(dllimport) condition OptimFROG_getInfo(void* decoderInstance,OptimFROG_Info* info);
__declspec(dllimport) condition OptimFROG_getTags(void* decoderInstance,OptimFROG_Tags* tags);

__declspec(dllimport) sInt32    OptimFROG_decodeFile(char* sourceFile,char* destinationFile,OptimFROG_callBack callBack DEFAULT_NULL,void* callBackParam DEFAULT_NULL);
__declspec(dllimport) sInt32    OptimFROG_infoFile(char* sourceFile,OptimFROG_Info* info,OptimFROG_Tags* tags DEFAULT_NULL);

#if defined(__cplusplus)
}
#endif


#define OptimFROG_NoError 0
#define OptimFROG_MemoryError 1
#define OptimFROG_OpenError 2
#define OptimFROG_WriteError 3
#define OptimFROG_FatalError 4
#define OptimFROG_RecoverableError 5


#endif// !defined(__OptimFROG_h)
