/* OggDropXPd - Filename: ENCODE.H
 *
 *   Copyright (c) 2002 - 2005 John Edwards <john.edwards33@ntlworld.com>
 *   Jack Moffit <jack@icecast.org> [as the original 'oggdrop' author]
 *
 *   Michael Smith <msmith@xiph.org> [as the author of the original oggenc]
 *
 *   Kenneth Arnold <kcarnold@yahoo.com> and 
 *   Christopher Montgomery <xiphmont@xiph.org>
 *   [as authors of code Mike borrowed from for oggenc]
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __ENCODE_H
#define __ENCODE_H

#include <windows.h>
#include <stdio.h>
#include <vorbis/codec.h>

typedef long (*audio_read_func)(void *src, float **buffer, int samples);
typedef void (*progress_func)(long totalsamples, long samples);
typedef void (*enc_end_func)(char *fn, double time, int rate, long samples, long bytes);
typedef void (*enc_start_func)(char *fn, char *outfn, int bitrate, float quality,int managed);
typedef double (*enc_speed_func)(double time_diff, int rate, long samples);
typedef void (*error_func)(char *errormessage);

int create_directories(char *);

void update_statistics_full(char *fn, long total, long done, double time);
void update_statistics_notime(char *fn, long total, long done, double time);
void update_statistics_null(char *fn, long total, long done, double time);
void start_encode_full(char *fn, char *outfn, int bitrate, float quality, int managed);
void start_encode_null(char *fn, char *outfn, int bitrate, float quality, int managed);
void final_statistics(char *fn, double time, int rate, long total_samples, long bytes);
void final_statistics_null(char *fn, double time, int rate, long total_samples, long bytes);
void encode_error(char *errmsg);

int tag_compare(const char *s1, const char *s2, int n);

typedef enum
{
  OE_MODE_QUALITY,
  OE_MODE_VBR,
  OE_MODE_BITRATE
} OE_MODE;

typedef struct
{
	char *Title;
	char *Artist;
	char *Album;
	char *Year;
	char *Enc_by;
	char *Genre;
	char *lyrics;
	char *lyrics_language;
	char *Separator;
	char *TrackNo;
	char *RenameFormat;
	char *RenameFormatFLAC;
	char *PlaylistFormat;

	int delete_input;
	int AUTOtag;
	int RenameOnTag;
	int PlaylistOnTag;

	int copy_comments;
	int with_skeleton;
	int use_flac_tag;
	int FLAC_scale;

	char *outfile;

	int managed;
	/* All 4 in kbps */
	int vbr_kbps;
	int min_kbps;
	int nom_kbps;
	int max_kbps;

	/* Float from 0 to 1 (low->high) */
	float quality_coefficient;

	int resamplefreq;
	unsigned int converter;
	int downmix;
	double scale;

	unsigned int serial;
	unsigned int skeleton_serial;
	unsigned int kate_serial;
	int fixedserial;
	OE_MODE oeMode;
	int logerr;
	int priority;
	int del_RG;

	/* Advanced Encoder Options */
	int    hard_q_limits;
	int    hard_min;
	int    hard_max;
	int    use_damp;
	double bitrate_average_damping;
	int    use_bias;
	double bitrate_limit_reservoir_bias;
	int    use_bits;
	long   bitrate_limit_reservoir_bits;
	int    use_noise;
	double impulse_noisetune;
//	int    use_trigger;
//	double impulse_trigger_profile;
	double lowpass;
} oe_options;

typedef struct
{
	vorbis_comment *comments;
	unsigned int serialno;
	unsigned int skeleton_serialno;
	unsigned int kate_serialno;

	audio_read_func read_samples;
	progress_func progress_update;
	enc_speed_func enc_speed;
	enc_end_func end_encode;
	enc_start_func start_encode;
	error_func error;
	
	void *readdata;

	long total_samples_per_channel;
	int channels;
	long rate;
	int samplesize;
	int endianness;
	int resamplefreq;
	unsigned int converter;
	double ratio;
	int copy_comments;
	int with_skeleton;

	/* Various bitrate/quality options */
	int managed;
	int vbr_bitrate;
	int min_bitrate;
	int nom_bitrate;
	int max_bitrate;
	float quality_coefficient;
	OE_MODE oeMode;

	FILE *out;
	char *filename;
	double ave_bitrate;
	int del_RG;
	int logerr;

	/* Advanced Encoder Options */
	int    hard_q_limits;
	int    hard_min;
	int    hard_max;
	int    use_damp;
	double bitrate_average_damping;
	int    use_bias;
	double bitrate_limit_reservoir_bias;
	int    use_bits;
	long   bitrate_limit_reservoir_bits;
	int    use_noise;
	double impulse_noisetune;
//	int    use_trigger;
//	double impulse_trigger_profile;
	double lowpass;

	char *lyrics;
	char *lyrics_language;
} oe_enc_opt;


long oe_encode(oe_enc_opt *opt);

typedef struct
{
	enc_speed_func  enc_speed;
	progress_func   progress_update;
	error_func      error;
	int             tagMode;
	int             man_gain;	
	int             decode_mode;
	int	        limiter;
	short	        dither;
	int             del_input;
	int             format;
	double          wrap_prev_pos;
	double          wrap_prev_neg;
	int             downmix;
	char            *input_ogg;
	char            *out_wave;
} oe_dec_opt;


int oe_decode(oe_dec_opt *opt);

typedef struct {
	char	TitleFormat[32];
	int	window_x;
	int	window_y;
	int	always_on_top;
	int	logerr;
	int	shownbr;
	int	mode;
	int	tagging;
	int	quality;
	int	approxbitrate;
	int	minbitrate;
	int	nombitrate;
	int	maxbitrate;
	int	cbrbitrate;
	int	del_input_wav;
	int	with_skeleton;
	int	del_input_ogg;
	char    title[64];
	char    artist[64];
	char    album[64];
	char    enc_by[64];
	char    separator[32];
	char    year[32];
	char    genre[64];
	char    track_no[32];
	int	volume;
	int	limiter;
	int	dither;
	int     dec_tag_mode;
	int     decode_mode;
	int     wav_format;
	int     resample_rate;
	int     resample_mode;
	unsigned int     converter;
	int     out_dir_mode;
	char    out_dir[MAX_PATH];
	char    temp_dir[MAX_PATH];
	float   scale_factor;
	int     lowpass_mode;
	int     lowpass;
	int     rename_on_tag;
	char    rename_format[64];
	int     priority;
	int     playlist_on_tag;
	char    playlist_format[64];
	int     copy_flac_tag;
	int     use_flac_tag;
	int     FLAC_scale;
	char    rename_format_flac[64];
	int     delete_RG;
	int     hard_q_limits;
	int     hard_min;
	int     hard_max;
	int     use_damp;
	double  bit_ave_damp;
	int     use_bias;
	double  bit_res_bias;
	int     use_bits;
	long    bit_res_bits;
	int     use_noise;
	double  imp_noise;
//	int     use_trigger;
//	double  imp_trigger;
	int     mc_downmix;
} SettingsOGG;

/*
 * GLOBALS
 */
SettingsOGG iniSettings;

#endif /* __ENCODE_H */
