/* OggDropXPd - Filename ENCTHREAD.C
 *
 * Function: Controls encoding/decoding processing.
 *
 *   Copyright (c) 2002 - 2005 John Edwards <john.edwards33@ntlworld.com>
 *   Jack Moffit <jack@icecast.org> [as the original 'oggdrop' author]
 *
 *   Michael Smith <msmith@xiph.org> [as the author of the original oggenc]
 *
 *   Kenneth Arnold <kcarnold@yahoo.com> and 
 *   Christopher Montgomery <xiphmont@xiph.org>
 *   [as authors of code Mike borrowed from for oggenc]
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <windows.h>
#include <process.h>
#include <time.h>
#include <string.h>
#include <errno.h>
#include <locale.h>
#include <ShlObj.h>

#include "misc.h"
#include "audio.h"
#include "encode.h"
#include "dither.h"
#include "utf8.h"
#include "i18n.h"

#include <vorbis/codec.h>
#include <vorbis/vorbisfile.h>

#ifndef _M_X64
/*
 * INCLUDES FOR LPAC
 */
#include "./LPAC/lpac_codec_api.h"

/*
 * INCLUDES FOR APE
 */
#include "./APE/MACDll.h"

/*
 * INCLUDES FOR OFR
 */
#include "./OptimFROG_SDK/OptimFROG.h"

/*
 * INCLUDES FOR WV
 */
#include "./WAVPACK/wvunpack_dll.h"
#endif
/*
 * INCLUDES FOR PLAYBACK
 */
#include "wave_out.h"

/*
 * INCLUDES FOR TAGGING
 */
#include "utf8.h"

/*
 * linker options
 */

//#pragma comment(linker, "/delayload:lpac_codec_api.dll")
//#pragma comment(linker, "/delayload:MACDll.dll")
//#pragma comment(linker, "/delayload:OptimFROG.dll")
//#pragma comment(linker, "/delayload:wvunpack.dll")

#define CUSTOM_TAG 1025

#define INVALID_CHARS "<>:\"\\/|*?"

extern char   config_OGGoutdir[],
              config_TEMPdir[];
extern char   approxBRCaption[];
extern char   nomBRCaption[];
extern char   kCaption[];
extern char   speedCaption[];
extern char   fileName[];
extern int    encoding_done,
              encoding,
              decoding,
              animate,
              numfiles,
              totalfiles;
extern double file_complete;
int           stop_encoding,
              stop_decoding;

oe_options    opt;
oe_dec_opt    dec_opts;

CRITICAL_SECTION mutex;

typedef struct enclist_tag {
	char *filename;
	struct enclist_tag *next;
} enclist_t;

enclist_t *head = NULL;

typedef struct taglist_FLAC {
	char *tag;
	struct taglist_FLAC *next_tag;
} taglist_F;

taglist_F *head_tag = NULL;

long encode_thread(LPVOID arg);
//unsigned int WINAPI encode_thread(LPVOID arg);
//void encode_thread(LPVOID arg);

static void add_tag(vorbis_comment *vc, char *name, char *value)
{
	char *utf8;
	if (utf8_encode(value, &utf8) >= 0) {
		if (name == NULL)
			vorbis_comment_add(vc, utf8);
		else
			vorbis_comment_add_tag(vc, name, utf8);
		free(utf8);
	}
}

void encthread_init(void)
{
	DWORD thread_id;
	HANDLE thand;

	numfiles = 0;
	totalfiles = 0;
	file_complete = 0.0;

	InitializeCriticalSection(&mutex);

	thand = (HANDLE)CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)encode_thread, NULL, 0, &thread_id);
	if (thand == NULL) {
		// something bad happened, might want to deal with that, maybe...
	}
}

void encthread_addfile(char *file)
{
	char *filename;
	enclist_t *entry, *node;

	if (file == NULL)
		return;

	// create entry
	filename = strdup(file);
	entry = (enclist_t *)malloc(sizeof(enclist_t));

	entry->filename = filename;
	entry->next = NULL;

	EnterCriticalSection(&mutex);

	// insert entry
	if (head == NULL)
		head = entry;
	else {
		node = head;
		while (node->next != NULL)
			node = node->next;

		node->next = entry;
	}
	numfiles++;
	totalfiles++;

	LeaveCriticalSection(&mutex);
}

/*
 * The caller is responsible for deleting the pointer
 */
char *_getfile()
{
	char *filename;
	enclist_t *entry;

	EnterCriticalSection(&mutex);

	if (head == NULL) {
		LeaveCriticalSection(&mutex);
		return NULL;
	}

	// pop entry
	entry = head;
	head = head->next;

	filename = entry->filename;
	free(entry);

	LeaveCriticalSection(&mutex);

	return filename;
}

/*
 * Creat TAG list for using FLAC Tags in File Renaming and Playlist Creation
 */
void add_TAG(char *flac_tag)
{
	char *tag_name;
	taglist_F *tag_entry, *tag_node;

	if (flac_tag == NULL)
		return;

	// create entry
	utf8_decode(flac_tag, &tag_name);
	tag_entry = (taglist_F *)malloc(sizeof(taglist_F));

	tag_entry->tag = tag_name;
	tag_entry->next_tag = NULL;

	// insert entry
	if (head_tag == NULL)
		head_tag = tag_entry;
	else {
		tag_node = head_tag;
		while (tag_node->next_tag != NULL)
			tag_node = tag_node->next_tag;

		tag_node->next_tag = tag_entry;
	}
}

/*
 * The caller is responsible for deleting the pointer
 */
char *_get_tag()
{
	char *tag_name;
	taglist_F *tag_entry;

	if (head_tag == NULL) {
		return NULL;
	}

	// pop entry
	tag_entry = head_tag;
	head_tag = head_tag->next_tag;

	tag_name = tag_entry->tag;
	free(tag_entry);

	return tag_name;
}

double _enc_speed(double time_diff, int rate, long samples)
{
	return ((double)samples / (double)rate / time_diff);
}

void _nothing_prog(long total, long done)
{
	// do nothing
}

void _error(char *errormessage)
{
	// do nothing
}

void _update(long total, long done)
{
	file_complete = (double)done / (double)total;
}

void CALLBACK ProgressCallback(int nPercentageDone)
{
	// do nothing
}

void set_tag_fields(char **tag_field, char *path_part)
{
	int i, j, len;
	len = strlen(opt.Separator);
	for (j = 0; j < 6; j++) {
		tag_field[j] = NULL;
	}
	for (j = 0; j < 6; j++) {
		if (path_part[0]) {
			tag_field[j] = path_part;
			for (i = 0; path_part[0]; i++) {
				if (strncmp(path_part, opt.Separator, len) != 0)
					path_part++;
				else {
					path_part[0] = '\0';
					path_part += len;
					break;
				}
			}
		} else
			break;
	}
}

void set_tag(char *tag, char *buf_tag, char **GGP_field, char **GP_field, char **P_field, char **FN_field)
{
	char tag_comp[12];
	int  i;

	for (i = 0; i < 6; i++) {
		sprintf(tag_comp, "%%%%%%%%%d", i+1);
		if (!strcmp(tag, tag_comp) && GGP_field[i]) {
			strcpy(buf_tag, GGP_field[i]);
			break;
		}
		sprintf(tag_comp, "%%%%%%%d", i+1);
		if (!strcmp(tag, tag_comp) && GP_field[i]) {
			strcpy(buf_tag, GP_field[i]);
				break;
		}
		sprintf(tag_comp, "%%%%%d", i+1);
		if (!strcmp(tag, tag_comp) && P_field[i]) {
			strcpy(buf_tag, P_field[i]);
			break;
		}
		sprintf(tag_comp, "%%%d", i+1);
		if (!strcmp(tag, tag_comp) && FN_field[i]) {
			strcpy(buf_tag, FN_field[i]);
			break;
		}
	}
}

void encoding_loop(void)
{
	char     *in_file;
	long     nextserial;
	long     next_skeleton_serial;

	/*
	 * We randomly pick a serial number. This is then incremented for each file
	 */
	srand(time(NULL));
	nextserial = rand();
	next_skeleton_serial = nextserial + totalfiles;

	while (in_file = _getfile()) {
		oe_enc_opt      enc_opts;
		char            buf_tag[MAX_PATH],
		                file_temp[MAX_PATH],
		                rename_temp[MAX_PATH],
		                m3u_file[MAX_PATH],
		                in_file_temp[MAX_PATH],
		                *out_fn = NULL,
		                szFile[MAX_PATH],
		                *p,
		                *q,
		                in_path[MAX_PATH],
		                part_path[MAX_PATH],
		                filename[MAX_PATH],
		                *file_name,
		                *parent,
		                *grand_parent,
		                *great_grand_parent,
		                **FN_field,
		                **P_field,
		                **GP_field,
		                **GGP_field,
		                OGGoutdir[MAX_PATH],
		                m3u_out_file[MAX_PATH],
		                strFilename[MAX_PATH],
		                *ext,
		                cd_artist[MAX_PATH],
		                cd_album[MAX_PATH],
		                cd_trackno[MAX_PATH],
		                cd_title[MAX_PATH],
		                cd_year[MAX_PATH],
		                cd_genre[MAX_PATH],
		                tag_buf[MAX_PATH],
		                *flac_comment;
		FILE            *in = NULL,
		                *out = NULL,
		                *m3u = NULL;
		unsigned int    j = 0;
		int             i,
		                k,
				lossless_file = 0,
		                channels,
		                downmixing = 0,
		                resampling = 0,
		                scaling    = 0,
		                dir_only   = 1,
		                new_dir = 0,
		                tag_count = 0,
		                tag_artist = 0,
		                tag_album = 0,
		                tag_title = 0,
		                tag_track = 0,
		                tag_date = 0,
		                tag_genre = 0,
		                tag_sum = 0;
		long            rate,
		                ret;
		float           nominalBitrate;
		double          gain_R = 0.0,
		                gain_A = 0.0;
		input_format    *format;
		vorbis_comment  vc;

		if (stop_encoding) {
			while (in_file = _getfile())
				continue;
			break;
		}

		vorbis_comment_init(&vc);

		animate = 1;

		memset(OGGoutdir, '\0', sizeof(OGGoutdir));
		enc_opts.comments = &vc;
		if (opt.FLAC_scale > 0)
			opt.copy_comments = 1;
		enc_opts.copy_comments = opt.copy_comments;

		strncpy(in_file_temp, in_file, MAX_PATH);
		/*
		 * Create a filename from existing filename, replacing extension with .ogg
		 */
		if (out == NULL) {
			int  len;

			strcpy(in_path, in_file);
			file_name = in_path + strlen(in_path);
			while (file_name[0] != '\\' && file_name[0] != ':' && file_name > in_path)
				file_name--;
			if (file_name[0] == '\\') {
				file_name[0] = '\0';
				file_name++;
			}

			strcpy(part_path, in_path);

			parent = part_path + strlen(part_path);
			while (parent[0] != '\\' && parent[0] != ':' && parent > part_path)
				parent--;
			if (parent[0] == '\\') {
				parent[0] = '\0';
				parent++;
			}

			grand_parent = part_path + strlen(part_path);
			while (grand_parent[0] != '\\' && grand_parent[0] != ':' && grand_parent > part_path)
				grand_parent--;
			if (grand_parent[0] == '\\') {
				grand_parent[0] = '\0';
				grand_parent++;
			}

			great_grand_parent = part_path + strlen(part_path);
			while (great_grand_parent[0] != '\\' && great_grand_parent[0] != ':' && great_grand_parent > part_path)
				great_grand_parent--;
			if (great_grand_parent[0] == '\\') {
				great_grand_parent[0] = '\0';
				great_grand_parent++;
			}

			q = strrchr(file_name, '.');
			q[0] = '\0';
			strcpy(filename, file_name);

			if (config_OGGoutdir[0]) {
//				wchar_t *unicode;
//				int wchars, err;

//				wchars = MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, config_OGGoutdir, strlen(config_OGGoutdir), NULL, 0);
//				unicode = calloc(wchars + 1, sizeof(unsigned short));
//				err = MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, config_OGGoutdir, strlen(config_OGGoutdir), unicode, wchars);
//				SHCreateDirectoryEx(NULL, unicode, NULL);
				SHCreateDirectoryEx(NULL, config_OGGoutdir, NULL);
//				free(unicode);
			}

//			len = strlen(config_OGGoutdir) - 1;
//			if (config_OGGoutdir[len] == '\\')
//				config_OGGoutdir[len] = '\0';
			p = config_OGGoutdir;
		}

		/*
		 * Next section relates to decoding lossless file formats
		 */
		strcpy(szFile, in_file_temp);
		ext = strrchr(szFile, '.');

#ifndef _M_X64

		if (!local_strcasecmp(ext, ".pac")) {
			HANDLE dll;

			if (!config_TEMPdir[0]) {
				if (in_path[strlen(in_path) - 1] == '\\')
					sprintf(in_file_temp,"%s%s.wav",in_path, filename);
				else
					sprintf(in_file_temp,"%s\\%s.wav",in_path, filename);
			}
			else if (config_TEMPdir[strlen(config_TEMPdir)-1] == '\\')
				sprintf(in_file_temp,"%s%s.wav",config_TEMPdir,filename);
			else
				sprintf(in_file_temp,"%s\\%s.wav",config_TEMPdir,filename);

			dll = LoadLibrary("lpac_codec_api.dll");
			if (dll) {
				lossless_file = 1;
				(void) strcpy(approxBRCaption, "Decoding LPAC file");
				strcpy(nomBRCaption, "");
				strcpy(kCaption, "");
				strcpy(speedCaption, "Please Wait . . .");
				DecodePacFile(szFile, in_file_temp);
			}
			else {
				MessageBox(NULL, "lpac_codec_api.dll missing!", "Missing dll", MB_OK | MB_ICONEXCLAMATION);
				numfiles--;
				continue;
			}
			FreeLibrary(dll);
		}
		else if (!local_strcasecmp(ext, ".ape")) {
			HANDLE dll;

			if (!config_TEMPdir[0]) {
				if (in_path[strlen(in_path) - 1] == '\\')
					sprintf(in_file_temp,"%s%s.wav",in_path, filename);
				else
					sprintf(in_file_temp,"%s\\%s.wav",in_path, filename);
			}
			else if (config_TEMPdir[strlen(config_TEMPdir)-1] == '\\')
				sprintf(in_file_temp,"%s%s.wav",config_TEMPdir,filename);
			else
				sprintf(in_file_temp,"%s\\%s.wav",config_TEMPdir,filename);

			dll = LoadLibrary("MACDll.dll");
			if (dll) {
				int nPercentageDone = 0;		//the percentage done... continually updated by the decoder
				int nKillFlag = 0;			//the kill flag for controlling the decoder
				lossless_file = 1;
				(void) strcpy(approxBRCaption, "Decoding APE file");
				strcpy(nomBRCaption, "");
				strcpy(kCaption, "");
				strcpy(speedCaption, "Please Wait . . .");
				DecompressFile(szFile, in_file_temp, &nPercentageDone, ProgressCallback, &nKillFlag);
			}
			else {
				MessageBox(NULL, "MACDll.dll missing!", "Missing dll", MB_OK | MB_ICONEXCLAMATION);
				numfiles--;
				continue;
			}
			FreeLibrary(dll);
		}
		else if (!local_strcasecmp(ext, ".ofr") || !local_strcasecmp(ext, ".ofs")) {
			HANDLE dll;

			if (!config_TEMPdir[0]) {
				if (in_path[strlen(in_path) - 1] == '\\')
					sprintf(in_file_temp,"%s%s.wav",in_path, filename);
				else
					sprintf(in_file_temp,"%s\\%s.wav",in_path, filename);
			}
			else if (config_TEMPdir[strlen(config_TEMPdir)-1] == '\\')
				sprintf(in_file_temp,"%s%s.wav",config_TEMPdir,filename);
			else
				sprintf(in_file_temp,"%s\\%s.wav",config_TEMPdir,filename);

			dll = LoadLibrary("OptimFROG.dll");
			if (dll) {
				lossless_file = 1;
				(void) strcpy(approxBRCaption, "Decoding OFR/OFS file");
				strcpy(nomBRCaption, "");
				strcpy(kCaption, "");
				strcpy(speedCaption, "Please Wait . . .");
				OptimFROG_decodeFile(szFile, in_file_temp, NULL, NULL);
			}
			else {
				MessageBox(NULL, "OptimFROG.dll missing!", "Missing dll", MB_OK | MB_ICONEXCLAMATION);
				numfiles--;
				continue;
			}
			FreeLibrary(dll);
		}
		else if (!local_strcasecmp(ext, ".wv")) {
			HANDLE dll;
			int result;

			if (!config_TEMPdir[0]) {
				if (in_path[strlen(in_path) - 1] == '\\')
					sprintf(in_file_temp,"%s%s.wav",in_path, filename);
				else
					sprintf(in_file_temp,"%s\\%s.wav",in_path, filename);
			}
			else if (config_TEMPdir[strlen(config_TEMPdir)-1] == '\\')
				sprintf(in_file_temp,"%s%s.wav",config_TEMPdir,filename);
			else
				sprintf(in_file_temp,"%s\\%s.wav",config_TEMPdir,filename);

			dll = LoadLibrary("wvunpack.dll");
			if (dll) {
				lossless_file = 1;
				(void) strcpy(approxBRCaption, "Decoding WV file");
				strcpy(nomBRCaption, "");
				strcpy(kCaption, "");
				strcpy(speedCaption, "Please Wait . . .");
				result = unpack_file (szFile, in_file_temp);
			}
			else {
				MessageBox(NULL, "wvunpack.dll missing!", "Missing dll", MB_OK | MB_ICONEXCLAMATION);
				numfiles--;
				continue;
			}
			FreeLibrary(dll);
		}
		/*
		 * End of lossless decoding
		 */
#endif

		set_filename(in_file_temp);

		in = fopen(in_file_temp, "rb");

		if (in == NULL) {
			error_handler("ERROR: Cannot open input file %s", in_file_temp);
			numfiles--;
			continue;
		}

		/*
		 * Now, we need to select an input audio format
		 */
		enc_opts.del_RG = opt.del_RG;
		enc_opts.logerr = opt.logerr;

		format = open_audio_file(in, &enc_opts);
		if (!format) {
			fclose(in);
			/* error reported by reader */
			numfiles--;
			continue;
		}

		/*
		 * Identify and fill fields for tagging
		 */

		GGP_field = malloc(sizeof(char *) * 6);
		GP_field = malloc(sizeof(char *) * 6);
		P_field = malloc(sizeof(char *) * 6);
		FN_field = malloc(sizeof(char *) * 6);
		/*
		 * Set up the Great Grand-parent fields
		 */
		if (great_grand_parent[0])
			set_tag_fields(GGP_field, great_grand_parent);
		/*
		 * Set up the Grand-parent fields
		 */
		if (grand_parent[0])
			set_tag_fields(GP_field, grand_parent);
		/*
		 * Set up the Parent fields
		 */
		if (parent[0])
			set_tag_fields(P_field, parent);
		/*
		 * Set up the File name fields
		 */
		if (file_name[0])
			set_tag_fields(FN_field, file_name);

		if (opt.AUTOtag) {

			/*
			 * Set up Tags
			 */
			memset(buf_tag, '\0', sizeof(buf_tag));
			if (opt.Artist[0]) {
				if (opt.Artist[0] == '%')
					set_tag(opt.Artist, buf_tag, GGP_field, GP_field, P_field, FN_field);
				else
					strcpy(buf_tag, opt.Artist);

				add_tag(enc_opts.comments, "ARTIST", buf_tag);
			}

			memset(buf_tag, '\0', sizeof(buf_tag));
			if (opt.Album[0]) {
				if (opt.Album[0] == '%')
					set_tag(opt.Album, buf_tag, GGP_field, GP_field, P_field, FN_field);
				else
					strcpy(buf_tag, opt.Album);

				add_tag(enc_opts.comments, "ALBUM", buf_tag);
			}

			memset(buf_tag, '\0', sizeof(buf_tag));
			if (opt.Title[0]) {
				if (opt.Title[0] == '%')
					set_tag(opt.Title, buf_tag, GGP_field, GP_field, P_field, FN_field);
				else
					strcpy(buf_tag, opt.Title);

				add_tag(enc_opts.comments, "TITLE", buf_tag);
			}

			memset(buf_tag, '\0', sizeof(buf_tag));
			if (opt.TrackNo[0]) {
				if (opt.TrackNo[0] == '%')
					set_tag(opt.TrackNo, buf_tag, GGP_field, GP_field, P_field, FN_field);
				else
					strcpy(buf_tag, opt.TrackNo);

				add_tag(enc_opts.comments, "TRACKNUMBER", buf_tag);
			}

			memset(buf_tag, '\0', sizeof(buf_tag));
			if (opt.Year[0]) {
				if (opt.Year[0] == '%')
					set_tag(opt.Year, buf_tag, GGP_field, GP_field, P_field, FN_field);
				else
					strcpy(buf_tag, opt.Year);

				add_tag(enc_opts.comments, "DATE", buf_tag);
			}

			memset(buf_tag, '\0', sizeof(buf_tag));
			if (opt.Genre[0]) {
				if (opt.Genre[0] == '%')
					set_tag(opt.Genre, buf_tag, GGP_field, GP_field, P_field, FN_field);
				else
					strcpy(buf_tag, opt.Genre);

				add_tag(enc_opts.comments, "GENRE", buf_tag);
			}


			add_tag(enc_opts.comments, "ENCODED_BY", opt.Enc_by);
		}

		else if (opt.copy_comments) {
			char invalid_chars[10];
			sprintf(invalid_chars, INVALID_CHARS);
			while (flac_comment = _get_tag()) {
				if (!tag_compare(flac_comment, "RG_RADIO=", 9)) {
					for (j = 9, k = 0; j <= strlen(flac_comment); j++, k++)
						tag_buf[k] = flac_comment[j];
					gain_R = atof(tag_buf);
				}
				else if (!tag_compare(flac_comment, "REPLAYGAIN_TRACK_GAIN=", 22)) {
					for (j = 22, k = 0; j <= strlen(flac_comment); j++, k++)
						tag_buf[k] = flac_comment[j];
					gain_R = atof(tag_buf);
				}
				else if (!tag_compare(flac_comment, "RG_AUDIOPHILE=", 14)) {
					for (j = 14, k = 0; j <= strlen(flac_comment); j++, k++)
						tag_buf[k] = flac_comment[j];
					gain_A = atof(tag_buf);
				}
				else if (!tag_compare(flac_comment, "REPLAYGAIN_ALBUM_GAIN=", 22)) {
					for (j = 22, k = 0; j <= strlen(flac_comment); j++, k++)
						tag_buf[k] = flac_comment[j];
					gain_A = atof(tag_buf);
				}
				else if (!tag_compare(flac_comment, "ARTIST=", 7)) {
					if (strlen(flac_comment) < 8)
						strcpy(cd_artist, "ARTIST");
					else {
						for (j = 7, k = 0; j <= strlen(flac_comment); j++, k++)
							cd_artist[k] = flac_comment[j];
						for (i = 0; i < 9; i++) {
							for (j = 0; j < strlen(cd_artist); j++) {
								if (cd_artist[j] == invalid_chars[i])
									cd_artist[j] = '_';
							}
						}
					}
					tag_count++;
					tag_artist = k;
					tag_sum += k;
				}
				else if (!tag_compare(flac_comment, "ALBUM=", 6)) {
					if (strlen(flac_comment) < 7)
						strcpy(cd_album, "ALBUM");
					else {
						for (j = 6, k = 0; j <= strlen(flac_comment); j++, k++)
							cd_album[k] = flac_comment[j];
						for (i = 0; i < 9; i++) {
							for (j = 0; j < strlen(cd_album); j++) {
								if (cd_album[j] == invalid_chars[i])
									cd_album[j] = '_';
							}
						}
					}
					tag_count++;
					tag_album = k;
					tag_sum += k;
				}
				else if (!tag_compare(flac_comment, "TRACKNUMBER=", 12)) {
					if (strlen(flac_comment) < 13)
						strcpy(cd_trackno, "TRACKNUMBER");
					else {
						for (j = 12, k = 0; j <= strlen(flac_comment); j++, k++)
							cd_trackno[k] = flac_comment[j];
						for (i = 0; i < 9; i++) {
							for (j = 0; j < strlen(cd_trackno); j++) {
								if (cd_trackno[j] == invalid_chars[i])
									cd_trackno[j] = '_';
							}
						}
					}
					if (strlen(cd_trackno) < 2) {
						strcpy(tag_buf, "0");
						strcat(tag_buf, cd_trackno);
						strcpy(cd_trackno, tag_buf);
					}
					tag_count++;
					tag_track = k;
					tag_sum += k;
				}
				else if (!tag_compare(flac_comment, "TITLE=", 6)) {
					if (strlen(flac_comment) < 7)
						strcpy(cd_title, "TITLE");
					else {
						for (j = 6, k = 0; j <= strlen(flac_comment); j++, k++)
							cd_title[k] = flac_comment[j];
						for (i = 0; i < 9; i++) {
							for (j = 0; j < strlen(cd_title); j++) {
								if (cd_title[j] == invalid_chars[i])
									cd_title[j] = '_';
							}
						}
					}
					tag_count++;
					tag_title = k;
					tag_sum += k;
				}
				else if (!tag_compare(flac_comment, "DATE=", 5)) {
					if (strlen(flac_comment) < 6)
						strcpy(cd_year, "YEAR");
					else {
						for (j = 5, k = 0; j <= strlen(flac_comment); j++, k++)
							cd_year[k] = flac_comment[j];
						for (i = 0; i < 9; i++) {
							for (j = 0; j < strlen(cd_year); j++) {
								if (cd_year[j] == invalid_chars[i])
									cd_year[j] = '_';
							}
						}
					}
					tag_count++;
					tag_date = k;
					tag_sum += k;
				}
				else if (!tag_compare(flac_comment, "GENRE=", 6)) {
					if (strlen(flac_comment) < 7)
						strcpy(cd_genre, "GENRE");
					else {
						for (j = 6, k = 0; j <= strlen(flac_comment); j++, k++)
							cd_genre[k] = flac_comment[j];
						for (i = 0; i < 9; i++) {
							for (j = 0; j < strlen(cd_genre); j++) {
								if (cd_genre[j] == invalid_chars[i])
									cd_genre[j] = '_';
							}
						}
					}
					tag_count++;
					tag_genre = k;
					tag_sum += k;
				}
				if (flac_comment)
					free(flac_comment);
			}
		}

		if (opt.RenameOnTag || (opt.use_flac_tag && tag_count)) {
			int path_length = 0;
			strcpy(OGGoutdir, (!config_OGGoutdir[0]) ? in_path : config_OGGoutdir);
			path_length = strlen(OGGoutdir);
			if (opt.RenameOnTag) {
				memset(rename_temp, '\0', sizeof(rename_temp));
				i = 0;
				while (opt.RenameFormat[i]) {
					if (opt.RenameFormat[i] == '%') {
						memset(buf_tag, '\0', sizeof(buf_tag));
						for (j = 0; opt.RenameFormat[i]; i++, j++) {
							if (opt.RenameFormat[i] == '%' || isdigit(opt.RenameFormat[i]))
								file_temp[j] = opt.RenameFormat[i];
							else {
								file_temp[j] = '\0';
								break;
							}
						}
						file_temp[j] = '\0';
						set_tag(file_temp, buf_tag, GGP_field, GP_field, P_field, FN_field);
						strncat(rename_temp, buf_tag, MAX_PATH - strlen(rename_temp) - path_length);
					}
					else if (opt.RenameFormat[i]) {
						j = 0;
						while (opt.RenameFormat[i] && opt.RenameFormat[i] != '%') {
							buf_tag[j] = opt.RenameFormat[i];
							i++;
							j++;
						}
						buf_tag[j] = '\0';
						strncat(rename_temp, buf_tag, MAX_PATH - strlen(rename_temp) - path_length);
					}
				}
			}
			else {
				int path_max = MAX_PATH;
				for (i = 0; opt.RenameFormatFLAC[i]; i++) {
					if (opt.RenameFormatFLAC[i] == '%')
						i++;
					else
						tag_sum++;
				}
				if (strlen(opt.RenameFormatFLAC)-1 == '\\')
					path_max -= (strlen(filename) - 6);
				else
					path_max -= 6;
				if ((tag_sum + path_length) > path_max)
					tag_sum = path_max - path_length - tag_track - tag_date;
				if ((tag_artist + tag_album + tag_genre) < (tag_sum * 3 / 4))
					tag_title = tag_sum - tag_artist - tag_album - tag_genre;
				else if ((tag_artist + tag_title + tag_genre) < (tag_sum * 3 / 4))
					tag_album = tag_sum - tag_artist - tag_title - tag_genre;
				else if ((tag_album + tag_title + tag_genre) < (tag_sum * 3 / 4))
					tag_artist = tag_sum - tag_album - tag_title - tag_genre;
				else if ((tag_artist + tag_album + tag_title) < (tag_sum * 3 / 4))
					tag_genre = tag_sum - tag_album - tag_title - tag_artist;
				else
					tag_artist = tag_album = tag_title = tag_genre = tag_sum / 4;
				memset(rename_temp, '\0', sizeof(rename_temp));
				i = 0;
				while (opt.RenameFormatFLAC[i]) {
					if (opt.RenameFormatFLAC[i] == '%') {
						memset(buf_tag, '\0', sizeof(buf_tag));
						j = 0;
						file_temp[j] = opt.RenameFormatFLAC[i];
						file_temp[j+1] = opt.RenameFormatFLAC[i+1];

						file_temp[j+2] = '\0';
						i += 2;
						if (!strcmp(file_temp, "%D")) {
							if (cd_artist[0])
								strncpy(buf_tag, cd_artist, tag_artist);
							else
								strcpy(buf_tag, "ARTIST");
						}
						else if (!strcmp(file_temp, "%C")) {
							if (cd_album[0])
								strncpy(buf_tag, cd_album, tag_album);
							else
								strcpy(buf_tag, "ALBUM");
						}
						else if (!strcmp(file_temp, "%N")) {
							if (cd_trackno[0])
								strcpy(buf_tag, cd_trackno);
							else
								strcpy(buf_tag, "TRACKNUMBER");
						}
						else if (!strcmp(file_temp, "%T")) {
							if (cd_title[0])
								strncpy(buf_tag, cd_title, tag_title);
							else
								strcpy(buf_tag, "TITLE");
						}
						else if (!strcmp(file_temp, "%Y")) {
							if (cd_year[0])
								strcpy(buf_tag, cd_year);
							else
								strcpy(buf_tag, "YEAR");
						}
						else if (!strcmp(file_temp, "%B")) {
							if (cd_genre[0])
								strncpy(buf_tag, cd_genre, tag_genre);
							else
								strcpy(buf_tag, "GENRE");
						}
						strncat(rename_temp, buf_tag, path_max - strlen(rename_temp) - path_length);
					}
					else if (opt.RenameFormatFLAC[i]) {
						j = 0;
						while (opt.RenameFormatFLAC[i] && opt.RenameFormatFLAC[i] != '%') {
							buf_tag[j] = opt.RenameFormatFLAC[i];
							i++;
							j++;
						}
						buf_tag[j] = '\0';
						strncat(rename_temp, buf_tag, path_max - strlen(rename_temp) - path_length);
					}
				}
			}
			dir_only = (rename_temp[strlen(rename_temp)-1] == '\\') ? 1 : 0;
			strcpy(buf_tag, rename_temp);

			for (i = 0, j = 0; buf_tag[i]; i++, j++) {
				if (buf_tag[i] != '\\')
					rename_temp[j] = buf_tag[i];
				else {
//					wchar_t *unicode;
//					int wchars, err;
					rename_temp[j] = '\0';
					if (OGGoutdir[strlen(OGGoutdir)-1] != '\\') {
						k = strlen(OGGoutdir);
						OGGoutdir[k] = '\\';
						OGGoutdir[k+1] = '\0';
						strcat(OGGoutdir, rename_temp);
					}
					else
						strcat(OGGoutdir, rename_temp);
					j = -1;
					new_dir = 1;

//					wchars = MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, OGGoutdir, strlen(OGGoutdir), NULL, 0);
//					unicode = calloc(wchars + 1, sizeof(unsigned short));
//					err = MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, OGGoutdir, strlen(OGGoutdir), unicode, wchars);
//					SHCreateDirectoryEx(NULL, unicode, NULL);
					SHCreateDirectoryEx(NULL, OGGoutdir, NULL);
//					free(unicode);
				}
			}
			rename_temp[j] = '\0';

			if (dir_only) {
				if (OGGoutdir[0] == '\\')
					sprintf(strFilename,"%s%s.ogg",OGGoutdir,filename);
				else
					sprintf(strFilename,"%s\\%s.ogg",OGGoutdir,filename);
			}
			else {
				if (OGGoutdir[0] == '\\')
					sprintf(strFilename,"%s%s.ogg",OGGoutdir,rename_temp);
				else
					sprintf(strFilename,"%s\\%s.ogg",OGGoutdir,rename_temp);
			}
		}
		else {
			if (!config_OGGoutdir[0]) {
				if (in_path[strlen(in_path) - 1] == '\\')
					sprintf(strFilename,"%s%s.ogg",in_path, filename);
				else
					sprintf(strFilename,"%s\\%s.ogg",in_path, filename);
			}
			else if (p[0] == '\\')
				sprintf(strFilename,"%s%s.ogg",config_OGGoutdir,filename);
			else
				sprintf(strFilename,"%s\\%s.ogg",config_OGGoutdir,filename);
		}

		if (opt.PlaylistOnTag) {
			char *pf;
			char *slash_point;
			char copy_parent[MAX_PATH];
			char copy_grand_parent[MAX_PATH];
			int  len,
		             len_sep = strlen(opt.Separator),
		             pf_slash_count = 0,
		             path_slash_count = 0;

			if (opt.RenameOnTag || (opt.use_flac_tag && tag_count)) {
				strcpy(in_path, strFilename);
				file_name = in_path + strlen(in_path);
				while (file_name[0] != '\\' && file_name[0] != ':' && file_name > in_path)
					file_name--;
				if (file_name[0] == '\\') {
					file_name[0] = '\0';
					file_name++;
				}

				strcpy(filename, file_name);
				strcpy(part_path, in_path);

				parent = part_path + strlen(part_path);
				while (parent[0] != '\\' && parent[0] != ':' && parent > part_path)
					parent--;
				if (parent[0] == '\\') {
					parent[0] = '\0';
					parent++;
				}
				strcpy(copy_parent, parent);

				grand_parent = part_path + strlen(part_path);
				while (grand_parent[0] != '\\' && grand_parent[0] != ':' && grand_parent > part_path)
					grand_parent--;
				if (grand_parent[0] == '\\') {
					grand_parent[0] = '\0';
					grand_parent++;
				}
				strcpy(copy_grand_parent, grand_parent);

				great_grand_parent = part_path + strlen(part_path);
				while (great_grand_parent[0] != '\\' && great_grand_parent[0] != ':' && great_grand_parent > part_path)
					great_grand_parent--;
				if (great_grand_parent[0] == '\\') {
					great_grand_parent[0] = '\0';
					great_grand_parent++;
				}

				q = strrchr(file_name, '.');
				q[0] = '\0';
				strcpy(filename, file_name);

				/*** Repeat for New Path ***/

				/*
				 * Set up the Great Grand-parent fields
				 */
				if (great_grand_parent[0])
					set_tag_fields(GGP_field, great_grand_parent);
				/*
				 * Set up the Grand-parent fields
				 */
				if (grand_parent[0])
					set_tag_fields(GP_field, grand_parent);
				/*
				 * Set up the Parent fields
				 */
				if (parent[0])
					set_tag_fields(P_field, parent);
				/*
				 * Set up the File name fields
				 */
				if (file_name[0])
					set_tag_fields(FN_field, file_name);
			}

			i = 0;
			while (strFilename[i]) {
				if (strFilename[i] == '\\')
					path_slash_count++;
				i++;
			}
			i = 0;
			while (opt.PlaylistFormat[i]) {
				if (opt.PlaylistFormat[i] == '\\')
					pf_slash_count++;
				i++;
			}
			if (pf_slash_count > path_slash_count)
				pf_slash_count = path_slash_count;
			if (pf_slash_count > 0) {
				pf = strrchr(opt.PlaylistFormat, '\\');
				pf++;
			} else
				pf = opt.PlaylistFormat;
			memset(m3u_file, '\0', sizeof(m3u_file));
			i = 0;

			while (pf[i]) {
				if (pf[i] == '%') {
					memset(buf_tag, '\0', sizeof(buf_tag));
					for (j = 0; pf[i]; i++, j++) {
						if (pf[i] == '%' || isdigit(pf[i]))
							file_temp[j] = pf[i];
						else {
							file_temp[j] = '\0';
							break;
						}
					}
					file_temp[j] = '\0';
					set_tag(file_temp, buf_tag, GGP_field, GP_field, P_field, FN_field);
					strcat(m3u_file, buf_tag);
				}
				else if (pf[i]) {
					j = 0;
					while (pf[i] != '%') {
						buf_tag[j] = pf[i];
						i++;
						j++;
					}
					buf_tag[j] = '\0';
					strcat(m3u_file, buf_tag);
				}
			}
			strcpy(buf_tag, strFilename);
			strcpy(tag_buf, strFilename);
			pf = strrchr(buf_tag, '\\');
			pf[1] = '\0';
			len = strlen(buf_tag);
			i = -1;
			while (len > 0) {
				if (pf_slash_count == i) {
					len += 2;
					slash_point = buf_tag + len;
					slash_point[0] = '\0';
					break;
				} else if (buf_tag[len] == '\\')
					i++;
				len--;
			}
			strcpy(m3u_out_file, buf_tag);
			strcat(m3u_out_file, m3u_file);
			pf = tag_buf + len;
			strcpy(m3u_file, pf);
		}

		out_fn = strFilename;

		/*
		 * Set various encoding defaults
		 */
		enc_opts.serialno = nextserial++;
		enc_opts.skeleton_serialno = next_skeleton_serial++;
		enc_opts.progress_update = _update;
		enc_opts.enc_speed = _enc_speed;
		enc_opts.error = _error;
		enc_opts.with_skeleton = opt.with_skeleton;

		/*
		 * OK. We can read the file - so now open the output file
		 */
		out = fopen(out_fn, "wb");
		if (out == NULL) {
			error_handler("ERROR: Cannot open output file %s", out_fn);
			numfiles--;
			fclose(in);
			continue;
		}	

		/*
		 * Now, set the rest of the options
		 */	
		enc_opts.out                          = out;
		enc_opts.filename                     = out_fn;
		enc_opts.vbr_bitrate                  = opt.vbr_kbps;
		enc_opts.managed                      = opt.managed;
		enc_opts.min_bitrate                  = opt.min_kbps;
		enc_opts.nom_bitrate                  = opt.nom_kbps;
		enc_opts.max_bitrate                  = opt.max_kbps;
		enc_opts.lowpass                      = opt.lowpass;
		enc_opts.quality_coefficient          = opt.quality_coefficient;
		enc_opts.oeMode                       = opt.oeMode;
		enc_opts.hard_q_limits                = opt.hard_q_limits;
		enc_opts.hard_min                     = opt.hard_min;
		enc_opts.hard_max                     = opt.hard_max;
		enc_opts.use_damp                     = opt.use_damp;
		enc_opts.bitrate_average_damping      = opt.bitrate_average_damping;
		enc_opts.use_bias                     = opt.use_bias;
		enc_opts.bitrate_limit_reservoir_bias = opt.bitrate_limit_reservoir_bias;
		enc_opts.use_bits                     = opt.use_bits;
		enc_opts.bitrate_limit_reservoir_bits = opt.bitrate_limit_reservoir_bits;
		enc_opts.use_noise                    = opt.use_noise;
		enc_opts.impulse_noisetune            = opt.impulse_noisetune;
//		enc_opts.use_trigger                  = opt.use_trigger;
//		enc_opts.impulse_trigger_profile      = opt.impulse_trigger_profile;

		channels = enc_opts.channels;
		rate = enc_opts.rate;

		if (opt.oeMode == OE_MODE_BITRATE) {
			if (opt.nom_kbps < 45 && enc_opts.rate > opt.resamplefreq) {
				enc_opts.converter = opt.converter;
				enc_opts.resamplefreq = opt.resamplefreq;
				enc_opts.ratio = (1.0 * opt.resamplefreq) / enc_opts.rate ;
				if (setup_resample(&enc_opts)) {
					numfiles--;
					continue;
				}
				resampling = 1;
			}
			else if (opt.nom_kbps > 44 && opt.resamplefreq && opt.resamplefreq != enc_opts.rate) {
				enc_opts.converter = opt.converter;
				enc_opts.resamplefreq = opt.resamplefreq;
				enc_opts.ratio = (1.0 * opt.resamplefreq) / enc_opts.rate ;
				if (setup_resample(&enc_opts)) {
					numfiles--;
					continue;
				}
				resampling = 1;
			}
		}
		else {
			if (opt.resamplefreq && opt.resamplefreq != enc_opts.rate) {
				enc_opts.converter = opt.converter;
				enc_opts.resamplefreq = opt.resamplefreq;
				enc_opts.ratio = (1.0 * opt.resamplefreq) / enc_opts.rate ;
				if (setup_resample(&enc_opts)) {
					numfiles--;
					continue;
				}
				resampling = 1;
			}
		}

		if (opt.downmix) {
			if (enc_opts.channels == 2) {
				setup_downmix(&enc_opts);
				downmixing = 1;
			}
			else{
				error_handler("ERROR: Can't downmix except from stereo to mono\n");
				numfiles--;
				if (resampling) {
					clear_resample(&enc_opts);
					resampling = 0;
				}
				continue;
			}
		}

		if (opt.FLAC_scale == 1)	// Title Gain
			opt.scale = pow(10., gain_R * 0.05);
		else if (opt.FLAC_scale == 2)	// Album Gain
			opt.scale = pow(10., gain_A * 0.05);
		if (opt.scale != 1.0) {
			setup_scaler(&enc_opts, (float)opt.scale);
			scaling = 1;
		}

		if (!enc_opts.total_samples_per_channel)
			enc_opts.progress_update = _nothing_prog;

		sprintf(fileName,"%s.ogg", filename);

		ret = oe_encode(&enc_opts); /* ret = bytes written */

		if (opt.logerr) {
			char date_buf[12],
			     time_buf[12];
			GetDateFormat(LOCALE_USER_DEFAULT, DATE_SHORTDATE, NULL,
			              NULL, date_buf, 12);
			GetTimeFormat(LOCALE_USER_DEFAULT, TIME_FORCE24HOURFORMAT,
			              NULL, NULL, time_buf, 12);
			error_handler("***********************************");
			error_handler(" %s - %s", date_buf, time_buf);
			error_handler(" Input filename: %s", szFile);
			error_handler(" Output filename: %s", out_fn);
			error_handler(" Output filesize: %ld bytes", ret);
			error_handler(" No. of Channels: %d", channels);
			error_handler(" Sampling Rate: %dHz", rate);
			error_handler(" Total Samples: %d", enc_opts.total_samples_per_channel);
			error_handler(" Length: %d seconds", enc_opts.total_samples_per_channel/enc_opts.rate);
			if (opt.oeMode == OE_MODE_QUALITY) {
				error_handler(" Quality setting: %0.2f", opt.quality_coefficient*10);
			}
			else if (opt.oeMode == OE_MODE_VBR) {
				error_handler(" Quality setting Approximate Bitrate: %dkbps", opt.vbr_kbps);
			}
			else {
				error_handler(" Using Bitrate Management:");
				error_handler(" Min. Bitrate: %dkbps", opt.min_kbps);
				error_handler(" Nom. Bitrate: %dkbps", opt.nom_kbps);
				error_handler(" Max. Bitrate: %dkbps", opt.max_kbps);
			}
			error_handler(" Average Bitrate: %0.1fkbps", enc_opts.ave_bitrate);
			if (resampling)
				error_handler(" Re-sampling to: %dHz", opt.resamplefreq);
			if (downmixing)
				error_handler(" Downmixing to Mono");
			if (scaling)
				error_handler(" Scaling to: %0.2f%%", (float)opt.scale*100);
			if (opt.lowpass)
				error_handler(" Lowpass at: %5.0fHz", opt.lowpass*1000);
			if (opt.use_damp)
				error_handler(" Applied Bitrate Average Damping of: %lfsecs", opt.bitrate_average_damping);
			if (opt.use_bias)
				error_handler(" Applied Bitrate Limit Reservoir Bias of: %lfsecs", opt.bitrate_limit_reservoir_bias);
			if (opt.use_bits)
				error_handler(" Applied Bitrate Limit Reservoir Bits of: %dbits", opt.bitrate_limit_reservoir_bits);
			if (opt.use_noise)
				error_handler(" Applied Impulse Noisetune of: %lf", opt.impulse_noisetune);
//			if (opt.use_trigger)
//				error_handler(" Applied Impulse Trigger Profile of: %lf", opt.impulse_trigger_profile);
			if (stop_encoding)
				error_handler(" *** WARNING: Encoding NOT completed ***");
		}

		if (opt.oeMode == OE_MODE_QUALITY) {
			nominalBitrate = qc2approxBitrate(opt.quality_coefficient*1000, enc_opts.rate, enc_opts.channels);
			strcpy(approxBRCaption, "Nominal Bitrate: ");
			sprintf(nomBRCaption, "%.1fkbps", nominalBitrate/1000);
			sprintf(kCaption, "-q %.2f", (float)opt.quality_coefficient*10);
			strcpy(speedCaption, "");
		}
		else if (opt.oeMode == OE_MODE_VBR) {
			strcpy(approxBRCaption, "Approximate Bitrate: ");
			sprintf(nomBRCaption, "%.1fkbps", (float)opt.vbr_kbps);
			strcpy(kCaption, "");
			strcpy(speedCaption, "");
		}
		else {
			strcpy(approxBRCaption, "Bitrate: ");
			sprintf(nomBRCaption, "%.1fkbps", (float)opt.nom_kbps);
			strcpy(kCaption, "");
			strcpy(speedCaption, "");
		}

		if (!stop_encoding && opt.PlaylistOnTag) {
			FILE *fp_temp;
			char buff[MAX_PATH];
			int entry_written = 0;
			strcat(m3u_out_file, ".m3u");
			if ((m3u = fopen (m3u_out_file, "r")) == NULL) {
				errno = 0;
				if ((m3u = fopen (m3u_out_file, "w")) == NULL)
					MessageBox(NULL, "Unable to open Playlist File - 1",
				        	   NULL, MB_OK | MB_ICONEXCLAMATION);
				else {
					fprintf (m3u, "%s\n", m3u_file);
					fclose (m3u);
				}
			}
			else if ((fp_temp = fopen ("m3u.tmp", "w")) == NULL)
				MessageBox(NULL, "Unable to open Playlist File - 2",
				           NULL, MB_OK | MB_ICONEXCLAMATION);
			else {
				while (fgets(buff, sizeof(buff), m3u) != NULL) {
					if (strncmp(buff, m3u_file, strlen(m3u_file)) < 0)
						fprintf (fp_temp, "%s", buff);
					else if (strncmp(buff, m3u_file, strlen(m3u_file)) == 0) {
						entry_written = 1;
						fprintf (fp_temp, "%s", buff);
						while (fgets(buff, sizeof(buff), m3u) != NULL)
							fprintf (fp_temp, "%s", buff);
					}
					else if (strncmp(buff, m3u_file, strlen(m3u_file)) > 0) {
						fprintf (fp_temp, "%s\n", m3u_file);
						entry_written = 1;
						fprintf (fp_temp, "%s", buff);
						while (fgets(buff, sizeof(buff), m3u) != NULL)
							fprintf (fp_temp, "%s", buff);
					}
				}
				if (!entry_written)
					fprintf (fp_temp, "%s\n", m3u_file);
				fclose (m3u);
				fclose (fp_temp);
				remove (m3u_out_file);
				rename ("m3u.tmp", m3u_out_file);
			}
		}

		if (scaling) {
			clear_scaler(&enc_opts);
			scaling = 0;
		}

		if (downmixing) {
			clear_downmix(&enc_opts);
			downmixing = 0;
		}

		if (resampling) {
			clear_resample(&enc_opts);
			resampling = 0;
		}

		if (GGP_field) free (GGP_field);
		if (GP_field) free (GP_field);
		if (P_field) free (P_field);
		if (FN_field) free (FN_field);

/*		for (i = 0; i < 6; i++) {
			FN_field[i] = NULL;
			P_field[i] = NULL;
			GP_field[i] = NULL;
			GGP_field[i] = NULL;
		}

		if (out_fn)
			free(out_fn);
		if (enc_opts.filename)
			free(enc_opts.filename);
*/
		vorbis_comment_clear(&vc);

		format->close_func(enc_opts.readdata);
		fclose(in);
		fclose(out);

		if ((opt.delete_input && !stop_encoding) || lossless_file) {
			if (remove(in_file_temp) != 0)
				error_handler("Error deleting input file '%s'\n", in_file_temp);
		}

		if (opt.delete_input && !stop_encoding && lossless_file) {
			if (remove(szFile) != 0)
				error_handler("Error deleting input file '%s'\n", szFile);
		}

		numfiles--;
	}
	/*
	 * Finished this file, loop around to next...
	 */
}

void decoding_loop(void)
{
	char            *in_file;
	float           nominalBitrate;

	while (in_file = _getfile()) {
		char        *out_fn = NULL,
		            *p,
		            *q,
		            in_path[MAX_PATH],
		            *new_path,
		            *title,
		            *ext,
		            strFilename[MAX_PATH];
		int         j = 0,
		            len,
		            pcmbitwidth;

		if (stop_decoding) {
			while (in_file = _getfile())
				continue;
			break;
		}
		animate = 1;

		set_filename(in_file);

		strcpy(in_path, in_file);
		title = in_path + strlen(in_path);
		while (title[0] != '\\' && title[0] != ':' && title > in_path)
			title--;
		if (title[0] == '\\') {
			title[0] = '\0';
			title++;
		}

		ext = strrchr(title, '.');
		ext[0] = '\0';

		new_path = malloc(strlen(in_path) * sizeof(char) + 1);
		strcpy(new_path, in_path);
		q = new_path + strlen(in_path);
		q[0] = '\\';
		q[1] = '\0';

		len = strlen(config_OGGoutdir);
		if (config_OGGoutdir[len-1] == '\\')
			config_OGGoutdir[len-1] = '\0';
		p=config_OGGoutdir;

		if (!config_OGGoutdir[0])
			sprintf(strFilename,"%s%s.wav", new_path, title);
		else if (p[0]=='\\')
			sprintf(strFilename,"%s%s.wav",config_OGGoutdir,title);
		else
			sprintf(strFilename,"%s\\%s.wav",config_OGGoutdir,title);

		out_fn = strFilename;

		sprintf(fileName,"%s.wav", title);

		switch(dec_opts.format) {
			case WAV_FMT_8BIT:
				pcmbitwidth = 8;
				dec_opts.wrap_prev_pos = 0x7f;
				dec_opts.wrap_prev_neg = -0x80;
				break;
			case WAV_FMT_16BIT:
				pcmbitwidth = 16;
				dec_opts.wrap_prev_pos = 0x7fff;
				dec_opts.wrap_prev_neg = -0x8000;
				break;
			case WAV_FMT_24BIT:
				pcmbitwidth = 24;
				dec_opts.wrap_prev_pos = 0x7fffff;
				dec_opts.wrap_prev_neg = -0x800000;
				break;
			case WAV_FMT_32BIT:
				pcmbitwidth = 32;
				dec_opts.wrap_prev_pos = 0x7fffffff;
				dec_opts.wrap_prev_neg = -0x80000000;
				break;
			case WAV_FMT_FLOAT:
				pcmbitwidth = 32;
				dec_opts.wrap_prev_pos = 1 - (1 / 0x80000000);
				dec_opts.wrap_prev_neg = -1.;
				break;
		}

		dec_opts.progress_update = _update;
		dec_opts.enc_speed = _enc_speed;
		dec_opts.input_ogg = in_file;
		dec_opts.out_wave = out_fn;

		Init_Dither (pcmbitwidth, (!dec_opts.dither) ? dec_opts.dither : dec_opts.dither - 1);

		oe_decode(&dec_opts);

		if (new_path)
			free(new_path);
		if (out_fn)
			free(out_fn);
		numfiles--;
	}
	/* 
	 *Finished this file, loop around to next...
	 */

	if (opt.oeMode == OE_MODE_QUALITY) {
		nominalBitrate = qc2approxBitrate(opt.quality_coefficient*1000, 44100, 2);
		strcpy(approxBRCaption, "Nominal Bitrate: ");
		sprintf(nomBRCaption, "%.1fkbps", nominalBitrate/1000);
		sprintf(kCaption, "-q %.2f", (float)opt.quality_coefficient*10);
		strcpy(speedCaption, "");
	}
	else if (opt.oeMode == OE_MODE_VBR) {
		strcpy(approxBRCaption, "Approximate Bitrate: ");
		sprintf(nomBRCaption, "%.1fkbps", (float)opt.vbr_kbps);
		strcpy(kCaption, "");
		strcpy(speedCaption, "");
	}
	else {
		strcpy(approxBRCaption, "Bitrate: ");
		sprintf(nomBRCaption, "%.1fkbps", (float)opt.nom_kbps);
		strcpy(kCaption, "");
		strcpy(speedCaption, "");
	}
}

long encode_thread(LPVOID arg)
//unsigned int WINAPI encode_thread(LPVOID arg)
//void encode_thread(LPVOID arg)
{
	setlocale(LC_ALL, "");
	bindtextdomain(PACKAGE, LOCALEDIR);
	textdomain(PACKAGE);

	while (!encoding_done) {

		SetPriorityClass(GetCurrentProcess(), NORMAL_PRIORITY_CLASS);

		switch(opt.priority) {
			case 1:
				SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_LOWEST);
				break;
			case 2:
				SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_BELOW_NORMAL);
				break;
			case 3:
				SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_NORMAL);
				break;
			case 4:
				SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_ABOVE_NORMAL);
				break;
			case 5:
				SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_HIGHEST);
				break;
			default:
				SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_NORMAL);
				break;
		}

		if (encoding)
			encoding_loop();
		else if (decoding)
			decoding_loop();
		encoding = decoding = 0;
		file_complete = 0.0;
		animate = 0;
		totalfiles = 0;
		numfiles = 0;

		Sleep(500);
	} 

	DeleteCriticalSection(&mutex);
	return (0);
}

/*
 * end of encthread.c
 */
