/* OggDropXPd - Filename INI_SETUP.C
 *
 * Function: Controls encoding/decoding processing.
 *
 *   Copyright (c) 2002 - 2005 John Edwards <john.edwards33@ntlworld.com>
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <windows.h>
#include <stdio.h>

#include "ini_setup.h"
#include "encode.h"

extern SettingsOGG  iniSettings;
extern char         CmdDir[];

/*
 * Write the .ini file using the current oggdropXPd settings
 */

int WriteIniFile ( const char* Filename )
{
	FILE* fp;

	if ( (fp = fopen (Filename, "w")) == NULL )
		return EOF;                             // could not open file

	fprintf (fp, "[OggDropXPd]\n");
	fprintf (fp, "Window_X=%i\n"          , iniSettings.window_x     );
	fprintf (fp, "Window_Y=%i\n"          , iniSettings.window_y     );
	fprintf (fp, "Always_on_top=%i\n"     , iniSettings.always_on_top);
	fprintf (fp, "Logerr=%i\n"            , iniSettings.logerr       );
	fprintf (fp, "Shownbr=%i\n"           , iniSettings.shownbr      );
	fprintf (fp, "Mode=%i\n"              , iniSettings.mode         );
	fprintf (fp, "Tagging=%i\n"           , iniSettings.tagging      );
	fprintf (fp, "Quality=%i\n"           , iniSettings.quality      );
	fprintf (fp, "ApproxBitrate=%i\n"     , iniSettings.approxbitrate);
	fprintf (fp, "MinBitrate=%i\n"        , iniSettings.minbitrate   );
	fprintf (fp, "NomBitrate=%i\n"        , iniSettings.nombitrate   );
	fprintf (fp, "MaxBitrate=%i\n"        , iniSettings.maxbitrate   );
	fprintf (fp, "CBRBitrate=%i\n"        , iniSettings.cbrbitrate   );
	fprintf (fp, "DelInputWav=%i\n"       , iniSettings.del_input_wav);
	fprintf (fp, "WithSkeleton=%i\n"      , iniSettings.with_skeleton);
	fprintf (fp, "DelInputOgg=%i\n"       , iniSettings.del_input_ogg);
	fprintf (fp, "Title=%s\n"             , iniSettings.title        );
	fprintf (fp, "Artist=%s\n"            , iniSettings.artist       );
	fprintf (fp, "Album=%s\n"             , iniSettings.album        );
	fprintf (fp, "EncodedBy=%s\n"         , iniSettings.enc_by       );
	fprintf (fp, "Separator=%s\n"         , iniSettings.separator    );
	fprintf (fp, "TrackNo=%s\n"           , iniSettings.track_no     );
	fprintf (fp, "Year=%s\n"              , iniSettings.year         );
	fprintf (fp, "Genre=%s\n"             , iniSettings.genre        );
	fprintf (fp, "Volume=%i\n"            , iniSettings.volume       );
	fprintf (fp, "Limiter=%i\n"           , iniSettings.limiter      );
	fprintf (fp, "Dither=%i\n"            , iniSettings.dither       );
	fprintf (fp, "DecTagMode=%i\n"        , iniSettings.dec_tag_mode );
	fprintf (fp, "DecodeMode=%i\n"        , iniSettings.decode_mode  );
	fprintf (fp, "WaveFormat=%i\n"        , iniSettings.wav_format   );
	fprintf (fp, "ReSampleRate=%i\n"      , iniSettings.resample_rate);
	fprintf (fp, "ReSampleMode=%i\n"      , iniSettings.resample_mode);
	fprintf (fp, "Converter=%i\n"         , iniSettings.converter    );
	fprintf (fp, "ScaleFactor=%f\n"       , iniSettings.scale_factor );
	fprintf (fp, "LowPassMode=%i\n"       , iniSettings.lowpass_mode );
	fprintf (fp, "LowPass=%i\n"           , iniSettings.lowpass      );
	fprintf (fp, "RenameOnTag=%i\n"       , iniSettings.rename_on_tag);
	fprintf (fp, "RenameFormat=%s\n"      , iniSettings.rename_format);
	fprintf (fp, "OutputDirMode=%i\n"     , iniSettings.out_dir_mode );
	fprintf (fp, "OutputDir=%s\n"         , iniSettings.out_dir      );
	fprintf (fp, "TempDir=%s\n"           , iniSettings.temp_dir     );
	fprintf (fp, "PriorityClass=%i\n"     , iniSettings.priority     );
	fprintf (fp, "PlaylistOnTag=%i\n"     , iniSettings.playlist_on_tag);
	fprintf (fp, "PlaylistFormat=%s\n"    , iniSettings.playlist_format);
	fprintf (fp, "CopyFLACTag=%i\n"       , iniSettings.copy_flac_tag);
	fprintf (fp, "UseFLACTag=%i\n"        , iniSettings.use_flac_tag );
	fprintf (fp, "FLACScaleMode=%i\n"     , iniSettings.FLAC_scale   );
	fprintf (fp, "RenameFormatFLAC=%s\n"  , iniSettings.rename_format_flac);
	fprintf (fp, "DelReplayGainTags=%i\n" , iniSettings.delete_RG    );
	fprintf (fp, "HardQLimits=%i\n"       , iniSettings.hard_q_limits);
	fprintf (fp, "HardMinBitrate=%i\n"    , iniSettings.hard_min     );
	fprintf (fp, "HardMaxBitrate=%i\n"    , iniSettings.hard_max     );
	fprintf (fp, "UseDamp=%i\n"           , iniSettings.use_damp     );
	fprintf (fp, "BitAveDamp=%lf\n"       , iniSettings.bit_ave_damp );
	fprintf (fp, "UseBias=%i\n"           , iniSettings.use_bias     );
	fprintf (fp, "BitResBias=%lf\n"       , iniSettings.bit_res_bias );
	fprintf (fp, "UseBits=%i\n"           , iniSettings.use_bits     );
	fprintf (fp, "BitResBits=%i\n"        , iniSettings.bit_res_bits );
	fprintf (fp, "UseNoise=%i\n"          , iniSettings.use_noise    );
	fprintf (fp, "ImpNoise=%lf\n"         , iniSettings.imp_noise    );
//	fprintf (fp, "UseTrigger=%i\n"        , iniSettings.use_trigger  );
//	fprintf (fp, "ImpTrigger=%lf\n"       , iniSettings.imp_trigger  );
	fprintf (fp, "MC_Downmix=%i\n"        , iniSettings.mc_downmix   );
	return fclose (fp);
}

/*
 * Read the .ini file and set the oggdropXPd settings
 */

int ReadIniFile ( FILE* fp )
{
	char    buff [256];
	int     val;
	float   val_f;
	double  val_d;

	rewind ( fp );
	fgets ( buff, sizeof(buff), fp );

	if ( 0 != memcmp ( buff, "[OggDropXPd]", 12 ) )
		return EOF;

	while ( fgets ( buff, sizeof(buff), fp ) != NULL ) {
		if      ( 1 == sscanf ( buff, "Window_X=%d"     , &val   ) )  iniSettings.window_x      = val;
		else if ( 1 == sscanf ( buff, "Window_Y=%d"     , &val   ) )  iniSettings.window_y      = val;
		else if ( 1 == sscanf ( buff, "Always_on_top=%d", &val   ) )  iniSettings.always_on_top = val;
		else if ( 1 == sscanf ( buff, "Logerr=%d"       , &val   ) )  iniSettings.logerr        = val;
		else if ( 1 == sscanf ( buff, "Shownbr=%d"      , &val   ) )  iniSettings.shownbr       = val;
		else if ( 1 == sscanf ( buff, "Mode=%d"         , &val   ) )  iniSettings.mode          = val;
		else if ( 1 == sscanf ( buff, "Tagging=%d"      , &val   ) )  iniSettings.tagging       = val;
		else if ( 1 == sscanf ( buff, "Quality=%d"      , &val   ) )  iniSettings.quality       = val;
		else if ( 1 == sscanf ( buff, "ApproxBitrate=%d", &val   ) )  iniSettings.approxbitrate = val;
		else if ( 1 == sscanf ( buff, "MinBitrate=%d"   , &val   ) )  iniSettings.minbitrate    = val;
		else if ( 1 == sscanf ( buff, "NomBitrate=%d"   , &val   ) )  iniSettings.nombitrate    = val;
		else if ( 1 == sscanf ( buff, "MaxBitrate=%d"   , &val   ) )  iniSettings.maxbitrate    = val;
		else if ( 1 == sscanf ( buff, "CBRBitrate=%d"   , &val   ) )  iniSettings.cbrbitrate    = val;
		else if ( 1 == sscanf ( buff, "DelInputWav=%d"  , &val   ) )  iniSettings.del_input_wav = val;
		else if ( 1 == sscanf ( buff, "WithSkeleton=%d" , &val   ) )  iniSettings.with_skeleton = val;
		else if ( 1 == sscanf ( buff, "DelInputOgg=%d"  , &val   ) )  iniSettings.del_input_ogg = val;
		else if ( 1 == sscanf ( buff, "Title=%64[^\r\n]"          , iniSettings.title ) );
		else if ( 1 == sscanf ( buff, "Artist=%64[^\r\n]"         , iniSettings.artist ) );
		else if ( 1 == sscanf ( buff, "Album=%64[^\r\n]"          , iniSettings.album ) );
		else if ( 1 == sscanf ( buff, "EncodedBy=%32[^\r\n]"      , iniSettings.enc_by ) );
		else if ( 1 == sscanf ( buff, "Separator=%32[^\r\n]"      , iniSettings.separator ) );
		else if ( 1 == sscanf ( buff, "TrackNo=%32[^\r\n]"        , iniSettings.track_no ) );
		else if ( 1 == sscanf ( buff, "Year=%32[^\r\n]"           , iniSettings.year ) );
		else if ( 1 == sscanf ( buff, "Genre=%64[^\r\n]"          , iniSettings.genre ) );
		else if ( 1 == sscanf ( buff, "Volume=%d"       , &val   ) )  iniSettings.volume        = val;
		else if ( 1 == sscanf ( buff, "Limiter=%d"      , &val   ) )  iniSettings.limiter       = val;
		else if ( 1 == sscanf ( buff, "Dither=%d"       , &val   ) )  iniSettings.dither        = val;
		else if ( 1 == sscanf ( buff, "DecTagMode=%d"   , &val   ) )  iniSettings.dec_tag_mode  = val;
		else if ( 1 == sscanf ( buff, "DecodeMode=%d"   , &val   ) )  iniSettings.decode_mode   = val;
		else if ( 1 == sscanf ( buff, "WaveFormat=%d"   , &val   ) )  iniSettings.wav_format    = val;
		else if ( 1 == sscanf ( buff, "ReSampleRate=%d" , &val   ) )  iniSettings.resample_rate = val;
		else if ( 1 == sscanf ( buff, "ReSampleMode=%d" , &val   ) )  iniSettings.resample_mode = val;
		else if ( 1 == sscanf ( buff, "Converter=%d"    , &val   ) )  iniSettings.converter     = val;
		else if ( 1 == sscanf ( buff, "ScaleFactor=%f"  , &val_f ) )  iniSettings.scale_factor  = val_f;
		else if ( 1 == sscanf ( buff, "LowPassMode=%d"  , &val   ) )  iniSettings.lowpass_mode  = val;
		else if ( 1 == sscanf ( buff, "LowPass=%d"      , &val   ) )  iniSettings.lowpass       = val;
		else if ( 1 == sscanf ( buff, "RenameOnTag=%d"  , &val   ) )  iniSettings.rename_on_tag = val;
		else if ( 1 == sscanf ( buff, "RenameFormat=%64[^\r\n]"   , iniSettings.rename_format ) );
		else if ( 1 == sscanf ( buff, "OutputDirMode=%d", &val   ) )  iniSettings.out_dir_mode  = val;
		else if ( 1 == sscanf ( buff, "OutputDir=%245[^\r\n]"     , iniSettings.out_dir ) );
		else if ( 1 == sscanf ( buff, "TempDir=%245[^\r\n]"       , iniSettings.temp_dir ) );
		else if ( 1 == sscanf ( buff, "PriorityClass=%d", &val   ) )  iniSettings.priority      = val;
		else if ( 1 == sscanf ( buff, "PlaylistOnTag=%d"  , &val ) )  iniSettings.playlist_on_tag = val;
		else if ( 1 == sscanf ( buff, "PlaylistFormat=%64[^\r\n]"   , iniSettings.playlist_format ) );
		else if ( 1 == sscanf ( buff, "CopyFLACTag=%d"  , &val   ) )  iniSettings.copy_flac_tag = val;
		else if ( 1 == sscanf ( buff, "UseFLACTag=%d"   , &val   ) )  iniSettings.use_flac_tag  = val;
		else if ( 1 == sscanf ( buff, "FLACScaleMode=%d", &val   ) )  iniSettings.FLAC_scale    = val;
		else if ( 1 == sscanf ( buff, "RenameFormatFLAC=%64[^\r\n]"   , iniSettings.rename_format_flac ) );
		else if ( 1 == sscanf ( buff, "DelReplayGainTags=%d", &val ) )  iniSettings.delete_RG   = val;
		else if ( 1 == sscanf ( buff, "HardQLimits=%d"  , &val   ) )  iniSettings.hard_q_limits = val;
		else if ( 1 == sscanf ( buff, "HardMinBitrate=%d",&val   ) )  iniSettings.hard_min      = val;
		else if ( 1 == sscanf ( buff, "HardMaxBitrate=%d",&val   ) )  iniSettings.hard_max      = val;
		else if ( 1 == sscanf ( buff, "UseDamp=%d"      , &val   ) )  iniSettings.use_damp      = val;
		else if ( 1 == sscanf ( buff, "BitAveDamp=%lf"  , &val_d ) )  iniSettings.bit_ave_damp  = val_d;
		else if ( 1 == sscanf ( buff, "UseBias=%d"      , &val   ) )  iniSettings.use_bias      = val;
		else if ( 1 == sscanf ( buff, "BitResBias=%lf"  , &val_d ) )  iniSettings.bit_res_bias  = val_d;
		else if ( 1 == sscanf ( buff, "UseBits=%d"      , &val   ) )  iniSettings.use_bits      = val;
		else if ( 1 == sscanf ( buff, "BitResBits=%d"   , &val   ) )  iniSettings.bit_res_bits  = val;
		else if ( 1 == sscanf ( buff, "UseNoise=%d"     , &val   ) )  iniSettings.use_noise     = val;
		else if ( 1 == sscanf ( buff, "ImpNoise=%lf"    , &val_d ) )  iniSettings.imp_noise     = val_d;
//		else if ( 1 == sscanf ( buff, "UseTrigger=%d"   , &val   ) )  iniSettings.use_trigger   = val;
//		else if ( 1 == sscanf ( buff, "ImpTrigger=%lf"  , &val_d ) )  iniSettings.imp_trigger   = val_d;
		else if ( 1 == sscanf ( buff, "MC_Downmix=%d"   , &val   ) )  iniSettings.mc_downmix    = val;
	}

	return 0;
}

/*
 * Get oggdropXPd settings at startup, writes .ini file, if not present
 */

void GetOggdropSettings ( void )
{
	FILE*  fp = NULL;
	char   PathAndName[MAX_PATH];

	strcpy(PathAndName, CmdDir);
	strcat(PathAndName, INI_FILE);

	// set default values
	iniSettings.window_x      =   64;        // default box position (x co-ord)
	iniSettings.window_y      =   64;        // default box position (y co-ord)
	iniSettings.always_on_top =    8;        // default = on
	iniSettings.logerr        =    0;        // default = off
	iniSettings.shownbr       =    8;        // default = on
	iniSettings.mode          = 1017;        // default = quality
	iniSettings.tagging       =    0;        // default = off
	iniSettings.quality       =  400;        // default quality = 0.40
	iniSettings.approxbitrate =  128;        // default = 128kbps
	iniSettings.minbitrate    =    0;        // default = 0kbps
	iniSettings.nombitrate    =  128;        // default = 128kbps
	iniSettings.maxbitrate    =    0;        // default = 0kbps
	iniSettings.cbrbitrate    =    0;        // default = 0kbps
	iniSettings.del_input_wav =    0;        // default = off
	iniSettings.with_skeleton =    0;        // default = without skeleton
	iniSettings.del_input_ogg =    0;        // default = off
	strcpy(iniSettings.title, "");           // default = blank
	strcpy(iniSettings.artist, "");          // default = blank
	strcpy(iniSettings.album, "");           // default = blank
	strcpy(iniSettings.enc_by, "");          // default = blank
	strcpy(iniSettings.separator, "");       // default = blank
	strcpy(iniSettings.track_no, "");        // default = blank
	strcpy(iniSettings.year, "");            // default = blank
	strcpy(iniSettings.genre, "");           // default = blank
	iniSettings.volume        =    0;        // default = 0dB
	iniSettings.limiter       =    0;        // default = off
	iniSettings.dither        =    0;        // default = don't dither output
	iniSettings.dec_tag_mode  =    0;        // default = don't use VorbisGain tags
	iniSettings.decode_mode   =    1;        // default = 1 (decode to file)
	iniSettings.wav_format    =    1;        // default = 1 (16 bit PCM)
	iniSettings.resample_rate =44100;        // default = 44100 (samples per second)
	iniSettings.resample_mode =    0;        // default = output rate = input rate
	iniSettings.converter     =    1;        // default = medium quality
	iniSettings.scale_factor  =100.00;       // default = 100%
	iniSettings.lowpass_mode  =    0;        // default = don't apply lowpass
	iniSettings.lowpass       =    0;        // default = lowpass value = 0
	iniSettings.rename_on_tag =    0;        // default = don't rename file
	strcpy(iniSettings.rename_format, "");   // default = blank
	iniSettings.out_dir_mode  =    0;        // default = same as input dir
	strcpy(iniSettings.out_dir, "");         // default = blank (as above)
	strcpy(iniSettings.temp_dir, "");        // default = blank (as above)
	iniSettings.priority      =    3;        // default = normal priority
	iniSettings.playlist_on_tag =  0;        // default = don't create playlist
	strcpy(iniSettings.playlist_format, ""); // default = blank
	iniSettings.copy_flac_tag =    0;        // default = don't copy FLAC tags
	iniSettings.use_flac_tag  =    0;        // default = don't use FLAC tags
	iniSettings.FLAC_scale    =    0;        // default = don't use FLAC scaling (1=Title, 2=Album)
	strcpy(iniSettings.rename_format_flac, "");   // default = blank
	iniSettings.delete_RG     =    0;        // default = don't delete ReplayGain Tags
	iniSettings.hard_q_limits =    0;        // No hard limits set
	iniSettings.hard_min      =    0;        // No hard limits set
	iniSettings.hard_max      =    0;        // No hard limits set
	iniSettings.use_damp      =    0;        //
	iniSettings.bit_ave_damp  =    0;        // nominal default NOT to use
	iniSettings.use_bias      =    0;        // 
	iniSettings.bit_res_bias  =    0;        // nominal default NOT to use
	iniSettings.use_bits      =    0;        //
	iniSettings.bit_res_bits  =    0;        // nominal default NOT to use
	iniSettings.use_noise     =    0;        //
	iniSettings.imp_noise     =    0;        // nominal default NOT to use
//	iniSettings.use_trigger   =    0;        //
//	iniSettings.imp_trigger   =    0;        // nominal default NOT to use
	iniSettings.mc_downmix    =    0;        // default = don't downmix multi-channel on decode

	// Read INI_FILE
	if ( (fp = fopen (PathAndName, "r")) == NULL ) {    // file does not exist: write it!
		WriteIniFile ( PathAndName );
	}
	else {                                              // file does exist: read it!
		ReadIniFile (fp);
		fclose (fp);
	}

	return;
}

/* end of ini_setup.c */
