/* OggDropXPd - Filename MAIN.C
 *
 * Function: Main control program for oggdropXPd.
 *
 *   Copyright (c) 2002 - 2005 John Edwards <john.edwards33@ntlworld.com>
 *   Jack Moffit <jack@icecast.org> [as the original 'oggdrop' author]
 *
 *   Michael Smith <msmith@xiph.org> [as the author of the original oggenc]
 *
 *   Kenneth Arnold <kcarnold@yahoo.com> and 
 *   Christopher Montgomery <xiphmont@xiph.org>
 *   [as authors of code Mike borrowed from for oggenc]
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <windows.h>
#include <shellapi.h>
#include <string.h>
#include <stdio.h>
#include <commctrl.h>
#include <process.h>
#include <math.h>
#include <shlobj.h>	/* for Output Directory options */

#include "resource.h"
#include "encthread.h"
#include "misc.h"
#include "encode.h"
#include "ini_setup.h"

#define LOSHORT(l)           ((SHORT)(l))
#define HISHORT(l)           ((SHORT)(((DWORD)(l) >> 16) & 0xFFFF))

#define GENRE_FILE "GenreData.txt"
#define LOG_FILE "oggdropXPd.log"

#define MAX_GENRE_SIZE 256

#define CREATEFONT(sz) \
	CreateFont((sz), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
		VARIABLE_PITCH | FF_SWISS, "")
   
char         fileName[MAX_PATH],
             CmdDir[MAX_PATH],
             GenreData[MAX_GENRE_SIZE][64],
             config_OGGoutdir[MAX_PATH] = "",
             config_TEMPdir[MAX_PATH] = "";
char         approxBRCaption[22];
char         nomBRCaption[20];
char         kCaption[20];
char         speedCaption[20];
int          width = 130,
             height = 130,
             frame = 0,
             encoding_done = 0,
             encoding = 0,
             decoding = 0,
             stop_decoding,
             stop_encoding,
             totalfiles,
             numfiles,
             showNBR,
             animate = 0;
double       file_complete;
RECT         bar1;
RECT         bar2;
RECT         vbrBR;
HANDLE       event = NULL;
HINSTANCE    hinst;
HBITMAP      hbm[12];
HMENU        menu;
HWND         g_hwnd;
HFONT        font2;

extern oe_options   opt;
extern oe_dec_opt   dec_opts;

extern SettingsOGG  iniSettings;

static const char *resampleCaption[] =
{
	"96000", "88200", "64000", "48000", "44100", "32000", "24000", "22050", "16000", "11025", "8000", 0
};

static const char *converter[] =
{
	"Best", "Medium", "Fast", 0
};

static const char *noiseShaping[] =
{
	"No Dither", "No Noise Shaping", "Light", "Medium", "Heavy", 0
};

static const char *wavFormat[] =
{
	"Unsigned 8 bit PCM", "16 bit PCM", "24 bit PCM", "32 bit PCM", "32 bit Float", 0
};

static const char *GenreCaption[] =
{
	"A Cappella", 		"Acid Jazz", 		"Acid Punk", 		"Acid",
	"Acoustic", 		"Alternative",  	"AlternRock",	 	"Ambient",
 	"Anime", 		"Avantgarde",		"Ballad",	 	"Bass",
 	"Beat", 		"Bebob", 		"Big Band",		"Black Metal",
 	"Blank",		"Bluegrass", 		"Blues", 		"Booty Bass", 		"BritPop",
	"Cabaret", 		"Celtic", 		"Chamber Music",	"Chanson",
 	"Chorus",		"Christian Gangsta Rap","Christian Rap",	"Christian Rock",
	"Classic Rock",		"Classical",		"Club",			"Club-House",
	"Comedy",		"Contemporary Christian","Country",		"Crossover",
	"Cult",			"Dance Hall",		"Dance",		"Darkwave",
	"Death Metal",		"Disco",		"Dream",		"Drum & Bass",
	"Drum Solo",		"Duet",			"Easy Listening",	"Electronic",
	"Ethnic",		"Eurodance",		"Euro-House",		"Euro-Techno",
	"Fast-Fusion",		"Folk",			"Folk/Rock",		"Folklore",
	"Freestyle",		"Funk",			"Fusion",		"Game",
	"Gangsta",		"Goa",			"Gospel",		"Gothic Rock",
	"Gothic",		"Grunge",		"Hard Rock",		"Hardcore",
	"Heavy Metal",		"Hip-Hop",		"House",		"Humour",
	"Indie",		"Industrial",		"Instrumental Pop",	"Instrumental Rock",
	"Instrumental",		"Jazz",			"Jazz+Funk",		"JPop",
	"Jungle",		"Latin",		"Lo-Fi",		"Meditative",
	"Merengue",		"Metal",		"Musical",		"National Folk",
	"Native American",	"Negerpunk",		"New Age",		"New Wave",
	"Noise",		"Oldies",		"Opera",		"Other",
	"Polka",		"Polsk Punk",		"Pop",			"Pop/Funk",
	"Pop-Folk",		"Porn Groove",		"Power Ballad",		"Pranks",
	"Primus",		"Progressive Rock",	"Psychedelic Rock",	"Psychedelic",
	"Punk Rock",		"Punk",			"R&B",			"Rap",
	"Rave",			"Reggae",		"Retro",		"Revival",
	"Rhythmic Soul",	"Rock & Roll",		"Rock",			"Salsa",
	"Samba",		"Satire",		"Showtunes",		"Ska",
	"Slow Jam",		"Slow Rock",		"Sonata",		"Soul",
	"Sound Clip",		"Soundtrack",		"Southern Rock",	"Space",
	"Speech",		"Swing",		"Symphonic Rock",	"Symphony",
	"Synthpop",		"Tango",		"Techno",		"Techno-Industrial",
	"Terror",		"Thrash Metal",		"Top 40",		"Trailer",
	"Trance",		"Tribal",		"Trip-Hop",		"Vocal",
	0
};

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);

BOOL CALLBACK EncodeProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK DecodeProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK AdvEncSetup(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK TagProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK TagHelpProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK FLACHelpProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK DirProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK TempProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK LogProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK AboutProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam);

/*
 * Next function for Output Directory options
 */

static int CALLBACK BrowseCallbackProc( HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData)
{
	if (uMsg == BFFM_INITIALIZED) {
		SetWindowText(hwnd,"Select Output Directory");
		SendMessage(hwnd,BFFM_SETSELECTION,(WPARAM)1,(LPARAM)config_OGGoutdir);
	}
	return 0;
}

/*
 * Next function for Temporary Directory options
 */

static int CALLBACK BrowseCallbackProcTemp( HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData)
{
	if (uMsg == BFFM_INITIALIZED) {
		SetWindowText(hwnd,"Select Temporary Directory");
		SendMessage(hwnd,BFFM_SETSELECTION,(WPARAM)1,(LPARAM)config_TEMPdir);
	}
	return 0;
}

int ReadGenreFile ( FILE* fp );

/*
 * Write the GenreData file using the Default Genre Categories
 */

int WriteGenreFile ( const char* Filename )
{
	FILE* fp;
	int i;

	if ( (fp = fopen (Filename, "w")) == NULL )
		return EOF;                             // could not open file

	for (i = 0; GenreCaption[i]; i++)
		fprintf (fp, "%s\n", GenreCaption[i]);

	return fclose (fp);
}

/*
 * Read the GenreData file and load the Genre Categories
 */

int ReadGenreFile ( FILE* fp )
{
	char    buff [64];
	int i = 0;

	while ( fgets ( buff, sizeof(buff), fp ) != NULL ) {
		sscanf(buff, "%64[^\r\n]", GenreData[i]);
		i++;
		if (i > MAX_GENRE_SIZE - 1)
			break;
	}
	GenreData[i][0] = '\0';
	return 0;
}

/*
 * Get Genre Categories at startup, writes GenreData file, if not present
 */

void GetGenreData ( void )
{
	FILE*  fp = NULL;
	char   PathAndName[MAX_PATH];

	strcpy(PathAndName, CmdDir);
	strcat(PathAndName, GENRE_FILE);

	// Read GENRE_FILE
	if ( (fp = fopen (PathAndName, "r")) == NULL ) {    // file does not exist: write it!
		WriteGenreFile ( PathAndName );
		fp = fopen (PathAndName, "r");
		ReadGenreFile (fp);
		fclose (fp);
	}
	else {                                              // file does exist: read it!
		ReadGenreFile (fp);
		fclose (fp);
	}

	return;
}

void set_mode(int v)
{
	iniSettings.mode = v;
}

void set_quality_coefficient(int v)
{
	if (v < -100)
		v = -100;
	else if (v > 1000)
		v = 1000;

	opt.quality_coefficient = (v == 0) ? 0.0f : ((float)v)/1000.f;
	opt.oeMode = OE_MODE_QUALITY;
	opt.managed = 0;
	iniSettings.quality = v;
}

void set_APPROXbitrate(int v)
{
	opt.vbr_kbps = v;
	opt.oeMode = OE_MODE_VBR;
	opt.managed = 0;
	iniSettings.approxbitrate = v;
}

void set_MINbitrate(int v)
{
	opt.min_kbps = v;
	iniSettings.minbitrate = v;
}

void set_NOMbitrate(int v)
{
	opt.nom_kbps = v;
	opt.oeMode = OE_MODE_BITRATE;
	opt.managed = 1;
	iniSettings.nombitrate = v;
}

void set_MAXbitrate(int v)
{
	opt.max_kbps = v;
	iniSettings.maxbitrate = v;
}

void set_CBRbitrate(int v)
{
	opt.min_kbps = v;
	opt.nom_kbps = v;
	opt.max_kbps = v;
	opt.oeMode = OE_MODE_BITRATE;
	opt.managed = 1;
	iniSettings.cbrbitrate = v;
	iniSettings.minbitrate = v;
	iniSettings.nombitrate = v;
	iniSettings.maxbitrate = v;
}

void set_AUTOtagging(HWND hwnd, int v)
{
	CheckMenuItem(menu, IDM_TAGGING, v ? MF_CHECKED : MF_UNCHECKED);
	opt.AUTOtag = v;
	iniSettings.tagging = v;
}

void set_always_on_top(HWND hwnd, int v)
{
	CheckMenuItem(menu, IDM_ONTOP, v ? MF_CHECKED : MF_UNCHECKED);
	SetWindowPos(hwnd, v ? HWND_TOPMOST : HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOSIZE | SWP_SHOWWINDOW | SWP_NOMOVE);
	iniSettings.always_on_top = v;
}

void set_logerr(HWND hwnd, int v)
{
	CheckMenuItem(menu, IDM_LOGERR, v ? MF_CHECKED : MF_UNCHECKED);
	iniSettings.logerr = v;
	set_use_dialogs(v);
	opt.logerr = v;
}

void set_showNBR(HWND hwnd, int v)
{
	CheckMenuItem(menu, IDM_SHOWNBR, v ? MF_CHECKED : MF_UNCHECKED);
	iniSettings.shownbr = v;
	showNBR = v;
}

void set_priority(HWND hwnd, int v)
{
	if (v == 1) {
		CheckMenuItem(menu, IDM_IDLE_PRIORITY, MF_CHECKED);
		CheckMenuItem(menu, IDM_BELOW_NORMAL_PRIORITY, MF_UNCHECKED);
		CheckMenuItem(menu, IDM_NORM_PRIORITY, MF_UNCHECKED);
		CheckMenuItem(menu, IDM_HIGH_PRIORITY, MF_UNCHECKED);
		CheckMenuItem(menu, IDM_REALTIME_PRIORITY, MF_UNCHECKED);
	}
	else if (v == 2) {
		CheckMenuItem(menu, IDM_IDLE_PRIORITY, MF_UNCHECKED);
		CheckMenuItem(menu, IDM_BELOW_NORMAL_PRIORITY, MF_CHECKED);
		CheckMenuItem(menu, IDM_NORM_PRIORITY, MF_UNCHECKED);
		CheckMenuItem(menu, IDM_HIGH_PRIORITY, MF_UNCHECKED);
		CheckMenuItem(menu, IDM_REALTIME_PRIORITY, MF_UNCHECKED);
	}
	else if (v == 3) {
		CheckMenuItem(menu, IDM_IDLE_PRIORITY, MF_UNCHECKED);
		CheckMenuItem(menu, IDM_BELOW_NORMAL_PRIORITY, MF_UNCHECKED);
		CheckMenuItem(menu, IDM_NORM_PRIORITY, MF_CHECKED);
		CheckMenuItem(menu, IDM_HIGH_PRIORITY, MF_UNCHECKED);
		CheckMenuItem(menu, IDM_REALTIME_PRIORITY, MF_UNCHECKED);
	}
	else if (v == 4) {
		CheckMenuItem(menu, IDM_IDLE_PRIORITY, MF_UNCHECKED);
		CheckMenuItem(menu, IDM_BELOW_NORMAL_PRIORITY, MF_UNCHECKED);
		CheckMenuItem(menu, IDM_NORM_PRIORITY, MF_UNCHECKED);
		CheckMenuItem(menu, IDM_HIGH_PRIORITY, MF_CHECKED);
		CheckMenuItem(menu, IDM_REALTIME_PRIORITY, MF_UNCHECKED);
	}
	else if (v == 5) {
		CheckMenuItem(menu, IDM_IDLE_PRIORITY, MF_UNCHECKED);
		CheckMenuItem(menu, IDM_BELOW_NORMAL_PRIORITY, MF_UNCHECKED);
		CheckMenuItem(menu, IDM_NORM_PRIORITY, MF_UNCHECKED);
		CheckMenuItem(menu, IDM_HIGH_PRIORITY, MF_UNCHECKED);
		CheckMenuItem(menu, IDM_REALTIME_PRIORITY, MF_CHECKED);
	}
	iniSettings.priority = v;
	opt.priority = v;
}

void set_del_input_wav(int v)
{
	opt.delete_input = v;
	iniSettings.del_input_wav = v;
}

void set_with_skeleton(int v)
{
	opt.with_skeleton = v;
	iniSettings.with_skeleton = v;
}

void set_del_input_ogg(int v)
{
	dec_opts.del_input = v;
	iniSettings.del_input_ogg = v;
}

void set_title(char *v)
{
	opt.Title = strdup(v);
	strcpy(iniSettings.title, v);
}

void set_artist(char *v)
{
	opt.Artist = strdup(v);
	strcpy(iniSettings.artist, v);
}

void set_album(char *v)
{
	opt.Album = strdup(v);
	strcpy(iniSettings.album, v);
}

void set_separator(char *v)
{
	opt.Separator = strdup(v);
	strcpy(iniSettings.separator, v);
}

void set_year(char *v)
{
	opt.Year = strdup(v);
	strcpy(iniSettings.year, v);
}

void set_genre(char *v)
{
	opt.Genre = strdup(v);
	strcpy(iniSettings.genre, v);
}

void set_enc_by(char *v)
{
	opt.Enc_by = strdup(v);
	strcpy(iniSettings.enc_by, v);
}

void set_track_no(char *v)
{
	opt.TrackNo = strdup(v);
	strcpy(iniSettings.track_no, v);
}

void set_downmix(int v)
{
	opt.downmix = v;
}

void set_hard_q_limits(int v)
{
	opt.hard_q_limits = v;
	iniSettings.hard_q_limits = v;
	opt.managed = v;
}

void set_hard_min(int v)
{
	opt.hard_min = v;
	iniSettings.hard_min = v;
}

void set_hard_max(int v)
{
	opt.hard_max = v;
	iniSettings.hard_max = v;
}

void set_use_damp(int v)
{
	opt.use_damp = v;
	iniSettings.use_damp = v;
}

void set_use_bias(int v)
{
	opt.use_bias = v;
	iniSettings.use_bias = v;
}

void set_use_bits(int v)
{
	opt.use_bits = v;
	iniSettings.use_bits = v;
}

void set_use_noise(int v)
{
	opt.use_noise = v;
	iniSettings.use_noise = v;
}

//void set_use_trigger(int v)
//{
//	opt.use_trigger = v;
//	iniSettings.use_trigger = v;
//}

void set_bit_ave_damp(double v)
{
	opt.bitrate_average_damping = v;
	iniSettings.bit_ave_damp = v;
}

void set_bit_res_bias(double v)
{
	opt.bitrate_limit_reservoir_bias = v;
	iniSettings.bit_res_bias = v;
}

void set_bit_res_bits(long v)
{
	opt.bitrate_limit_reservoir_bits = v;
	iniSettings.bit_res_bits = v;
}

void set_imp_noise(double v)
{
	opt.impulse_noisetune = v;
	iniSettings.imp_noise = v;
}

//void set_imp_trigger(double v)
//{
//	opt.impulse_trigger_profile = v;
//	iniSettings.imp_trigger = v;
//}

void set_resample_rate(int v)
{
	opt.resamplefreq = v;
	if (v != 0) {
		iniSettings.resample_rate = v;
	} else {
		iniSettings.resample_rate = 44100;
	}
}

void set_resample_mode(int v)
{
	iniSettings.resample_mode = v;
}

void set_converter(int v)
{
	opt.converter = v;
	iniSettings.converter = v;
}

void set_scale_factor(double v)
{
	opt.scale = (float)v/100;
	iniSettings.scale_factor = (float)v;
}

void set_FLAC_tag_scale(int v)
{
	opt.FLAC_scale = v;
	iniSettings.FLAC_scale = v;
}

void set_del_RG_tag(int v)
{
	opt.del_RG = v;
	iniSettings.delete_RG = v;
}

void set_lowpass_mode(int v)
{
	iniSettings.lowpass_mode = v;
}

void set_lowpass(int v)
{
	opt.lowpass = (v != 0 ? (double)v/1000 : 0.0);
	iniSettings.lowpass = v;
}

void set_rename_on_tag(int v)
{
	opt.RenameOnTag = v;
	iniSettings.rename_on_tag = v;
}

void set_rename_format(char *v)
{
	opt.RenameFormat = strdup(v);
	strcpy(iniSettings.rename_format, v);
}

void set_rename_format_flac(char *v)
{
	opt.RenameFormatFLAC = strdup(v);
	strcpy(iniSettings.rename_format_flac, v);
}

void set_playlist_on_tag(int v)
{
	opt.PlaylistOnTag = v;
	iniSettings.playlist_on_tag = v;
}

void set_playlist_format(char *v)
{
	opt.PlaylistFormat = strdup(v);
	strcpy(iniSettings.playlist_format, v);
}

void set_volume(int v)
{
	dec_opts.man_gain = (int)((float)v/10);
	iniSettings.volume = v;
}

void set_limiter(int v)
{
	dec_opts.limiter = v;
	iniSettings.limiter = v;
}

void set_dither(int v)
{
	dec_opts.dither = v;
	iniSettings.dither = v;
}

void set_dec_tag_mode(int v)
{
	dec_opts.tagMode = v;
	iniSettings.dec_tag_mode = v;
}

void set_decode_mode(int v)
{
	dec_opts.decode_mode = v;
	iniSettings.decode_mode = v;
}

void set_wav_format(int v)
{
	dec_opts.format = v + 1;
	iniSettings.wav_format = v;
}

void set_mc_downmix(int v)
{
	dec_opts.downmix = v;
	iniSettings.mc_downmix = v;
}

void set_out_dir_mode(int v)
{
	iniSettings.out_dir_mode = v;
}

void set_out_dir(char *v)
{
	strcpy(iniSettings.out_dir, v);
}

void set_temp_dir(char *v)
{
	strcpy(iniSettings.temp_dir, v);
}

void set_copy_flac_tag(int v)
{
	opt.copy_comments = v;
	iniSettings.copy_flac_tag = v;
}

void set_use_flac_tag(int v)
{
	opt.use_flac_tag = v;
	iniSettings.use_flac_tag = v;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	static char    szAppName[] = "oggdropWin";
	char           *p;
	HWND           hwnd;
	MSG            msg;
	WNDCLASSEX     wndclass;
	const int      width = 130,
	               height = 130;
	int            x,
	               y;

	hinst = hInstance;

	wndclass.cbSize        = sizeof(WNDCLASSEX);
	wndclass.style         = CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc   = WndProc;
	wndclass.cbClsExtra    = 0;
	wndclass.cbWndExtra    = 0;
	wndclass.hInstance     = hInstance;
	wndclass.hIcon         = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON1));
	wndclass.hCursor       = LoadCursor(NULL, IDC_ARROW);
	wndclass.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
	wndclass.lpszMenuName  = NULL;
	wndclass.lpszClassName = szAppName;
	wndclass.hIconSm       = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON2));

	RegisterClassEx(&wndclass);

	GetModuleFileName(NULL, CmdDir, MAX_PATH);
	p = strrchr(CmdDir, '\\') + 1;
	p[0] = '\0';

	GetOggdropSettings();

	x = max(min(iniSettings.window_x, GetSystemMetrics(SM_CXSCREEN) - width), 0);
	y = max(min(iniSettings.window_y, GetSystemMetrics(SM_CYSCREEN) - height), 0);

	hwnd = CreateWindow(szAppName, "oggdropXPd", WS_POPUP | WS_DLGFRAME, x, y,
		width, height, NULL, NULL, hinst, NULL);

	g_hwnd = hwnd;

	ShowWindow(hwnd, nCmdShow);
	UpdateWindow(hwnd);

	font2 = CREATEFONT(10);

	SetTimer(hwnd, 1, 80, NULL);

	/*
	 * Apply all the .ini settings
	 */

	{
		double nominalBitrate;
		int qcValue;

		if (iniSettings.mode == IDC_USEQUALITY) {
			set_quality_coefficient(iniSettings.quality);
			qcValue = iniSettings.quality;
			nominalBitrate = qc2approxBitrate( (float)qcValue, iniSettings.resample_rate, 2 );
			strcpy(approxBRCaption, "Nominal Bitrate: ");
			sprintf(nomBRCaption, "%.1fkbps", (float)nominalBitrate/1000);
			sprintf(kCaption, "-q %.2f", (float)qcValue/100);
		}
		else if (iniSettings.mode == IDC_USEVBR) {
			set_APPROXbitrate(iniSettings.approxbitrate);
			nominalBitrate = (double)iniSettings.approxbitrate;
			strcpy(approxBRCaption, "Approximate Bitrate: ");
			sprintf(nomBRCaption, "%.1fkbps", (float)nominalBitrate);
			strcpy(kCaption, "");
		}
		else {
			if (iniSettings.mode == IDC_USEABRBITRATE) {
				set_NOMbitrate(iniSettings.nombitrate);
				nominalBitrate = (double)iniSettings.nombitrate;
				set_MINbitrate(iniSettings.minbitrate);
				set_MAXbitrate(iniSettings.maxbitrate);
			}
			else {
				set_CBRbitrate(iniSettings.cbrbitrate);
				nominalBitrate = (double)iniSettings.cbrbitrate;
			}
			strcpy(approxBRCaption, "Bitrate: ");
			sprintf(nomBRCaption, "%.1fkbps", (float)nominalBitrate);
			strcpy(kCaption, "");
		}
		strcpy(speedCaption, "");
	}

	set_always_on_top(hwnd, iniSettings.always_on_top);
	set_logerr(hwnd, iniSettings.logerr);
	set_showNBR(hwnd, iniSettings.shownbr);
	set_del_input_wav(iniSettings.del_input_wav);
	set_with_skeleton(iniSettings.with_skeleton);
	set_del_input_ogg(iniSettings.del_input_ogg);
	set_mode(iniSettings.mode);
	set_title(iniSettings.title);
	set_artist(iniSettings.artist);
	set_album(iniSettings.album);
	set_separator(iniSettings.separator);
	set_year(iniSettings.year);
	set_genre(iniSettings.genre);
	set_track_no(iniSettings.track_no);
	set_enc_by(iniSettings.enc_by);
	set_AUTOtagging(hwnd, iniSettings.tagging);
	set_priority(hwnd, iniSettings.priority);
	set_volume(iniSettings.volume);
	set_limiter(iniSettings.limiter);
	set_dither(iniSettings.dither);
	set_dec_tag_mode(iniSettings.dec_tag_mode);
	set_decode_mode(iniSettings.decode_mode);
	set_wav_format(iniSettings.wav_format);
	set_resample_mode(iniSettings.resample_mode);
	set_lowpass_mode(iniSettings.lowpass_mode);
	set_lowpass(iniSettings.lowpass);
	set_downmix(0);
	set_mc_downmix(iniSettings.mc_downmix);
	set_rename_on_tag(iniSettings.rename_on_tag);
	set_rename_format(iniSettings.rename_format);
	set_playlist_on_tag(iniSettings.playlist_on_tag);
	set_playlist_format(iniSettings.playlist_format);
	set_scale_factor((double)iniSettings.scale_factor);
	set_resample_rate(!iniSettings.resample_mode ? 0 : iniSettings.resample_rate);
	set_converter(iniSettings.converter);
	set_copy_flac_tag(iniSettings.copy_flac_tag);
	set_use_flac_tag(iniSettings.use_flac_tag);
	set_FLAC_tag_scale(iniSettings.FLAC_scale);
	set_del_RG_tag(iniSettings.delete_RG);
	set_rename_format_flac(iniSettings.rename_format_flac);
	set_out_dir_mode(iniSettings.out_dir_mode);
	if (iniSettings.out_dir_mode) {
		set_out_dir(iniSettings.out_dir);
		strcpy(config_OGGoutdir, iniSettings.out_dir);
	}
	else {
		set_out_dir("");
		strcpy(config_OGGoutdir, "");
	}
	if (iniSettings.temp_dir[0]) {
		set_temp_dir(iniSettings.temp_dir);
		strcpy(config_TEMPdir, iniSettings.temp_dir);
	}
	else {
		set_temp_dir("");
		strcpy(config_TEMPdir, "");
	}
	set_hard_q_limits(iniSettings.hard_q_limits);
	set_hard_min(iniSettings.hard_min);
	set_hard_max(iniSettings.hard_max);
	if (iniSettings.use_damp == 1) {
		set_use_damp(1);
		set_bit_ave_damp(iniSettings.bit_ave_damp);
	}
	if (iniSettings.use_bias == 1) {
		set_use_bias(1);
		set_bit_res_bias(iniSettings.bit_res_bias);
	}
	if (iniSettings.use_bits == 1) {
		set_use_bits(1);
		set_bit_res_bits(iniSettings.bit_res_bits);
	}
	if (iniSettings.use_noise == 1) {
		set_use_noise(1);
		set_imp_noise(iniSettings.imp_noise);
	}
//	if (iniSettings.use_trigger == 1) {
//		set_use_trigger(1);
//		set_imp_trigger(iniSettings.imp_trigger);
//	}

	for (frame = 0; frame < 12; frame++)
		hbm[frame] = LoadImage(hinst, MAKEINTRESOURCE(IDB_TF01 + frame), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION);
	frame = 0;

	while (GetMessage(&msg, NULL, 0, 0)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	for (frame = 0; frame < 12; frame++) 
		DeleteObject(hbm[frame]);

	return msg.wParam;
}

void HandleDrag(HWND hwnd, HDROP hDrop)
{
	int    cFiles,
	       i,
	       flag = 0;
	char   szFile[MAX_PATH],
	       *ext;

	cFiles = DragQueryFile(hDrop, 0xFFFFFFFF, NULL, 0);
	for (i = 0; i < cFiles; i++) {
		DragQueryFile(hDrop, i, szFile, sizeof(szFile));

		if (ext = strrchr(szFile, '.'))	{
			if ((local_strcasecmp(ext, ".wav") == 0) || (local_strcasecmp(ext, ".aif") == 0) ||
			    (local_strcasecmp(ext, ".flac") == 0) || (local_strcasecmp(ext, ".pac") == 0) ||
			    (local_strcasecmp(ext, ".ape") == 0) || (local_strcasecmp(ext, ".wv") == 0) ||
			    (local_strcasecmp(ext, ".ofr") == 0) || (local_strcasecmp(ext, ".ofs") == 0)) {
				flag = 1;
				encthread_addfile(szFile);
				encoding = 1;
				decoding = 0;
				stop_encoding = 0;
			}
			else if (local_strcasecmp(ext, ".ogg") == 0) {
				flag = 1;
				encthread_addfile(szFile);
				encoding = 0;
				decoding = 1;
				stop_decoding = 0;
			}
			else
				MessageBox(NULL, "Unsupported input file type", NULL, MB_OK | MB_ICONEXCLAMATION);
		}
	}

	DragFinish(hDrop);

	if (flag)
		SetEvent(event);
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	HDC          hdc,
	             hmem;
	static HDC   offscreen;
	PAINTSTRUCT  ps;
	RECT         rect;
	BITMAP       bm;
	POINT        point;
	static POINT start;
	static int   dragging = 0;
	HDC          desktop;
	HBITMAP      hbitmap;
	HANDLE       hdrop;
	HFONT        dfltFont;
	int          dfltBGMode;
	double       percomp;

	switch (message) {
		case WM_CREATE:

			menu = LoadMenu(hinst, MAKEINTRESOURCE(IDR_MENU1));
			menu = GetSubMenu(menu, 0);

			offscreen = CreateCompatibleDC(NULL);
			desktop = GetDC(GetDesktopWindow());
			hbitmap = CreateCompatibleBitmap(desktop, 200, 200);
			ReleaseDC(GetDesktopWindow(), desktop);
			SelectObject(offscreen, hbitmap);

			// Start the engines
			encthread_init();

			// We accept drag&drop
			DragAcceptFiles(hwnd, TRUE);
			return 0;

		case WM_PAINT:

			hdc = BeginPaint(hwnd, &ps);
			GetClientRect(hwnd, &rect);
			width = rect.right + 1;
			height = rect.bottom + 1;

			FillRect(offscreen, &rect, (HBRUSH)GetStockObject(WHITE_BRUSH));
			DrawText(offscreen, "Drop Files Here", -1, &rect, DT_SINGLELINE | DT_CENTER);

			hmem = CreateCompatibleDC(offscreen);
			SelectObject(hmem, hbm[frame]);
			GetObject(hbm[frame], sizeof(BITMAP), &bm);
			BitBlt(offscreen, width / 2 - 33, height / 2 - 46, bm.bmWidth, bm.bmHeight, hmem, 0, 0, SRCCOPY);
			DeleteDC(hmem);
		
			percomp = ((double)(totalfiles - numfiles) + 1 - (1 - file_complete)) / (double)totalfiles;

			SetRect(&vbrBR, 0, height - 45, width, height - 19);

			dfltBGMode = SetBkMode(offscreen, TRANSPARENT);
			dfltFont = SelectObject(offscreen, font2);

			if (showNBR) {
				char nbrCaption[80];

				sprintf(nbrCaption, "%s%s\r\n%s%s", approxBRCaption, nomBRCaption, kCaption, speedCaption);

				DrawText(offscreen, nbrCaption, -1, &vbrBR, DT_CENTER);

			}

			SetRect(&bar1, 0, height - 23, (int)(file_complete * width), height - 13);
			SetRect(&bar2, 0, height - 12, (int)(percomp * width), height - 2);

			FillRect(offscreen, &bar1, (HBRUSH)GetStockObject(LTGRAY_BRUSH));
			FillRect(offscreen, &bar2, (HBRUSH)GetStockObject(DKGRAY_BRUSH));

			if (fileName[0]) {
				char  fileCaption[80];

				strcpy(fileCaption, " ");
				strncat(fileCaption, fileName, 78);

				DrawText(offscreen, fileCaption, -1, &bar1, DT_SINGLELINE | DT_LEFT);
			}

			SelectObject(offscreen, dfltFont);
			SetBkMode(offscreen, dfltBGMode);

			BitBlt(hdc, 0, 0, width, height, offscreen, 0, 0, SRCCOPY);

			EndPaint(hwnd, &ps);

			DefWindowProc(hwnd, message, wParam, lParam);
			return 0;

		case WM_TIMER:

			if (animate || frame) {
				frame++;
				if (frame > 11) 
					frame = 0;
			}
			else {
				frame = 0;
			}
			GetClientRect(hwnd, &rect);
			InvalidateRect(hwnd, &rect, FALSE);
			return 0;

		case WM_LBUTTONDOWN:

			start.x = LOWORD(lParam);
			start.y = HIWORD(lParam);
			ClientToScreen(hwnd, &start);
			GetWindowRect(hwnd, &rect);
			start.x -= rect.left;
			start.y -= rect.top;
			dragging = 1;
			SetCapture(hwnd);
			return 0;

		case WM_LBUTTONUP:

			if (dragging) {
				dragging = 0;
				ReleaseCapture();
			}
			return 0;

		case WM_MOUSEMOVE:

			if (dragging) {
				point.x = LOSHORT(lParam);
				point.y = HISHORT(lParam);

				/* lParam can contain negative coordinates !
				point.x = LOWORD(lParam);
				point.y = HIWORD(lParam);
				*/

				ClientToScreen(hwnd, &point);
				SetWindowPos(hwnd, 0, point.x - start.x, point.y - start.y, 0, 0, SWP_NOSIZE | SWP_NOZORDER | SWP_SHOWWINDOW);
				iniSettings.window_x = point.x - start.x;
				iniSettings.window_y = point.y - start.y;
			}
			return 0;

		case WM_CAPTURECHANGED:

			if (dragging) {
				dragging = 0;
				ReleaseCapture();
			}
			return 0;

		case WM_RBUTTONUP:

			point.x = LOWORD(lParam);
			point.y = HIWORD(lParam);
			ClientToScreen(hwnd, &point);
			TrackPopupMenu(menu, TPM_RIGHTBUTTON, point.x, point.y, 0, hwnd, NULL);
			return 0;

		case WM_COMMAND:

			switch (LOWORD(wParam)) {
				case IDM_QUIT: {

					char PathAndName[MAX_PATH];
					strcpy(PathAndName, CmdDir);
					strcat(PathAndName, INI_FILE);
					WriteIniFile(PathAndName);
					encoding_done = 1;
					PostQuitMessage(0);
				}
					break;

				case IDM_TAGGING:

					set_AUTOtagging(hwnd, ~GetMenuState(menu, LOWORD(wParam), MF_BYCOMMAND) & MF_CHECKED);
					break;

				case IDM_ONTOP:

					set_always_on_top(hwnd, ~GetMenuState(menu, LOWORD(wParam), MF_BYCOMMAND) & MF_CHECKED);
					break;	

				case IDM_LOGERR:

					set_logerr(hwnd, ~GetMenuState(menu, LOWORD(wParam), MF_BYCOMMAND) & MF_CHECKED);
					break;

				case IDM_SHOWNBR:

					set_showNBR(hwnd, ~GetMenuState(menu, LOWORD(wParam), MF_BYCOMMAND) & MF_CHECKED);
					break;

				case IDM_IDLE_PRIORITY:{

					int v = ~GetMenuState(menu, LOWORD(wParam), MF_BYCOMMAND) & MF_CHECKED;
					if (v == 8)
						set_priority(hwnd, 1);
					break;
				}
				case IDM_BELOW_NORMAL_PRIORITY:{

					int v = ~GetMenuState(menu, LOWORD(wParam), MF_BYCOMMAND) & MF_CHECKED;
					if (v == 8)
						set_priority(hwnd, 2);
					break;
				}
				case IDM_NORM_PRIORITY:{

					int v = ~GetMenuState(menu, LOWORD(wParam), MF_BYCOMMAND) & MF_CHECKED;
					if (v == 8)
						set_priority(hwnd, 3);
					break;
				}
				case IDM_HIGH_PRIORITY:{

					int v = ~GetMenuState(menu, LOWORD(wParam), MF_BYCOMMAND) & MF_CHECKED;
					if (v == 8)
						set_priority(hwnd, 4);
					break;
				}
				case IDM_REALTIME_PRIORITY:{

					int v = ~GetMenuState(menu, LOWORD(wParam), MF_BYCOMMAND) & MF_CHECKED;
					if (v == 8)
						set_priority(hwnd, 5);
					break;
				}
				case IDM_STOP_ENC:{

					int v = ~GetMenuState(menu, LOWORD(wParam), MF_BYCOMMAND) & MF_CHECKED;
					if (v == 8)
						stop_encoding = 1;
					break;
				}
				case IDM_STOP_DEC: {

					int v = ~GetMenuState(menu, LOWORD(wParam), MF_BYCOMMAND) & MF_CHECKED;
					if (v == 8)
						stop_decoding = 1;
					break;
				}
				case IDM_SAVEQUALITY: {

					DialogBox(hinst, MAKEINTRESOURCE(IDD_QUALITY), hwnd, EncodeProc);

					break;
				}

				case IDM_VOLUME: {

					DialogBox(hinst, MAKEINTRESOURCE(IDD_VOLUME), hwnd, DecodeProc);

					break;
				}

				case IDM_SAVEDIR: {

					DialogBox(hinst, MAKEINTRESOURCE(IDD_DIRECTORY), hwnd, DirProc);

					break;
				}

				case IDM_SHOW_LOG: {

					DialogBox(hinst, MAKEINTRESOURCE(IDD_SHOW_LOG), hwnd, LogProc);

					break;
				}

				case IDM_TEMPDIR: {

					DialogBox(hinst, MAKEINTRESOURCE(IDD_TEMP_DIRECTORY), hwnd, TempProc);

					break;
				}

				case IDM_TAG_SETUP: {

					DialogBox(hinst, MAKEINTRESOURCE(IDD_TAGGING), hwnd, TagProc);

					break;
				}

				case IDM_ABOUT: {

					DialogBox(hinst, MAKEINTRESOURCE(IDD_ABOUT), hwnd, AboutProc);

					break;
				}
			} // LOWORD(wParam)
			return 0;

		case WM_DROPFILES:

			hdrop = (HANDLE)wParam;
			HandleDrag(hwnd, hdrop);
			return 0;
		

		case WM_DESTROY:

			encoding_done = 1;
			PostQuitMessage(0);
			return 0;

		default:
			return DefWindowProc(hwnd, message, wParam, lParam);
	}
	return 0;
}

/*
 *  Encode parameters dialog procedures.
 */

BOOL CALLBACK EncodeProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	char     buf2[16],
	         dB_buf[48];
	int      br;
	int      qcValue;
	int      setup_mode          = iniSettings.mode;
	int      setup_quality       = iniSettings.quality;
	int      temp_VBRbitrate     = iniSettings.approxbitrate;
	int      setup_VBRbitrate    = iniSettings.approxbitrate;
	int      setup_MINbitrate    = iniSettings.minbitrate;
	int      temp_NOMbitrate     = iniSettings.nombitrate;
	int      setup_NOMbitrate    = iniSettings.nombitrate;
	int      setup_MAXbitrate    = iniSettings.maxbitrate;
	int      setup_CBRbitrate    = iniSettings.cbrbitrate;
	int      setup_hard_q_limits = iniSettings.hard_q_limits;
	int      setup_hard_min      = iniSettings.hard_min;
	int      setup_hard_max      = iniSettings.hard_max;

	switch (message) {
		case WM_INITDIALOG: {

			double nominalBitrate;
			double dB;

			HICON hBgIcon = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON1));
			PostMessage(hwndDlg, WM_SETICON, ICON_BIG, (LPARAM)hBgIcon);

			SendDlgItemMessage(hwndDlg, IDC_SCROLL_QUALITY, SBM_SETRANGEREDRAW, 
        			(WPARAM) 0, (LPARAM) 1100);   // min. & max. positions 

		   	SendDlgItemMessage(hwndDlg, IDC_SCROLL_QUALITY, SBM_SETPOS, 
        			(WPARAM) (setup_quality + 100),
		        	(LPARAM) TRUE);                  // redraw flag 

		   	SendDlgItemMessage(hwndDlg, IDC_EDIT1, EM_SETLIMITTEXT,
        			(WPARAM) 5, (LPARAM)0);

			sprintf(buf2, "%3.2f", ((float)(setup_quality)/100.0));

			SendDlgItemMessage(hwndDlg, IDC_EDIT1, WM_SETTEXT,
				(WPARAM)0, (LPARAM)buf2);

			qcValue = setup_quality;

			nominalBitrate = qc2approxBitrate( (float)qcValue, iniSettings.resample_rate, 2 );

			sprintf(buf2, "%.1fkbps", (float)nominalBitrate/1000);

			SendDlgItemMessage(hwndDlg, IDC_EDIT2, WM_SETTEXT,
				(WPARAM)0, (LPARAM)buf2);

			(void) CheckRadioButton(hwndDlg, IDC_USEQUALITY,
											 IDC_USEVBR,
											 iniSettings.mode);

			if (iniSettings.del_input_wav)
				CheckDlgButton(hwndDlg,IDC_DEL_WAVE_IN, TRUE);

			if (iniSettings.with_skeleton)
				CheckDlgButton(hwndDlg,IDC_SKELETON, TRUE);

			/* set to default scale factor */
		   	SendDlgItemMessage(hwndDlg, IDC_EDIT_SCALE_FACTOR, EM_SETLIMITTEXT,
        			(WPARAM) 6, (LPARAM)0);

			sprintf(buf2, "%.2f", iniSettings.scale_factor);

			SendDlgItemMessage(hwndDlg, IDC_EDIT_SCALE_FACTOR, WM_SETTEXT,
				(WPARAM)0, (LPARAM)buf2);

			if (iniSettings.scale_factor != 100.00 && iniSettings.scale_factor != 0.) {
				dB = 20. * log10(iniSettings.scale_factor/100);
				CheckDlgButton(hwndDlg,IDC_CHECK_SCALE, TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT_SCALE_FACTOR), TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT_DB), TRUE);
			}
			else {
				dB = 0.;
				EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT_SCALE_FACTOR), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT_DB), FALSE);
			}

		   	SendDlgItemMessage(hwndDlg, IDC_EDIT_DB, EM_SETLIMITTEXT,
        			(WPARAM) 48, (LPARAM)0);

			if (dB < 0.) {
				sprintf(dB_buf, "will DECREASE the volume by %.4fdB.", dB);

				SendDlgItemMessage(hwndDlg, IDC_EDIT_DB, WM_SETTEXT,
					(WPARAM)0, (LPARAM)dB_buf);
			}
			else if (dB > 0.) {
				sprintf(dB_buf, "will INCREASE the volume by %.4fdB.", dB);

				SendDlgItemMessage(hwndDlg, IDC_EDIT_DB, WM_SETTEXT,
					(WPARAM)0, (LPARAM)dB_buf);
			}
			else {
				sprintf(dB_buf, "will leave the volume UNCHANGED.");

				SendDlgItemMessage(hwndDlg, IDC_EDIT_DB, WM_SETTEXT,
					(WPARAM)0, (LPARAM)dB_buf);
			}

			if (iniSettings.FLAC_scale == 1) {
				CheckDlgButton(hwndDlg,IDC_USE_FLAC_TITLE, TRUE);
			}
			else if (iniSettings.FLAC_scale == 2) {
				CheckDlgButton(hwndDlg,IDC_USE_FLAC_ALBUM, TRUE);
			}

			if (iniSettings.delete_RG == 1) {
				CheckDlgButton(hwndDlg,IDC_DEL_RG_TAG, TRUE);
			}

			/* set to default iniSettings.lowpass */
		   	SendDlgItemMessage(hwndDlg, IDC_EDIT_LOWPASS, EM_SETLIMITTEXT,
        			(WPARAM) 5, (LPARAM)0);

			if (iniSettings.lowpass_mode) {
				CheckDlgButton(hwndDlg,IDC_CHECK_LOWPASS, TRUE);
				sprintf(buf2, "%d", iniSettings.lowpass);
				SendDlgItemMessage(hwndDlg, IDC_EDIT_LOWPASS, WM_SETTEXT,
					(WPARAM) 0, (LPARAM) buf2);
			}
			else {
				CheckDlgButton(hwndDlg,IDC_CHECK_LOWPASS, FALSE);
				sprintf(buf2, "%d", 0);
				SendDlgItemMessage(hwndDlg, IDC_EDIT_LOWPASS, WM_SETTEXT,
					(WPARAM) 0, (LPARAM) buf2);
				EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT_LOWPASS), FALSE);
			}

			for (br=0; resampleCaption[br] != 0; br++) {
				SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE, CB_ADDSTRING,
	        	  		(WPARAM)0, (LPARAM) resampleCaption[br]);

				SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE, CB_SETITEMDATA, 
	        	  		(WPARAM) br, (LPARAM) atoi(resampleCaption[br]));
			}

			/* set to default sample rate */
			for (br=0; resampleCaption[br] != 0; br++) {
				if ( atoi(resampleCaption[br]) == iniSettings.resample_rate) {
					SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE, CB_SETCURSEL,
	        	  			(WPARAM) br, (LPARAM) 0);
					break;
				}
			}

			if (iniSettings.resample_mode == 0) {
				CheckDlgButton(hwndDlg,IDC_RESAMPLE_NEW, FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_RESAMPLE_RATE), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_CHECK1), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_CONVERTER), FALSE);
			}
			else if (iniSettings.resample_mode == 1) {
				CheckDlgButton(hwndDlg,IDC_RESAMPLE_NEW, TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_RESAMPLE_RATE), TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_CHECK1), TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_CONVERTER), TRUE);
			}

			for (br=0; converter[br] != 0; br++) {
				SendDlgItemMessage(hwndDlg, IDC_CONVERTER, CB_ADDSTRING,
	        	  		(WPARAM)0, (LPARAM) converter[br]);

				SendDlgItemMessage(hwndDlg, IDC_CONVERTER, CB_SETITEMDATA, 
	        	  		(WPARAM) br, (LPARAM) converter[br]);
			}

			SendDlgItemMessage(hwndDlg, IDC_CONVERTER, CB_SETCURSEL,
	            		(WPARAM) iniSettings.converter, (LPARAM) 0);

			for (br = 0; br < 501; br++) {
				char br_str[4];
				sprintf(br_str, "%d", br);
				SendDlgItemMessage(hwndDlg, IDC_APPROX_BITRATE, CB_ADDSTRING,
	        	  		(WPARAM)0, (LPARAM) br_str);

				SendDlgItemMessage(hwndDlg, IDC_APPROX_BITRATE, CB_SETITEMDATA, 
	        	  		(WPARAM) br, (LPARAM) atoi(br_str));

				SendDlgItemMessage(hwndDlg, IDC_MIN_BITRATE, CB_ADDSTRING,
	        	  		(WPARAM)0, (LPARAM) br_str);

				SendDlgItemMessage(hwndDlg, IDC_MIN_BITRATE, CB_SETITEMDATA, 
	        	  		(WPARAM) br, (LPARAM) atoi(br_str));

				SendDlgItemMessage(hwndDlg, IDC_NOM_BITRATE, CB_ADDSTRING,
        	  			(WPARAM)0, (LPARAM) br_str);

				SendDlgItemMessage(hwndDlg, IDC_NOM_BITRATE, CB_SETITEMDATA, 
        			  	(WPARAM) br, (LPARAM) atoi(br_str));

				SendDlgItemMessage(hwndDlg, IDC_MAX_BITRATE, CB_ADDSTRING,
        			  	(WPARAM)0, (LPARAM) br_str);

				SendDlgItemMessage(hwndDlg, IDC_MAX_BITRATE, CB_SETITEMDATA, 
        			  	(WPARAM) br, (LPARAM) atoi(br_str));
			}

			if (setup_mode == IDC_USEVBR) {
				EnableWindow(GetDlgItem(hwndDlg, IDC_SCROLL_QUALITY), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_MIN_BITRATE), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_NOM_BITRATE), TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_MAX_BITRATE), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT1), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT2), FALSE);
				if (!decoding) {
					strcpy(approxBRCaption, "Approximate Bitrate: ");
					sprintf(nomBRCaption, "%.1fkbps", (float)setup_VBRbitrate);
					strcpy(kCaption, "");
					strcpy(speedCaption, "");
				}
				SendDlgItemMessage(hwndDlg, IDC_NOM_BITRATE, CB_SETCURSEL,
		            		(WPARAM) setup_VBRbitrate, (LPARAM) 0);
				SendDlgItemMessage(hwndDlg, IDC_NOMBR_TITLE, WM_SETTEXT,(WPARAM)0,
					(LPARAM)"Approx. Bitrate");
				if (setup_hard_q_limits) {
					EnableWindow(GetDlgItem(hwndDlg, IDC_Q_HARD_LIMITS), TRUE);
					CheckDlgButton(hwndDlg,IDC_Q_HARD_LIMITS, TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_MIN_BITRATE), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_MAX_BITRATE), TRUE);
					SendDlgItemMessage(hwndDlg, IDC_MINBR_TITLE, WM_SETTEXT,(WPARAM)0,
						(LPARAM)"Hard Min. Bitrate");
					SendDlgItemMessage(hwndDlg, IDC_MAXBR_TITLE, WM_SETTEXT,(WPARAM)0,
						(LPARAM)"Hard Max. Bitrate");
					SendDlgItemMessage(hwndDlg, IDC_MIN_BITRATE, CB_SETCURSEL,
			            		(WPARAM) setup_hard_min, (LPARAM) 0);
					SendDlgItemMessage(hwndDlg, IDC_MAX_BITRATE, CB_SETCURSEL,
	            				(WPARAM) setup_hard_max, (LPARAM) 0);
				}
				else {
					SendDlgItemMessage(hwndDlg, IDC_MINBR_TITLE, WM_SETTEXT,(WPARAM)0,
						(LPARAM)"Min. Bitrate");
					SendDlgItemMessage(hwndDlg, IDC_MAXBR_TITLE, WM_SETTEXT,(WPARAM)0,
						(LPARAM)"Max. Bitrate");
				}
				EnableWindow(GetDlgItem(hwndDlg, IDC_Q_HARD_LIMITS), TRUE);
			}
			else if (setup_mode == IDC_USEABRBITRATE) {
				EnableWindow(GetDlgItem(hwndDlg, IDC_SCROLL_QUALITY), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_APPROX_BITRATE), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_MIN_BITRATE), TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_NOM_BITRATE), TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_MAX_BITRATE), TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT1), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT2), FALSE);
				if (!decoding) {
					strcpy(approxBRCaption, "Bitrate: ");
					sprintf(nomBRCaption, "%.1fkbps", (float)setup_NOMbitrate);
					strcpy(kCaption, "");
					strcpy(speedCaption, "");
				}
				SendDlgItemMessage(hwndDlg, IDC_NOM_BITRATE, CB_SETCURSEL,
		            		(WPARAM) setup_NOMbitrate, (LPARAM) 0);
				SendDlgItemMessage(hwndDlg, IDC_NOMBR_TITLE, WM_SETTEXT,(WPARAM)0,
					(LPARAM)"Nom. Bitrate");
				SendDlgItemMessage(hwndDlg, IDC_MIN_BITRATE, CB_SETCURSEL,
		            		(WPARAM) setup_MINbitrate, (LPARAM) 0);
				SendDlgItemMessage(hwndDlg, IDC_MAX_BITRATE, CB_SETCURSEL,
	            			(WPARAM) setup_MAXbitrate, (LPARAM) 0);
				SendDlgItemMessage(hwndDlg, IDC_MINBR_TITLE, WM_SETTEXT,(WPARAM)0,
					(LPARAM)"Min. Bitrate");
				SendDlgItemMessage(hwndDlg, IDC_MAXBR_TITLE, WM_SETTEXT,(WPARAM)0,
					(LPARAM)"Max. Bitrate");
				EnableWindow(GetDlgItem(hwndDlg, IDC_Q_HARD_LIMITS), FALSE);
			}
			else if (setup_mode == IDC_USECBRBITRATE) {
				EnableWindow(GetDlgItem(hwndDlg, IDC_SCROLL_QUALITY), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_APPROX_BITRATE), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_MIN_BITRATE), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_NOM_BITRATE), TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_MAX_BITRATE), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT1), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT2), FALSE);
				if (!decoding) {
					strcpy(approxBRCaption, "Bitrate: ");
					sprintf(nomBRCaption, "%.1fkbps", (float)setup_CBRbitrate);
					strcpy(kCaption, "");
					strcpy(speedCaption, "");
				}
				SendDlgItemMessage(hwndDlg, IDC_NOM_BITRATE, CB_SETCURSEL,
		            		(WPARAM) setup_CBRbitrate, (LPARAM) 0);
				SendDlgItemMessage(hwndDlg, IDC_NOMBR_TITLE, WM_SETTEXT,(WPARAM)0,
					(LPARAM)"Nom. Bitrate");
				SendDlgItemMessage(hwndDlg, IDC_MIN_BITRATE, CB_SETCURSEL,
		            		(WPARAM) setup_MINbitrate, (LPARAM) 0);
				SendDlgItemMessage(hwndDlg, IDC_MAX_BITRATE, CB_SETCURSEL,
	            			(WPARAM) setup_MAXbitrate, (LPARAM) 0);
				SendDlgItemMessage(hwndDlg, IDC_MINBR_TITLE, WM_SETTEXT,(WPARAM)0,
					(LPARAM)"Min. Bitrate");
				SendDlgItemMessage(hwndDlg, IDC_MAXBR_TITLE, WM_SETTEXT,(WPARAM)0,
					(LPARAM)"Max. Bitrate");
				EnableWindow(GetDlgItem(hwndDlg, IDC_Q_HARD_LIMITS), FALSE);
			}
			else {
				EnableWindow(GetDlgItem(hwndDlg, IDC_APPROX_BITRATE), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_MIN_BITRATE), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_NOM_BITRATE), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_MAX_BITRATE), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT1), TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT2), TRUE);
				qcValue = setup_quality;
				nominalBitrate = qc2approxBitrate( (float)qcValue, iniSettings.resample_rate, 2 );
				if (!decoding) {
					strcpy(approxBRCaption, "Nominal Bitrate: ");
					sprintf(nomBRCaption, "%.1fkbps", (float)nominalBitrate/1000);
					sprintf(kCaption, "-q %.2f", (float)qcValue/100);
					strcpy(speedCaption, "");
				}
				SendDlgItemMessage(hwndDlg, IDC_NOMBR_TITLE, WM_SETTEXT,(WPARAM)0,
					(LPARAM)"Nom. Bitrate");
				if (setup_hard_q_limits) {
					EnableWindow(GetDlgItem(hwndDlg, IDC_Q_HARD_LIMITS), TRUE);
					CheckDlgButton(hwndDlg,IDC_Q_HARD_LIMITS, TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_MIN_BITRATE), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_MAX_BITRATE), TRUE);
					SendDlgItemMessage(hwndDlg, IDC_MINBR_TITLE, WM_SETTEXT,(WPARAM)0,
						(LPARAM)"Hard Min. Bitrate");
					SendDlgItemMessage(hwndDlg, IDC_MAXBR_TITLE, WM_SETTEXT,(WPARAM)0,
						(LPARAM)"Hard Max. Bitrate");
					SendDlgItemMessage(hwndDlg, IDC_MIN_BITRATE, CB_SETCURSEL,
			            		(WPARAM) setup_hard_min, (LPARAM) 0);
					SendDlgItemMessage(hwndDlg, IDC_MAX_BITRATE, CB_SETCURSEL,
	            				(WPARAM) setup_hard_max, (LPARAM) 0);
				}
				else {
					SendDlgItemMessage(hwndDlg, IDC_MINBR_TITLE, WM_SETTEXT,(WPARAM)0,
						(LPARAM)"Min. Bitrate");
					SendDlgItemMessage(hwndDlg, IDC_MAXBR_TITLE, WM_SETTEXT,(WPARAM)0,
						(LPARAM)"Max. Bitrate");
				}
				EnableWindow(GetDlgItem(hwndDlg, IDC_Q_HARD_LIMITS), TRUE);
			}
		}
		return TRUE;

		case WM_HSCROLL: {

			double nominalBitrate;

			EnableWindow(GetDlgItem(hwndDlg, IDC_MIN_BITRATE), FALSE);
			EnableWindow(GetDlgItem(hwndDlg, IDC_NOM_BITRATE), FALSE);
			EnableWindow(GetDlgItem(hwndDlg, IDC_MAX_BITRATE), FALSE);
			EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT1), TRUE);
			EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT2), TRUE);

			switch (LOWORD(wParam)) {

				case SB_THUMBPOSITION:
				case SB_THUMBTRACK:
				   	SendDlgItemMessage(hwndDlg, IDC_SCROLL_QUALITY, SBM_SETPOS, 
        					(WPARAM) (HIWORD(wParam)),
		        			(LPARAM) TRUE);                  // redraw flag 
					break;
				case SB_LINELEFT:
				   	SendDlgItemMessage(hwndDlg, IDC_SCROLL_QUALITY, SBM_SETPOS, 
        					(WPARAM) (SendDlgItemMessage(hwndDlg, IDC_SCROLL_QUALITY, SBM_GETPOS, (WPARAM)0, (LPARAM)0) - 1),
		        			(LPARAM) TRUE);                  // redraw flag 
					break;
				case SB_LINERIGHT:
				   	SendDlgItemMessage(hwndDlg, IDC_SCROLL_QUALITY, SBM_SETPOS, 
        					(WPARAM) (SendDlgItemMessage(hwndDlg, IDC_SCROLL_QUALITY, SBM_GETPOS, (WPARAM)0, (LPARAM)0) + 1),
		        			(LPARAM) TRUE);                  // redraw flag 
					break;
			}

			qcValue = SendDlgItemMessage(hwndDlg, IDC_SCROLL_QUALITY, SBM_GETPOS, (WPARAM)0, (LPARAM)0) - 100;

			sprintf(buf2, "%3.2f", (float)qcValue/100);

			SendDlgItemMessage(hwndDlg, IDC_EDIT1, WM_SETTEXT,(WPARAM)0, (LPARAM)buf2);

			nominalBitrate = qc2approxBitrate( (float)qcValue, iniSettings.resample_rate, 2 );

			sprintf(buf2, "%.1fkbps", (float)nominalBitrate/1000);

			SendDlgItemMessage(hwndDlg, IDC_EDIT2, WM_SETTEXT,(WPARAM)0, (LPARAM)buf2);

			(void) CheckRadioButton(hwndDlg, IDC_USEQUALITY,
											 IDC_USECBRBITRATE,
											 IDC_USEQUALITY);
			if (!decoding) {
				strcpy(approxBRCaption, "Nominal Bitrate: ");
				sprintf(nomBRCaption, "%.1fkbps", (float)nominalBitrate/1000);
				sprintf(kCaption, "-q %.2f", (float)qcValue/100);
				strcpy(speedCaption, "");
			}

			break;
		}

		case WM_CLOSE:

			// Closing the Dialog behaves the same as Cancel               
			PostMessage(hwndDlg, WM_COMMAND, IDCANCEL, 0);
			break; // End of WM_CLOSE                                      

		case WM_COMMAND: 

			switch (LOWORD(wParam)) {
				case IDC_BUTTON1: {

					double nominalBitrate;
					int    MinbitRate;
					int    NombitRate;
					int    MaxbitRate;
					int    resample_rate;
					double v = 0.0;

					if (IsDlgButtonChecked(hwndDlg, IDC_DOWNMIX) == BST_CHECKED)
						set_downmix(1);               // Downmix to mono
					else
						set_downmix(0);

					if ((IsDlgButtonChecked(hwndDlg, IDC_CHECK1) == BST_CHECKED) &&
					    (IsDlgButtonChecked(hwndDlg, IDC_RESAMPLE_NEW) == BST_CHECKED)) {
						if ((br = SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE, CB_GETCURSEL,
								(WPARAM) 0, (LPARAM) 0)) != CB_ERR) {
							set_resample_rate(SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE,
									CB_GETITEMDATA, (WPARAM) br, (LPARAM) 0));
							resample_rate = SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE,
									CB_GETITEMDATA, (WPARAM) br, (LPARAM) 0);
							set_resample_mode(0);
						}
					}
					else if (IsDlgButtonChecked(hwndDlg, IDC_RESAMPLE_NEW) == BST_CHECKED) {
						if ((br = SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE, CB_GETCURSEL,
								(WPARAM) 0, (LPARAM) 0)) != CB_ERR) {
							set_resample_rate(SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE, 
									CB_GETITEMDATA,	(WPARAM) br, (LPARAM) 0));
							resample_rate = SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE,
									CB_GETITEMDATA, (WPARAM) br, (LPARAM) 0);
							set_resample_mode(1);
						}
					}
					else {
						set_resample_mode(0);
						set_resample_rate(0);
						resample_rate = 44100;
					}

					if ((br = SendDlgItemMessage(hwndDlg, IDC_CONVERTER, CB_GETCURSEL,
							(WPARAM) 0, (LPARAM) 0)) != CB_ERR) {
						set_converter(br);
					}
					if (IsDlgButtonChecked(hwndDlg, IDC_CHECK_LOWPASS) == BST_CHECKED) {
						SendDlgItemMessage(hwndDlg, IDC_EDIT_LOWPASS, WM_GETTEXT,
    	          					(WPARAM) 6, (LPARAM) buf2);
						set_lowpass_mode(1);
						set_lowpass(atoi(buf2));
					}
					else {
						set_lowpass_mode(0);
						set_lowpass(0);
					}

					if (IsDlgButtonChecked(hwndDlg, IDC_CHECK_SCALE) == BST_CHECKED) {
						SendDlgItemMessage(hwndDlg, IDC_EDIT_SCALE_FACTOR, WM_GETTEXT,
        		  				(WPARAM) 6, (LPARAM) buf2);

						set_scale_factor(atof(buf2));
					}
					else
						set_scale_factor(100.0);

					if (IsDlgButtonChecked(hwndDlg, IDC_USE_FLAC_TITLE) == BST_CHECKED) {
						set_FLAC_tag_scale(1);
						set_copy_flac_tag(1);
					}
					else if (IsDlgButtonChecked(hwndDlg, IDC_USE_FLAC_ALBUM) == BST_CHECKED) {
						set_FLAC_tag_scale(2);
						set_copy_flac_tag(1);
					}
					else
						set_FLAC_tag_scale(0);

					if (IsDlgButtonChecked(hwndDlg, IDC_DEL_RG_TAG) == BST_CHECKED)
						set_del_RG_tag(1);
					else
						set_del_RG_tag(0);

					set_del_input_wav(IsDlgButtonChecked(hwndDlg, IDC_DEL_WAVE_IN) == BST_CHECKED ? 1 : 0);

					set_with_skeleton(IsDlgButtonChecked(hwndDlg, IDC_SKELETON) == BST_CHECKED ? 1 : 0);

					if ((br = SendDlgItemMessage(hwndDlg, IDC_NOM_BITRATE, CB_GETCURSEL,
							(WPARAM) 0, (LPARAM) 0)) != CB_ERR)
						NombitRate = SendDlgItemMessage(hwndDlg, IDC_NOM_BITRATE, CB_GETITEMDATA,
							(WPARAM) br, (LPARAM) 0);

					if ((br = SendDlgItemMessage(hwndDlg, IDC_MIN_BITRATE, CB_GETCURSEL,
							(WPARAM) 0, (LPARAM) 0)) != CB_ERR)
						MinbitRate = SendDlgItemMessage(hwndDlg, IDC_MIN_BITRATE, CB_GETITEMDATA,
							(WPARAM) br, (LPARAM) 0);

					if ((br = SendDlgItemMessage(hwndDlg, IDC_MAX_BITRATE, CB_GETCURSEL,
							(WPARAM) 0, (LPARAM) 0)) != CB_ERR)
						MaxbitRate = SendDlgItemMessage(hwndDlg, IDC_MAX_BITRATE, CB_GETITEMDATA,
							(WPARAM) br, (LPARAM) 0);

					if (IsDlgButtonChecked(hwndDlg, IDC_USEQUALITY) == BST_CHECKED) {
						set_mode(IDC_USEQUALITY);
						SendDlgItemMessage(hwndDlg, IDC_EDIT1, WM_GETTEXT,
							(WPARAM) 6, (LPARAM) buf2);
						v = atof(buf2);
						if (v > 0)
							v += 0.005;
						else
							v -= 0.005;
						qcValue = (int)(v * 100.0f);
						nominalBitrate = qc2approxBitrate( (float)qcValue, iniSettings.resample_rate, 2 );
						if (!decoding) {
							strcpy(approxBRCaption, "Nominal Bitrate: ");
							sprintf(nomBRCaption, "%.1fkbps", (float)nominalBitrate/1000);
							sprintf(kCaption, "-q %.2f", (float)qcValue/100);
							strcpy(speedCaption, "");
						}
						set_quality_coefficient(qcValue);
						if (IsDlgButtonChecked(hwndDlg, IDC_Q_HARD_LIMITS) == BST_CHECKED) {
							set_hard_q_limits(1);
							set_hard_min(MinbitRate);
							set_hard_max(MaxbitRate);
						}
						else
							set_hard_q_limits(0);
					}
					else if (IsDlgButtonChecked(hwndDlg, IDC_USEVBR) == BST_CHECKED) {
						set_mode(IDC_USEVBR);
						if (!decoding) {
							strcpy(approxBRCaption, "Approximate Bitrate: ");
							sprintf(nomBRCaption, "%.1fkbps", (float)NombitRate);
							strcpy(kCaption, "");
							strcpy(speedCaption, "");
						}
						set_APPROXbitrate(NombitRate);
						if (IsDlgButtonChecked(hwndDlg, IDC_Q_HARD_LIMITS) == BST_CHECKED) {
							set_hard_q_limits(1);
							set_hard_min(MinbitRate);
							set_hard_max(MaxbitRate);
						}
						else
							set_hard_q_limits(0);
					}
					else if (IsDlgButtonChecked(hwndDlg, IDC_USEABRBITRATE) == BST_CHECKED) {
						set_mode(IDC_USEABRBITRATE);
						if (!decoding) {
							strcpy(approxBRCaption, "Bitrate: ");
							sprintf(nomBRCaption, "%.1fkbps", (float)NombitRate);
							strcpy(kCaption, "");
							strcpy(speedCaption, "");
						}

						set_MINbitrate(MinbitRate);
						set_NOMbitrate(NombitRate > 0 ? NombitRate:128);
						set_MAXbitrate(MaxbitRate);
					}
					else if (IsDlgButtonChecked(hwndDlg, IDC_USECBRBITRATE) == BST_CHECKED) {
						set_mode(IDC_USECBRBITRATE);
						if (!decoding) {
							strcpy(approxBRCaption, "Bitrate: ");
							sprintf(nomBRCaption, "%.1fkbps", (float)NombitRate);
							strcpy(kCaption, "");
							strcpy(speedCaption, "");
						}
						set_CBRbitrate(NombitRate > 0 ? NombitRate:128);
					}

					if ((IsDlgButtonChecked(hwndDlg, IDC_USEABRBITRATE) == BST_CHECKED) ||
					    (IsDlgButtonChecked(hwndDlg, IDC_USECBRBITRATE) == BST_CHECKED)) {
						if (NombitRate < 45 && NombitRate > 35) {
							if (resample_rate > 32000) {
								set_resample_rate(32000);
								set_resample_mode(0);
							}
						}
						else if (NombitRate < 36 && NombitRate > 29) {
							if (resample_rate > 24000) {
								set_resample_rate(24000);
								set_resample_mode(0);
							}
						}
						else if (NombitRate < 30 && NombitRate > 23) {
							if (resample_rate > 16000) {
								set_resample_rate(16000);
								set_resample_mode(0);
							}
						}
						else if (NombitRate < 24 && NombitRate > 15) {
							if (resample_rate > 11025) {
								set_resample_rate(11025);
								set_resample_mode(0);
							}
						}
						else if (NombitRate < 16 && NombitRate > 11) {
							if (resample_rate > 8000) {
								set_resample_rate(8000);
								set_resample_mode(0);
							}
						}
					}

					EndDialog(hwndDlg, FALSE);
				}
				break;

				case IDCANCEL: {

					double nominalBitrate;

					set_quality_coefficient(setup_quality);
					set_APPROXbitrate(setup_VBRbitrate);
					set_MINbitrate(setup_MINbitrate);
					set_NOMbitrate(setup_NOMbitrate);
					set_MAXbitrate(setup_MAXbitrate);

					if (setup_mode == IDC_USEQUALITY) {
						set_quality_coefficient(setup_quality);
						qcValue = setup_quality;
						nominalBitrate = qc2approxBitrate( (float)qcValue, iniSettings.resample_rate, 2 );
						if (!decoding) {
							strcpy(approxBRCaption, "Nominal Bitrate: ");
							sprintf(nomBRCaption, "%.1fkbps", (float)nominalBitrate/1000);
							sprintf(kCaption, "-q %.2f", (float)qcValue/100);
							strcpy(speedCaption, "");
						}
					}
					else if (setup_mode == IDC_USEVBR) {
						set_APPROXbitrate(setup_VBRbitrate);
						if (!decoding) {
							strcpy(approxBRCaption, "Approximate Bitrate: ");
							sprintf(nomBRCaption, "%.1fkbps", (float)setup_VBRbitrate);
							strcpy(kCaption, "");
							strcpy(speedCaption, "");
						}
					}
					else if (setup_mode == IDC_USEABRBITRATE) {
						set_NOMbitrate(setup_NOMbitrate);
						if (!decoding) {
							strcpy(approxBRCaption, "Bitrate: ");
							sprintf(nomBRCaption, "%.1fkbps", (float)setup_NOMbitrate);
							strcpy(kCaption, "");
							strcpy(speedCaption, "");
						}
					}
					else if (setup_mode == IDC_USECBRBITRATE) {
						set_CBRbitrate(setup_CBRbitrate);
						if (!decoding) {
							strcpy(approxBRCaption, "Bitrate: ");
							sprintf(nomBRCaption, "%.1fkbps", (float)setup_CBRbitrate);
							strcpy(kCaption, "");
							strcpy(speedCaption, "");
						}
					}
					set_hard_q_limits(setup_hard_q_limits);
					set_hard_min(setup_hard_min);
					set_hard_max(setup_hard_max);

					EndDialog(hwndDlg, FALSE);
				}
				break;

				case IDC_USEQUALITY: {

					double v=0.0;
					double nominalBitrate;

					EnableWindow(GetDlgItem(hwndDlg, IDC_SCROLL_QUALITY), TRUE);
					CheckDlgButton(hwndDlg,IDC_USEVBR,FALSE);
					CheckDlgButton(hwndDlg,IDC_USEABRBITRATE,FALSE);
					CheckDlgButton(hwndDlg,IDC_USECBRBITRATE,FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_NOM_BITRATE), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT1), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT2), TRUE);

					SendDlgItemMessage(hwndDlg, IDC_EDIT1, WM_GETTEXT,
						(WPARAM) 6, (LPARAM) buf2);

					v = atof(buf2);

					qcValue = (int)(v * 100.0f);

					nominalBitrate = qc2approxBitrate( (float)qcValue, iniSettings.resample_rate, 2 );
					if (!decoding) {
						strcpy(approxBRCaption, "Nominal Bitrate: ");
						sprintf(nomBRCaption, "%.1fkbps", (float)nominalBitrate/1000);
						sprintf(kCaption, "-q %.2f", (float)qcValue/100);
						strcpy(speedCaption, "");
					}
					SendDlgItemMessage(hwndDlg, IDC_NOMBR_TITLE, WM_SETTEXT,(WPARAM)0,
						(LPARAM)"Nom. Bitrate");
					EnableWindow(GetDlgItem(hwndDlg, IDC_Q_HARD_LIMITS), TRUE);
					if (IsDlgButtonChecked(hwndDlg, IDC_Q_HARD_LIMITS) == BST_CHECKED) {
						CheckDlgButton(hwndDlg,IDC_Q_HARD_LIMITS, TRUE);
						SendDlgItemMessage(hwndDlg, IDC_MINBR_TITLE, WM_SETTEXT,(WPARAM)0,
							(LPARAM)"Hard Min. Bitrate");
						SendDlgItemMessage(hwndDlg, IDC_MAXBR_TITLE, WM_SETTEXT,(WPARAM)0,
							(LPARAM)"Hard Max. Bitrate");
						EnableWindow(GetDlgItem(hwndDlg, IDC_MIN_BITRATE), TRUE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_MAX_BITRATE), TRUE);
						SendDlgItemMessage(hwndDlg, IDC_MIN_BITRATE, CB_SETCURSEL,
				            		(WPARAM) setup_hard_min, (LPARAM) 0);
						SendDlgItemMessage(hwndDlg, IDC_MAX_BITRATE, CB_SETCURSEL,
	            					(WPARAM) setup_hard_max, (LPARAM) 0);
					}
					else {
						EnableWindow(GetDlgItem(hwndDlg, IDC_MIN_BITRATE), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_MAX_BITRATE), FALSE);
					}
				}
				break;

				case IDC_USEVBR: {

					SendDlgItemMessage(hwndDlg, IDC_NOM_BITRATE, CB_SETCURSEL,
							(WPARAM) temp_VBRbitrate, (LPARAM) 0);
					EnableWindow(GetDlgItem(hwndDlg, IDC_SCROLL_QUALITY), FALSE);
					CheckDlgButton(hwndDlg,IDC_USEABRBITRATE,FALSE);
					CheckDlgButton(hwndDlg,IDC_USECBRBITRATE,FALSE);
					CheckDlgButton(hwndDlg,IDC_USEQUALITY,FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_MIN_BITRATE), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_NOM_BITRATE), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_MAX_BITRATE), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT1), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT2), FALSE);
					if (!decoding) {
						strcpy(approxBRCaption, "Approximate Bitrate: ");
						sprintf(nomBRCaption, "%.1fkbps", (float)temp_VBRbitrate);
						strcpy(kCaption, "");
						strcpy(speedCaption, "");
					}
					SendDlgItemMessage(hwndDlg, IDC_NOMBR_TITLE, WM_SETTEXT,(WPARAM)0,
						(LPARAM)"Approx. Bitrate");
					EnableWindow(GetDlgItem(hwndDlg, IDC_Q_HARD_LIMITS), TRUE);
					if (IsDlgButtonChecked(hwndDlg, IDC_Q_HARD_LIMITS) == BST_CHECKED) {
						CheckDlgButton(hwndDlg,IDC_Q_HARD_LIMITS, TRUE);
						SendDlgItemMessage(hwndDlg, IDC_MINBR_TITLE, WM_SETTEXT,(WPARAM)0,
							(LPARAM)"Hard Min. Bitrate");
						SendDlgItemMessage(hwndDlg, IDC_MAXBR_TITLE, WM_SETTEXT,(WPARAM)0,
							(LPARAM)"Hard Max. Bitrate");
						EnableWindow(GetDlgItem(hwndDlg, IDC_MIN_BITRATE), TRUE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_MAX_BITRATE), TRUE);
						SendDlgItemMessage(hwndDlg, IDC_MIN_BITRATE, CB_SETCURSEL,
				            		(WPARAM) setup_hard_min, (LPARAM) 0);
						SendDlgItemMessage(hwndDlg, IDC_MAX_BITRATE, CB_SETCURSEL,
	            					(WPARAM) setup_hard_max, (LPARAM) 0);
					}
					else {
						SendDlgItemMessage(hwndDlg, IDC_MINBR_TITLE, WM_SETTEXT,(WPARAM)0,
							(LPARAM)"Min. Bitrate");
						SendDlgItemMessage(hwndDlg, IDC_MAXBR_TITLE, WM_SETTEXT,(WPARAM)0,
							(LPARAM)"Max. Bitrate");
						SendDlgItemMessage(hwndDlg, IDC_MIN_BITRATE, CB_SETCURSEL,
							(WPARAM) 0, (LPARAM) 0);
						SendDlgItemMessage(hwndDlg, IDC_MAX_BITRATE, CB_SETCURSEL,
							(WPARAM) 0, (LPARAM) 0);
						EnableWindow(GetDlgItem(hwndDlg, IDC_MIN_BITRATE), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_MAX_BITRATE), FALSE);
					}
				}
				break;

				case IDC_USEABRBITRATE: {

					SendDlgItemMessage(hwndDlg, IDC_NOM_BITRATE, CB_SETCURSEL,
						(WPARAM) temp_NOMbitrate, (LPARAM) 0);
					SendDlgItemMessage(hwndDlg, IDC_MIN_BITRATE, CB_SETCURSEL,
						(WPARAM) 0, (LPARAM) 0);
					SendDlgItemMessage(hwndDlg, IDC_MAX_BITRATE, CB_SETCURSEL,
						(WPARAM) 0, (LPARAM) 0);
					EnableWindow(GetDlgItem(hwndDlg, IDC_SCROLL_QUALITY), FALSE);
					CheckDlgButton(hwndDlg,IDC_USEVBR,FALSE);
					CheckDlgButton(hwndDlg,IDC_USECBRBITRATE,FALSE);
					CheckDlgButton(hwndDlg,IDC_USEQUALITY,FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_MIN_BITRATE), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_NOM_BITRATE), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_MAX_BITRATE), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT1), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT2), FALSE);
					if (!decoding) {
						strcpy(approxBRCaption, "Bitrate: ");
						sprintf(nomBRCaption, "%.1fkbps", (float)temp_NOMbitrate);
						strcpy(kCaption, "");
						strcpy(speedCaption, "");
					}
					SendDlgItemMessage(hwndDlg, IDC_NOMBR_TITLE, WM_SETTEXT,(WPARAM)0,
						(LPARAM)"Nom. Bitrate");
					SendDlgItemMessage(hwndDlg, IDC_MINBR_TITLE, WM_SETTEXT,(WPARAM)0,
						(LPARAM)"Min. Bitrate");
					SendDlgItemMessage(hwndDlg, IDC_MAXBR_TITLE, WM_SETTEXT,(WPARAM)0,
						(LPARAM)"Max. Bitrate");
					EnableWindow(GetDlgItem(hwndDlg, IDC_Q_HARD_LIMITS), FALSE);
				}
				break;

				case IDC_USECBRBITRATE: {

					SendDlgItemMessage(hwndDlg, IDC_NOM_BITRATE, CB_SETCURSEL,
							(WPARAM) temp_NOMbitrate, (LPARAM) 0);
					EnableWindow(GetDlgItem(hwndDlg, IDC_SCROLL_QUALITY), FALSE);
					CheckDlgButton(hwndDlg,IDC_USEVBR,FALSE);
					CheckDlgButton(hwndDlg,IDC_USEABRBITRATE,FALSE);
					CheckDlgButton(hwndDlg,IDC_USEQUALITY,FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_MIN_BITRATE), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_NOM_BITRATE), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_MAX_BITRATE), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT1), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT2), FALSE);

					SendDlgItemMessage(hwndDlg, IDC_MIN_BITRATE, CB_SETCURSEL,
						(WPARAM) temp_NOMbitrate, (LPARAM) 0);
					SendDlgItemMessage(hwndDlg, IDC_MAX_BITRATE, CB_SETCURSEL,
						(WPARAM) temp_NOMbitrate, (LPARAM) 0);

					if (!decoding) {
						strcpy(approxBRCaption, "Bitrate: ");
						sprintf(nomBRCaption, "%.1fkbps", (float)temp_NOMbitrate);
						strcpy(kCaption, "");
						strcpy(speedCaption, "");
					}
					SendDlgItemMessage(hwndDlg, IDC_NOMBR_TITLE, WM_SETTEXT,(WPARAM)0,
						(LPARAM)"Nom. Bitrate");
					SendDlgItemMessage(hwndDlg, IDC_MINBR_TITLE, WM_SETTEXT,(WPARAM)0,
						(LPARAM)"Min. Bitrate");
					SendDlgItemMessage(hwndDlg, IDC_MAXBR_TITLE, WM_SETTEXT,(WPARAM)0,
						(LPARAM)"Max. Bitrate");
					EnableWindow(GetDlgItem(hwndDlg, IDC_Q_HARD_LIMITS), FALSE);
				}
				break;

				case IDC_MIN_BITRATE: {

					int   MinbitRate;
					int   NombitRate;

					if ((br = SendDlgItemMessage(hwndDlg, IDC_NOM_BITRATE, CB_GETCURSEL,
							(WPARAM) 0, (LPARAM) 0)) != CB_ERR) {
						NombitRate = SendDlgItemMessage(hwndDlg, IDC_NOM_BITRATE, CB_GETITEMDATA,
								(WPARAM) br, (LPARAM) 0);
					}
					if ((br = SendDlgItemMessage(hwndDlg, IDC_MIN_BITRATE, CB_GETCURSEL,
							(WPARAM) 0, (LPARAM) 0)) != CB_ERR) {
						MinbitRate = SendDlgItemMessage(hwndDlg, IDC_MIN_BITRATE, CB_GETITEMDATA,
								(WPARAM) br, (LPARAM) 0);
						if (MinbitRate > NombitRate) {
							MinbitRate = NombitRate;
							SendDlgItemMessage(hwndDlg, IDC_MIN_BITRATE, CB_SETCURSEL,
									(WPARAM) MinbitRate, (LPARAM) 0);
						}
					}
					break;
				}

				case IDC_NOM_BITRATE: {

					int   MinbitRate;
					int   NombitRate;
					int   MaxbitRate;

					if ((br = SendDlgItemMessage(hwndDlg, IDC_NOM_BITRATE, CB_GETCURSEL,
							(WPARAM) 0, (LPARAM) 0)) != CB_ERR)
						NombitRate = SendDlgItemMessage(hwndDlg, IDC_NOM_BITRATE, CB_GETITEMDATA,
								(WPARAM) br, (LPARAM) 0);

					if (IsDlgButtonChecked(hwndDlg, IDC_USEVBR) == BST_CHECKED) {
						temp_VBRbitrate = NombitRate;
						if (!decoding) {
							strcpy(approxBRCaption, "Approximate Bitrate: ");
							sprintf(nomBRCaption, "%.1fkbps", (float)temp_VBRbitrate);
							strcpy(kCaption, "");
							strcpy(speedCaption, "");
						}
					}
					else {
						if (NombitRate < 12 ) {
							NombitRate = 12;

							SendDlgItemMessage(hwndDlg, IDC_NOM_BITRATE, CB_SETCURSEL,
								(WPARAM) NombitRate, (LPARAM) 0);
						}
						if (IsDlgButtonChecked(hwndDlg, IDC_USECBRBITRATE) == BST_CHECKED) {
							SendDlgItemMessage(hwndDlg, IDC_MIN_BITRATE, CB_SETCURSEL,
								(WPARAM) NombitRate, (LPARAM) 0);
							SendDlgItemMessage(hwndDlg, IDC_MAX_BITRATE, CB_SETCURSEL,
								(WPARAM) NombitRate, (LPARAM) 0);
							temp_NOMbitrate = NombitRate;
						}
						else {
							if ((br = SendDlgItemMessage(hwndDlg, IDC_MIN_BITRATE, CB_GETCURSEL,
									(WPARAM) 0, (LPARAM) 0)) != CB_ERR) {
								MinbitRate = SendDlgItemMessage(hwndDlg, IDC_MIN_BITRATE, CB_GETITEMDATA,
									(WPARAM) br, (LPARAM) 0);
								if (MinbitRate > temp_NOMbitrate) {
									MinbitRate = temp_NOMbitrate;
									SendDlgItemMessage(hwndDlg, IDC_MIN_BITRATE, CB_SETCURSEL,
    											(WPARAM) MinbitRate, (LPARAM) 0);
								}
							}
							if ((br = SendDlgItemMessage(hwndDlg, IDC_MAX_BITRATE, CB_GETCURSEL,
									(WPARAM) 0, (LPARAM) 0)) != CB_ERR) {
								MaxbitRate = SendDlgItemMessage(hwndDlg, IDC_MAX_BITRATE, CB_GETITEMDATA,
									(WPARAM) br, (LPARAM) 0);
								if ((MaxbitRate < temp_NOMbitrate) && (MaxbitRate != 0)) {
									MaxbitRate = temp_NOMbitrate;
									SendDlgItemMessage(hwndDlg, IDC_MAX_BITRATE, CB_SETCURSEL,
    											(WPARAM) MaxbitRate, (LPARAM) 0);
								}
							}
						}
						if (!decoding) {
							strcpy(approxBRCaption, "Bitrate: ");
							sprintf(nomBRCaption, "%.1fkbps", (float)temp_NOMbitrate);
							strcpy(kCaption, "");
							strcpy(speedCaption, "");
						}
					}
					break;
				}

				case IDC_MAX_BITRATE: {

					int   NombitRate;
					int   MaxbitRate;

					if ((br = SendDlgItemMessage(hwndDlg, IDC_NOM_BITRATE, CB_GETCURSEL,
							(WPARAM) 0, (LPARAM) 0)) != CB_ERR)
						NombitRate = SendDlgItemMessage(hwndDlg, IDC_NOM_BITRATE, CB_GETITEMDATA,
							(WPARAM) br, (LPARAM) 0);

					if ((br = SendDlgItemMessage(hwndDlg, IDC_MAX_BITRATE, CB_GETCURSEL,
							(WPARAM) 0, (LPARAM) 0)) != CB_ERR) {
						MaxbitRate = SendDlgItemMessage(hwndDlg, IDC_MAX_BITRATE, CB_GETITEMDATA,
								(WPARAM) br, (LPARAM) 0);
						if (MaxbitRate < NombitRate) {
							if (MaxbitRate > 0) {
								MaxbitRate = NombitRate;
								SendDlgItemMessage(hwndDlg, IDC_MAX_BITRATE, CB_SETCURSEL,
    										(WPARAM) MaxbitRate, (LPARAM) 0);
							}
						}
					}
					break;
				}

				case IDC_Q_HARD_LIMITS: {

					if (IsDlgButtonChecked(hwndDlg, IDC_Q_HARD_LIMITS) == BST_CHECKED) {
						EnableWindow(GetDlgItem(hwndDlg, IDC_MIN_BITRATE), TRUE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_MAX_BITRATE), TRUE);
						SendDlgItemMessage(hwndDlg, IDC_MINBR_TITLE, WM_SETTEXT,(WPARAM)0,
							(LPARAM)"Hard Min. Bitrate");
						SendDlgItemMessage(hwndDlg, IDC_MAXBR_TITLE, WM_SETTEXT,(WPARAM)0,
							(LPARAM)"Hard Max. Bitrate");
						SendDlgItemMessage(hwndDlg, IDC_MIN_BITRATE, CB_SETCURSEL,
			            			(WPARAM) setup_hard_min, (LPARAM) 0);
						SendDlgItemMessage(hwndDlg, IDC_MAX_BITRATE, CB_SETCURSEL,
		            				(WPARAM) setup_hard_max, (LPARAM) 0);
					}
					else {
						EnableWindow(GetDlgItem(hwndDlg, IDC_MIN_BITRATE), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_MAX_BITRATE), FALSE);
						SendDlgItemMessage(hwndDlg, IDC_MINBR_TITLE, WM_SETTEXT,(WPARAM)0,
							(LPARAM)"Min. Bitrate");
						SendDlgItemMessage(hwndDlg, IDC_MAXBR_TITLE, WM_SETTEXT,(WPARAM)0,
							(LPARAM)"Max. Bitrate");
						SendDlgItemMessage(hwndDlg, IDC_MIN_BITRATE, CB_SETCURSEL,
			            			(WPARAM) setup_MINbitrate, (LPARAM) 0);
						SendDlgItemMessage(hwndDlg, IDC_MAX_BITRATE, CB_SETCURSEL,
		            				(WPARAM) setup_MAXbitrate, (LPARAM) 0);
					}
					break;
				}
				

				case IDC_EDIT1:

					switch (HIWORD(wParam)) {
						double nominalBitrate;

						case EN_UPDATE: {
							int i;
							double v=0.0;

							int len = SendDlgItemMessage(hwndDlg, IDC_EDIT1, WM_GETTEXT,
								(WPARAM) 6, (LPARAM) buf2);

							for (i=0; i<len; i++) {
								if (! isdigit(buf2[i]) && buf2[i] != '.'
									&& buf2[i] != ',' && buf2[i] != '-') {

									buf2[i] = '\0';
									SendDlgItemMessage(hwndDlg, IDC_EDIT1, WM_SETTEXT,
										(WPARAM) 0, (LPARAM) buf2);
								}
							}

							v = atof(buf2);

							if ( v > 10.00f ) {
								v = 10.0;
								sprintf(buf2, "%.2f", (float)v);

								SendDlgItemMessage(hwndDlg, IDC_EDIT1, WM_SETTEXT,
									(WPARAM) 0, (LPARAM) buf2);

								SendDlgItemMessage(hwndDlg, IDC_EDIT1, EM_SETSEL,
									(WPARAM)0, (LPARAM)-1);
							}
							else if ( v < -1.00f ) {
								v = -1.0;
								sprintf(buf2, "%.2f", (float)v);

								SendDlgItemMessage(hwndDlg, IDC_EDIT1, WM_SETTEXT,
									(WPARAM) 0, (LPARAM) buf2);

								SendDlgItemMessage(hwndDlg, IDC_EDIT1, EM_SETSEL,
									(WPARAM)0, (LPARAM)-1);
							}

							(void) CheckRadioButton(hwndDlg, IDC_USEQUALITY,
															 IDC_USECBRBITRATE,
															 IDC_USEQUALITY);
							break;
						}

						case EN_CHANGE:	{

							double v=0.0;

							SendDlgItemMessage(hwndDlg, IDC_EDIT1, WM_GETTEXT,
								(WPARAM) 6, (LPARAM) buf2);

							v = atof(buf2);

							if (v > 0)
								v += 0.005;
							else
								v -= 0.005;

							qcValue = (int)(v * 100.0f);

							nominalBitrate = qc2approxBitrate( (float)qcValue, iniSettings.resample_rate, 2 );

							sprintf(buf2, "%.1fkbps", (float)nominalBitrate/1000);

							SendDlgItemMessage(hwndDlg, IDC_EDIT2, WM_SETTEXT,(WPARAM)0, (LPARAM)buf2);

							SendDlgItemMessage(hwndDlg, IDC_SCROLL_QUALITY, SBM_SETPOS, 
					        		(WPARAM) (int)(qcValue + 100),
							        (LPARAM) TRUE);                  // redraw flag 

							if (!decoding) {
								strcpy(approxBRCaption, "Nominal Bitrate: ");
								sprintf(nomBRCaption, "%.1fkbps", (float)nominalBitrate/1000);
								sprintf(kCaption, "-q %.2f", (float)qcValue/100);
								strcpy(speedCaption, "");
							}
							break;
						}

						default:
							break;
        	  			}
					break;

				case IDC_SET_ADV_OPT:

					DialogBox(hinst, MAKEINTRESOURCE(IDD_ADV_ENC_SETUP), hwndDlg, AdvEncSetup);

				case IDC_USE_FLAC_TITLE:

					if (IsDlgButtonChecked(hwndDlg, IDC_USE_FLAC_TITLE) == BST_CHECKED) {
						EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT_SCALE_FACTOR), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT_DB), FALSE);
						CheckDlgButton(hwndDlg,IDC_CHECK_SCALE,FALSE);
						CheckDlgButton(hwndDlg,IDC_USE_FLAC_ALBUM,FALSE);
					}
					break;

				case IDC_USE_FLAC_ALBUM:

					if (IsDlgButtonChecked(hwndDlg, IDC_USE_FLAC_ALBUM) == BST_CHECKED) {
						EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT_SCALE_FACTOR), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT_DB), FALSE);
						CheckDlgButton(hwndDlg,IDC_CHECK_SCALE,FALSE);
						CheckDlgButton(hwndDlg,IDC_USE_FLAC_TITLE,FALSE);
					}
					break;

				case IDC_CHECK_SCALE:

					if (IsDlgButtonChecked(hwndDlg, IDC_CHECK_SCALE) == BST_CHECKED) {
						EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT_SCALE_FACTOR), TRUE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT_DB), TRUE);
						CheckDlgButton(hwndDlg,IDC_USE_FLAC_TITLE,FALSE);
						CheckDlgButton(hwndDlg,IDC_USE_FLAC_ALBUM,FALSE);
					}
					else {
						EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT_SCALE_FACTOR), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT_DB), FALSE);
					}
					break;

				case IDC_EDIT_SCALE_FACTOR:

					switch (HIWORD(wParam)) {
						case EN_UPDATE: {

							int i;

							int len = SendDlgItemMessage(hwndDlg, IDC_EDIT_SCALE_FACTOR, WM_GETTEXT,
    		          					(WPARAM) 6, (LPARAM) buf2);

							for (i=0; i<len; i++) {
								if ((! isdigit(buf2[i]) && buf2[i] != '.')
									&& (! isdigit(buf2[i]) && buf2[i] != '-')) {

									buf2[i] = '\0';
									SendDlgItemMessage(hwndDlg, IDC_EDIT_SCALE_FACTOR, WM_SETTEXT,
										(WPARAM) 0, (LPARAM) buf2);
								}
							}

							if ( atof(buf2) > 400. ) {
								SendDlgItemMessage(hwndDlg, IDC_EDIT_SCALE_FACTOR, WM_SETTEXT,
									(WPARAM) 0, (LPARAM) "400.00");

								SendDlgItemMessage(hwndDlg, IDC_EDIT_SCALE_FACTOR, EM_SETSEL,
									(WPARAM)0, (LPARAM)-1);
							}
							if ( atof(buf2) < 1. ) {
								SendDlgItemMessage(hwndDlg, IDC_EDIT_SCALE_FACTOR, WM_SETTEXT,
									(WPARAM) 0, (LPARAM) "1.00");

								SendDlgItemMessage(hwndDlg, IDC_EDIT_SCALE_FACTOR, EM_SETSEL,
									(WPARAM)0, (LPARAM)-1);
							}
						}
						break;
						case EN_CHANGE: {

							double dB;
							double ScaleFactor;

							SendDlgItemMessage(hwndDlg, IDC_EDIT_SCALE_FACTOR, WM_GETTEXT,
    	        	  					(WPARAM) 6, (LPARAM) buf2);

							ScaleFactor = atof(buf2);

							if (ScaleFactor != 100.00 && ScaleFactor != 0.)
								dB = 20. * log10((double)ScaleFactor/100);
							else
								dB = 0.;

							if (dB < 0.) {
								sprintf(dB_buf, "will DECREASE the volume by %.4fdB.", dB);

								SendDlgItemMessage(hwndDlg, IDC_EDIT_DB, WM_SETTEXT,
									(WPARAM)0, (LPARAM)dB_buf);
							}
							else if (dB > 0.) {
								sprintf(dB_buf, "will INCREASE the volume by %.4fdB.", dB);

								SendDlgItemMessage(hwndDlg, IDC_EDIT_DB, WM_SETTEXT,
									(WPARAM)0, (LPARAM)dB_buf);
							}
							else {
								sprintf(dB_buf, "will leave the volume UNCHANGED.");

								SendDlgItemMessage(hwndDlg, IDC_EDIT_DB, WM_SETTEXT,
									(WPARAM)0, (LPARAM)dB_buf);
							}
						}
						break;
						default:
							break;
	        	  		}
					break;

				case IDC_CHECK_LOWPASS:

					if (IsDlgButtonChecked(hwndDlg, IDC_CHECK_LOWPASS) == BST_CHECKED)
						EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT_LOWPASS), TRUE);
					else
						EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT_LOWPASS), FALSE);
					break;

				case IDC_EDIT_LOWPASS:

					switch (HIWORD(wParam)) {
						case EN_UPDATE: {

							SendDlgItemMessage(hwndDlg, IDC_EDIT_LOWPASS, WM_GETTEXT,
    	          						(WPARAM) 6, (LPARAM) buf2);

							if ( atoi(buf2) > 22050 ) {
								SendDlgItemMessage(hwndDlg, IDC_EDIT_LOWPASS, WM_SETTEXT,
									(WPARAM) 0, (LPARAM) "22050");

								SendDlgItemMessage(hwndDlg, IDC_EDIT_LOWPASS, EM_SETSEL,
									(WPARAM)0, (LPARAM)-1);
							}
						}
						break;
						default:
							break;
	        	  		}
					break;

				case IDC_RESAMPLE_NEW:

					if (IsDlgButtonChecked(hwndDlg, IDC_RESAMPLE_NEW) == BST_CHECKED) {
						EnableWindow(GetDlgItem(hwndDlg, IDC_RESAMPLE_RATE), TRUE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_CHECK1), TRUE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_CONVERTER), TRUE);
					}
					else {
						EnableWindow(GetDlgItem(hwndDlg, IDC_RESAMPLE_RATE), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_CHECK1), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_CONVERTER), FALSE);
					}
					break;

				default:
					break;
			}
			break;
	}
return FALSE; 
}

/*
 *  Advanced Encoder Setup dialog procedures.
 */

BOOL CALLBACK AdvEncSetup(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{

	switch (message) { 

		case WM_INITDIALOG: {

			char msg_buf[10];

			HICON hBgIcon = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON1));
			PostMessage(hwndDlg, WM_SETICON, ICON_BIG, (LPARAM)hBgIcon);

			if (iniSettings.use_damp == 1) {
				CheckDlgButton(hwndDlg, IDC_USE_DAMP, TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_BIT_AVE_DAMP), TRUE);
			}
			else
				EnableWindow(GetDlgItem(hwndDlg, IDC_BIT_AVE_DAMP), FALSE);

			sprintf(msg_buf, "%.4lf", iniSettings.bit_ave_damp);
			SendDlgItemMessage(hwndDlg, IDC_BIT_AVE_DAMP, WM_SETTEXT,
				(WPARAM)0, (LPARAM)msg_buf);

			if (iniSettings.use_bias == 1) {
				CheckDlgButton(hwndDlg, IDC_USE_BIAS, TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_BIT_RES_BIAS), TRUE);
			}
			else
				EnableWindow(GetDlgItem(hwndDlg, IDC_BIT_RES_BIAS), FALSE);

			sprintf(msg_buf, "%.4lf", iniSettings.bit_res_bias);
			SendDlgItemMessage(hwndDlg, IDC_BIT_RES_BIAS, WM_SETTEXT,
				(WPARAM)0, (LPARAM)msg_buf);

			if (iniSettings.use_bits == 1) {
				CheckDlgButton(hwndDlg, IDC_USE_BITS, TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_BIT_RES_BITS), TRUE);
			}
			else
				EnableWindow(GetDlgItem(hwndDlg, IDC_BIT_RES_BITS), FALSE);

			sprintf(msg_buf, "%d", iniSettings.bit_res_bits);
			SendDlgItemMessage(hwndDlg, IDC_BIT_RES_BITS, WM_SETTEXT,
				(WPARAM)0, (LPARAM)msg_buf);

			if (iniSettings.use_noise == 1) {
				CheckDlgButton(hwndDlg, IDC_USE_NOISE, TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_IMP_NOISE), TRUE);
			}
			else
				EnableWindow(GetDlgItem(hwndDlg, IDC_IMP_NOISE), FALSE);

			sprintf(msg_buf, "%.4lf", iniSettings.imp_noise);
			SendDlgItemMessage(hwndDlg, IDC_IMP_NOISE, WM_SETTEXT,
				(WPARAM)0, (LPARAM)msg_buf);

//			if (iniSettings.use_trigger == 1) {
//				CheckDlgButton(hwndDlg, IDC_USE_TRIGGER, TRUE);
//				EnableWindow(GetDlgItem(hwndDlg, IDC_IMP_TRIG), TRUE);
//			}
//			else
//				EnableWindow(GetDlgItem(hwndDlg, IDC_IMP_TRIG), FALSE);

//			sprintf(msg_buf, "%.4lf", iniSettings.imp_trigger);
//			SendDlgItemMessage(hwndDlg, IDC_IMP_TRIG, WM_SETTEXT,
//				(WPARAM)0, (LPARAM)msg_buf);
		}
		return TRUE;

		case WM_CLOSE:

			// Closing the Dialog behaves the same as Cancel               
			PostMessage(hwndDlg, WM_COMMAND, IDCANCEL, 0);
			break; // End of WM_CLOSE                                      

		case WM_COMMAND: 

			switch (LOWORD(wParam)) {
				case IDOK: {
					char     szTemp[64];

					if (IsDlgButtonChecked(hwndDlg, IDC_USE_DAMP) == BST_CHECKED) {
						GetDlgItemText(hwndDlg, IDC_BIT_AVE_DAMP, szTemp, sizeof(szTemp));
						set_bit_ave_damp(atof(szTemp));
						set_use_damp(1);
					}
					else
						set_use_damp(0);

					if (IsDlgButtonChecked(hwndDlg, IDC_USE_BIAS) == BST_CHECKED) {
						GetDlgItemText(hwndDlg, IDC_BIT_RES_BIAS, szTemp, sizeof(szTemp));
						set_bit_res_bias(atof(szTemp));
						set_use_bias(1);
					}
					else
						set_use_bias(0);

					if (IsDlgButtonChecked(hwndDlg, IDC_USE_BITS) == BST_CHECKED) {
						GetDlgItemText(hwndDlg, IDC_BIT_RES_BITS, szTemp, sizeof(szTemp));
						set_bit_res_bits(atoi(szTemp));
						set_use_bits(1);
					}
					else
						set_use_bits(0);

					if (IsDlgButtonChecked(hwndDlg, IDC_USE_NOISE) == BST_CHECKED) {
						GetDlgItemText(hwndDlg, IDC_IMP_NOISE, szTemp, sizeof(szTemp));
						set_imp_noise(atof(szTemp));
						set_use_noise(1);
					}
					else
						set_use_noise(0);

//					if (IsDlgButtonChecked(hwndDlg, IDC_USE_TRIGGER) == BST_CHECKED) {
//						GetDlgItemText(hwndDlg, IDC_IMP_TRIG, szTemp, sizeof(szTemp));
//						set_imp_trigger(atof(szTemp));
//						set_use_trigger(1);
//					}
//					else
//						set_use_trigger(0);

					EndDialog(hwndDlg, FALSE);
					break;
				}
				case IDCANCEL:
					EndDialog(hwndDlg, FALSE);
					break;

				case IDC_USE_DAMP:

					if (IsDlgButtonChecked(hwndDlg, IDC_USE_DAMP) == BST_CHECKED)
						EnableWindow(GetDlgItem(hwndDlg, IDC_BIT_AVE_DAMP), TRUE);
					else
						EnableWindow(GetDlgItem(hwndDlg, IDC_BIT_AVE_DAMP), FALSE);
					break;

				case IDC_USE_BIAS:

					if (IsDlgButtonChecked(hwndDlg, IDC_USE_BIAS) == BST_CHECKED)
						EnableWindow(GetDlgItem(hwndDlg, IDC_BIT_RES_BIAS), TRUE);
					else
						EnableWindow(GetDlgItem(hwndDlg, IDC_BIT_RES_BIAS), FALSE);
					break;

				case IDC_USE_BITS:

					if (IsDlgButtonChecked(hwndDlg, IDC_USE_BITS) == BST_CHECKED)
						EnableWindow(GetDlgItem(hwndDlg, IDC_BIT_RES_BITS), TRUE);
					else
						EnableWindow(GetDlgItem(hwndDlg, IDC_BIT_RES_BITS), FALSE);
					break;

				case IDC_USE_NOISE:

					if (IsDlgButtonChecked(hwndDlg, IDC_USE_NOISE) == BST_CHECKED)
						EnableWindow(GetDlgItem(hwndDlg, IDC_IMP_NOISE), TRUE);
					else
						EnableWindow(GetDlgItem(hwndDlg, IDC_IMP_NOISE), FALSE);
					break;

//				case IDC_USE_TRIGGER:

//					if (IsDlgButtonChecked(hwndDlg, IDC_USE_TRIGGER) == BST_CHECKED)
//						EnableWindow(GetDlgItem(hwndDlg, IDC_IMP_TRIG), TRUE);
//					else
//						EnableWindow(GetDlgItem(hwndDlg, IDC_IMP_TRIG), FALSE);
//					break;

			}
			break;
	}
return FALSE; 
}

/*
 *  Decode parameters dialog procedures.
 */

BOOL CALLBACK DecodeProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	int      br;
	char     szTemp[10];

	switch (message) {
		case WM_INITDIALOG: {
 
			HICON hBgIcon = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON1));
			PostMessage(hwndDlg, WM_SETICON, ICON_BIG, (LPARAM)hBgIcon);

			SendDlgItemMessage(hwndDlg, IDC_SCROLL_VOLUME, SBM_SETRANGEREDRAW, 
        			(WPARAM) 0, (LPARAM) 240);   // min. & max. positions 

		   	SendDlgItemMessage(hwndDlg, IDC_SCROLL_VOLUME, SBM_SETPOS, 
        			(WPARAM) (iniSettings.volume + 120),
		        	(LPARAM) TRUE);                  // redraw flag 

			sprintf(szTemp, "%.1fdB", (float)iniSettings.volume/10);

			SendDlgItemMessage(hwndDlg, IDC_EDITDB, WM_SETTEXT, (WPARAM)0, (LPARAM)szTemp);

			if (iniSettings.dec_tag_mode == 0) {
				CheckDlgButton(hwndDlg,IDC_NOGAIN,FALSE);
				CheckDlgButton(hwndDlg,IDC_RADIOGAIN,FALSE);
				CheckDlgButton(hwndDlg,IDC_ALBUMGAIN,FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_RADIOGAIN), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_ALBUMGAIN), FALSE);
			}
			else if (iniSettings.dec_tag_mode == 1) {
				CheckDlgButton(hwndDlg,IDC_NOGAIN,TRUE);
				CheckDlgButton(hwndDlg,IDC_RADIOGAIN,TRUE);
				CheckDlgButton(hwndDlg,IDC_ALBUMGAIN,FALSE);
			}
			else if (iniSettings.dec_tag_mode == 2) {
				CheckDlgButton(hwndDlg,IDC_NOGAIN,TRUE);
				CheckDlgButton(hwndDlg,IDC_RADIOGAIN,FALSE);
				CheckDlgButton(hwndDlg,IDC_ALBUMGAIN,TRUE);
			}

			for (br=0; noiseShaping[br] != 0; br++) {
				SendDlgItemMessage(hwndDlg, IDC_NOISE_SHAPING, CB_ADDSTRING,
	        	  		(WPARAM)0, (LPARAM) noiseShaping[br]);

				SendDlgItemMessage(hwndDlg, IDC_NOISE_SHAPING, CB_SETITEMDATA, 
	        	  		(WPARAM) br, (LPARAM) noiseShaping[br]);
			}

			if (iniSettings.dither == 0) {
				CheckDlgButton(hwndDlg,IDC_CHECK_DITHER,FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_NOISE_SHAPING), FALSE);
			}
			else {
				CheckDlgButton(hwndDlg,IDC_CHECK_DITHER,TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_NOISE_SHAPING), TRUE);
			}

			SendDlgItemMessage(hwndDlg, IDC_NOISE_SHAPING, CB_SETCURSEL,
	            		(WPARAM) iniSettings.dither, (LPARAM) 0);

			for (br=0; wavFormat[br] != 0; br++) {
				SendDlgItemMessage(hwndDlg, IDC_WAV_FORMAT, CB_ADDSTRING,
	        	  		(WPARAM)0, (LPARAM) wavFormat[br]);

				SendDlgItemMessage(hwndDlg, IDC_WAV_FORMAT, CB_SETITEMDATA, 
	        	  		(WPARAM) br, (LPARAM) wavFormat[br]);
			}

			if (iniSettings.decode_mode == 0) {
				SendDlgItemMessage(hwndDlg, IDC_WAV_FORMAT, CB_SETCURSEL,
		            		(WPARAM) 1, (LPARAM) 0);
			}
			else {
				SendDlgItemMessage(hwndDlg, IDC_WAV_FORMAT, CB_SETCURSEL,
		            		(WPARAM) iniSettings.wav_format, (LPARAM) 0);
			}

			if (iniSettings.mc_downmix == 1)
				CheckDlgButton(hwndDlg,IDC_MC_DOWNMIX,TRUE);
			else
				CheckDlgButton(hwndDlg,IDC_MC_DOWNMIX,FALSE);

			if (iniSettings.decode_mode == 0)
				CheckDlgButton(hwndDlg,IDC_PLAYBACK,TRUE);
			else
				CheckDlgButton(hwndDlg,IDC_DECODE,TRUE);

			if (iniSettings.limiter == 1)
				CheckDlgButton(hwndDlg,IDC_HARD_LIMITER,TRUE);

			if (iniSettings.del_input_ogg == 1)
				CheckDlgButton(hwndDlg,IDC_CHECK_DEL_OGG,TRUE);
		}

		return TRUE;

		case WM_HSCROLL: {

			int vol;
			switch (LOWORD(wParam)) {

				case SB_THUMBPOSITION:
				case SB_THUMBTRACK:
				   	SendDlgItemMessage(hwndDlg, IDC_SCROLL_VOLUME, SBM_SETPOS, 
        					(WPARAM) (HIWORD(wParam)),
		        			(LPARAM) TRUE);                  // redraw flag 
					break;
				case SB_LINELEFT:
				   	SendDlgItemMessage(hwndDlg, IDC_SCROLL_VOLUME, SBM_SETPOS, 
        					(WPARAM) (SendDlgItemMessage(hwndDlg, IDC_SCROLL_VOLUME, SBM_GETPOS, (WPARAM)0, (LPARAM)0) - 1),
		        			(LPARAM) TRUE);                  // redraw flag 
					break;
				case SB_LINERIGHT:
				   	SendDlgItemMessage(hwndDlg, IDC_SCROLL_VOLUME, SBM_SETPOS, 
        					(WPARAM) (SendDlgItemMessage(hwndDlg, IDC_SCROLL_VOLUME, SBM_GETPOS, (WPARAM)0, (LPARAM)0) + 1),
		        			(LPARAM) TRUE);                  // redraw flag 
					break;
			}

			vol = SendDlgItemMessage(hwndDlg, IDC_SCROLL_VOLUME, SBM_GETPOS, (WPARAM)0, (LPARAM)0) - 120;

			sprintf(szTemp, "%.1fdB", (float)vol/10);

			SendDlgItemMessage(hwndDlg, IDC_EDITDB, WM_SETTEXT, (WPARAM)0, (LPARAM)szTemp);
		}
		break;

		case WM_CLOSE:

			// Closing the Dialog behaves the same as Cancel               
			PostMessage(hwndDlg, WM_COMMAND, IDCANCEL, 0);
			break; // End of WM_CLOSE                                      

		case WM_COMMAND: 

			switch (LOWORD(wParam)) {
				case IDC_BUTTON2: {

					if ((IsDlgButtonChecked(hwndDlg, IDC_NOGAIN) == BST_CHECKED) &&
					    (IsDlgButtonChecked(hwndDlg, IDC_RADIOGAIN) == BST_CHECKED))
						set_dec_tag_mode(1);               // Radio gain
					else if ((IsDlgButtonChecked(hwndDlg, IDC_NOGAIN) == BST_CHECKED) &&
					    (IsDlgButtonChecked(hwndDlg, IDC_ALBUMGAIN) == BST_CHECKED))
						set_dec_tag_mode(2);               // Album gain
					else
						set_dec_tag_mode(0);               // No gain

					set_limiter(IsDlgButtonChecked(hwndDlg, IDC_HARD_LIMITER) == BST_CHECKED ? 1 : 0);

					if (IsDlgButtonChecked(hwndDlg, IDC_CHECK_DITHER) == BST_CHECKED) {
						if ((br = SendDlgItemMessage(hwndDlg, IDC_NOISE_SHAPING, CB_GETCURSEL,
								(WPARAM) 0, (LPARAM) 0)) != CB_ERR) {
							set_dither(br);
						}
					}
					else
						set_dither(0);            // Don't dither output

					if (IsDlgButtonChecked(hwndDlg, IDC_MC_DOWNMIX) == BST_CHECKED)
						set_mc_downmix(1);
					else
						set_mc_downmix(0);

					set_decode_mode(IsDlgButtonChecked(hwndDlg, IDC_PLAYBACK) == BST_CHECKED ? 0 : 1);

					if ((br = SendDlgItemMessage(hwndDlg, IDC_WAV_FORMAT, CB_GETCURSEL,
							(WPARAM) 0, (LPARAM) 0)) != CB_ERR) {
						set_wav_format(br);
					}

					set_del_input_ogg(IsDlgButtonChecked(hwndDlg, IDC_CHECK_DEL_OGG) == BST_CHECKED ? 1 : 0);

					set_volume(SendDlgItemMessage(hwndDlg, IDC_SCROLL_VOLUME, SBM_GETPOS,
							(WPARAM)0, (LPARAM)0) - 120);

					EndDialog(hwndDlg, FALSE);
				}
				break;

				case IDCANCEL:

					EndDialog(hwndDlg, FALSE);
					break;

				case IDC_PLAYBACK:

					SendDlgItemMessage(hwndDlg, IDC_WAV_FORMAT, CB_SETCURSEL,
			            		(WPARAM) 1, (LPARAM) 0);
					break;

				case IDC_NOGAIN:

					if (IsDlgButtonChecked(hwndDlg, IDC_NOGAIN) == BST_CHECKED) {
						EnableWindow(GetDlgItem(hwndDlg, IDC_RADIOGAIN), TRUE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_ALBUMGAIN), TRUE);
					}
					else {
						EnableWindow(GetDlgItem(hwndDlg, IDC_RADIOGAIN), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_ALBUMGAIN), FALSE);
					}
					break;

				case IDC_WAV_FORMAT:

					if (IsDlgButtonChecked(hwndDlg, IDC_PLAYBACK) == BST_CHECKED)
						SendDlgItemMessage(hwndDlg, IDC_WAV_FORMAT, CB_SETCURSEL,
				            		(WPARAM) 1, (LPARAM) 0);
					break;

				case IDC_CHECK_DITHER:

					if (IsDlgButtonChecked(hwndDlg, IDC_CHECK_DITHER) == BST_CHECKED) {
						EnableWindow(GetDlgItem(hwndDlg, IDC_NOISE_SHAPING), TRUE);
						if ((br = SendDlgItemMessage(hwndDlg, IDC_NOISE_SHAPING, CB_GETCURSEL,
								(WPARAM) 0, (LPARAM) 0)) != CB_ERR) {
							if (br == 0)
								SendDlgItemMessage(hwndDlg, IDC_NOISE_SHAPING, CB_SETCURSEL,
						            		(WPARAM) 1, (LPARAM) 0);
						}
					}
					else {
						EnableWindow(GetDlgItem(hwndDlg, IDC_NOISE_SHAPING), FALSE);
						SendDlgItemMessage(hwndDlg, IDC_NOISE_SHAPING, CB_SETCURSEL,
				            		(WPARAM) 0, (LPARAM) 0);
					}
					break;

				case IDC_NOISE_SHAPING:

					if (IsDlgButtonChecked(hwndDlg, IDC_CHECK_DITHER) == BST_CHECKED) {
						if ((br = SendDlgItemMessage(hwndDlg, IDC_NOISE_SHAPING, CB_GETCURSEL,
								(WPARAM) 0, (LPARAM) 0)) != CB_ERR) {
							if (br == 0)
								SendDlgItemMessage(hwndDlg, IDC_NOISE_SHAPING, CB_SETCURSEL,
						            		(WPARAM) 1, (LPARAM) 0);
						}
					}
					break;

				default:
					break;
			}
			break;
	}
return FALSE; 
}

/*
 *  Tag parameters dialog procedures.
 */

BOOL CALLBACK TagProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	int      br;

	switch (message) { 
		case WM_INITDIALOG: {
 
			HICON hBgIcon = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON1));
			PostMessage(hwndDlg, WM_SETICON, ICON_BIG, (LPARAM)hBgIcon);

			GetGenreData();

			SendDlgItemMessage(hwndDlg, IDC_ENCBY, WM_SETTEXT,
				(WPARAM)0, (LPARAM)iniSettings.enc_by);
			SendDlgItemMessage(hwndDlg, IDC_ARTIST, WM_SETTEXT,
				(WPARAM)0, (LPARAM)iniSettings.artist);
			SendDlgItemMessage(hwndDlg, IDC_ALBUM, WM_SETTEXT,
				(WPARAM)0, (LPARAM)iniSettings.album);
			SendDlgItemMessage(hwndDlg, IDC_TITLE, WM_SETTEXT,
				(WPARAM)0, (LPARAM)iniSettings.title);
			SendDlgItemMessage(hwndDlg, IDC_YEAR, WM_SETTEXT,
				(WPARAM)0, (LPARAM)iniSettings.year);
			SendDlgItemMessage(hwndDlg, IDC_TRACK_NO, WM_SETTEXT,
				(WPARAM)0, (LPARAM)iniSettings.track_no);
			SendDlgItemMessage(hwndDlg, IDC_SEPARATOR, WM_SETTEXT,
				(WPARAM)0, (LPARAM)iniSettings.separator);

			if (iniSettings.tagging == 8) {
				CheckDlgButton(hwndDlg,IDC_CHECK_AUTO_TAG,TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_GENRE), TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_GENRE_2), TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_YEAR), TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_ENCBY), TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_ARTIST), TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_ALBUM), TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_TITLE), TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_TRACK_NO), TRUE);
			}
			else {
				CheckDlgButton(hwndDlg,IDC_CHECK_AUTO_TAG,FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_GENRE), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_GENRE_2), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_YEAR), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_ENCBY), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_ARTIST), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_ALBUM), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_TITLE), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_TRACK_NO), FALSE);
			}
			
			for (br=0; GenreData[br][0]; br++) {
				SendDlgItemMessage(hwndDlg, IDC_GENRE, CB_ADDSTRING,
	        	  		(WPARAM)0, (LPARAM) GenreData[br]);

				SendDlgItemMessage(hwndDlg, IDC_GENRE, CB_SETITEMDATA, 
	        	  		(WPARAM) br, (LPARAM) GenreData[br]);
			}

			if (iniSettings.genre[0] == '%') {
				SendDlgItemMessage(hwndDlg, IDC_GENRE_2, WM_SETTEXT,
					(WPARAM)0, (LPARAM)iniSettings.genre);
			}
			else {
			   	for (br=0; GenreData[br][0]; br++) {
					if ( ! strcmp(GenreData[br], iniSettings.genre )) {
       						SendDlgItemMessage(hwndDlg, IDC_GENRE, CB_SETCURSEL,
				            		(WPARAM) br, (LPARAM) 0);
       						break;
					}
				}
			}

			/* Set Rename on Tag */
			SendDlgItemMessage(hwndDlg, IDC_FILENAME, WM_SETTEXT,
				(WPARAM)0, (LPARAM)iniSettings.rename_format);
			if (iniSettings.rename_on_tag) {
				CheckDlgButton(hwndDlg,IDC_RENAME_FILE_ON_TAG,TRUE);
			}
			else
				EnableWindow(GetDlgItem(hwndDlg, IDC_FILENAME), FALSE);

			SendDlgItemMessage(hwndDlg, IDC_EDIT_PLAYLIST, WM_SETTEXT,
				(WPARAM)0, (LPARAM)iniSettings.playlist_format);
			if (iniSettings.playlist_on_tag) {
				CheckDlgButton(hwndDlg,IDC_CHECK_PLAYLIST,TRUE);
			}
			else
				EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT_PLAYLIST), FALSE);

			if (iniSettings.copy_flac_tag) {
				CheckDlgButton(hwndDlg,IDC_FLAC_COMMENTS,TRUE);
				CheckDlgButton(hwndDlg,IDC_CHECK_AUTO_TAG,FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_CHECK_USE_FLAC), TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_FILENAME_FLAC), TRUE);
			}
			else {
				EnableWindow(GetDlgItem(hwndDlg, IDC_CHECK_USE_FLAC), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_FILENAME_FLAC), FALSE);
			}

			if (iniSettings.FLAC_scale) {
				CheckDlgButton(hwndDlg,IDC_FLAC_COMMENTS,TRUE);
				CheckDlgButton(hwndDlg,IDC_CHECK_AUTO_TAG,FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_CHECK_AUTO_TAG), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_FLAC_COMMENTS), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_GENRE), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_GENRE_2), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_YEAR), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_ENCBY), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_ARTIST), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_ALBUM), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_TITLE), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_TRACK_NO), FALSE);
			}

			SendDlgItemMessage(hwndDlg, IDC_FILENAME_FLAC, WM_SETTEXT,
				(WPARAM)0, (LPARAM)iniSettings.rename_format_flac);
			if (iniSettings.use_flac_tag) {
				CheckDlgButton(hwndDlg,IDC_CHECK_USE_FLAC,TRUE);
			}
			else
				EnableWindow(GetDlgItem(hwndDlg, IDC_FILENAME_FLAC), FALSE);

		}
		return TRUE;

		case WM_CLOSE:

			// Closing the Dialog behaves the same as Cancel               
			PostMessage(hwndDlg, WM_COMMAND, IDCANCEL, 0);
			break; // End of WM_CLOSE                                      

		case WM_COMMAND: 

			switch (LOWORD(wParam)) {
				case IDC_BUTTON5: {

					char     szTemp[64];

					GetDlgItemText(hwndDlg, IDC_GENRE_2, szTemp, sizeof(szTemp));
					if (strlen(szTemp) > 0) {
						set_genre(szTemp);
					}
					else if ((br = SendDlgItemMessage(hwndDlg, IDC_GENRE, CB_GETCURSEL,
							(WPARAM) 0, (LPARAM) 0)) != CB_ERR) {
						set_genre(GenreData[br]);
					}
					else {
						FILE*   fp;
						FILE*   fp_temp;
						char    buff [64];

						GetDlgItemText(hwndDlg, IDC_GENRE, szTemp, sizeof(szTemp));
						if (strlen(szTemp) > 0) {
							set_genre(szTemp);

							if ((fp = fopen (GENRE_FILE, "r")) == NULL)
								MessageBox(NULL, "Unable to open GenreData.txt File",
								           NULL, MB_OK | MB_ICONEXCLAMATION);
							else if ((fp_temp = fopen ("genre.tmp", "w")) == NULL)
								MessageBox(NULL, "Unable to open genre.tmp File",
								           NULL, MB_OK | MB_ICONEXCLAMATION);
							else {
								while (fgets(buff, sizeof(buff), fp) != NULL) {
									if (strcmp(buff, szTemp) <= 0)
										fprintf (fp_temp, "%s", buff);
									else if (strcmp(buff, szTemp) > 0) {
										fprintf (fp_temp, "%s\n", szTemp);
										fprintf (fp_temp, "%s", buff);
										while (fgets(buff, sizeof(buff), fp) != NULL)
											fprintf (fp_temp, "%s", buff);
									}
								}
								fclose (fp);
								fclose (fp_temp);
								remove (GENRE_FILE);
								rename ("genre.tmp", GENRE_FILE);
							}
						}
						else
							set_genre("");
					}
					GetDlgItemText(hwndDlg, IDC_YEAR, szTemp, sizeof(szTemp));
					set_year(szTemp);
					GetDlgItemText(hwndDlg, IDC_ENCBY, szTemp, sizeof(szTemp));
					set_enc_by(szTemp);
					GetDlgItemText(hwndDlg, IDC_TITLE, szTemp, sizeof(szTemp));
					set_title(szTemp);
					GetDlgItemText(hwndDlg, IDC_ARTIST, szTemp, sizeof(szTemp));
					set_artist(szTemp);
					GetDlgItemText(hwndDlg, IDC_ALBUM, szTemp, sizeof(szTemp));
					set_album(szTemp);
					GetDlgItemText(hwndDlg, IDC_TRACK_NO, szTemp, sizeof(szTemp));
					set_track_no(szTemp);
					GetDlgItemText(hwndDlg, IDC_SEPARATOR, szTemp, sizeof(szTemp));
					set_separator(szTemp);
					set_rename_on_tag(IsDlgButtonChecked(hwndDlg, IDC_RENAME_FILE_ON_TAG) == BST_CHECKED ? 1 : 0);
					GetDlgItemText(hwndDlg, IDC_FILENAME, szTemp, sizeof(szTemp));
					set_rename_format(szTemp);
					set_playlist_on_tag(IsDlgButtonChecked(hwndDlg, IDC_CHECK_PLAYLIST) == BST_CHECKED ? 1 : 0);
					GetDlgItemText(hwndDlg, IDC_EDIT_PLAYLIST, szTemp, sizeof(szTemp));
					set_playlist_format(szTemp);
					set_copy_flac_tag(IsDlgButtonChecked(hwndDlg, IDC_FLAC_COMMENTS) == BST_CHECKED ? 1 : 0);
					set_use_flac_tag(IsDlgButtonChecked(hwndDlg, IDC_CHECK_USE_FLAC) == BST_CHECKED ? 1 : 0);
					GetDlgItemText(hwndDlg, IDC_FILENAME_FLAC, szTemp, sizeof(szTemp));
					set_rename_format_flac(szTemp);

					EndDialog(hwndDlg, FALSE);
				}
				break;

				case IDCANCEL:

					EndDialog(hwndDlg, FALSE);
					break;

				case IDC_CHECK_AUTO_TAG:

					if (IsDlgButtonChecked(hwndDlg, IDC_CHECK_AUTO_TAG) == BST_CHECKED) {
						CheckDlgButton(hwndDlg,IDC_FLAC_COMMENTS,FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_GENRE), TRUE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_GENRE_2), TRUE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_YEAR), TRUE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_ENCBY), TRUE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_ARTIST), TRUE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_ALBUM), TRUE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_TITLE), TRUE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_TRACK_NO), TRUE);
						CheckDlgButton(hwndDlg,IDC_CHECK_USE_FLAC,FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_CHECK_USE_FLAC), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_FILENAME_FLAC), FALSE);
						set_AUTOtagging(hwndDlg, 8);
					}
					else {
						EnableWindow(GetDlgItem(hwndDlg, IDC_GENRE), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_GENRE_2), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_YEAR), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_ENCBY), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_ARTIST), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_ALBUM), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_TITLE), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_TRACK_NO), FALSE);
						set_AUTOtagging(hwndDlg, 0);
					}
					break;

				case IDC_FLAC_COMMENTS:

					if (IsDlgButtonChecked(hwndDlg, IDC_FLAC_COMMENTS) == BST_CHECKED) {
						CheckDlgButton(hwndDlg,IDC_CHECK_AUTO_TAG,FALSE);
						set_AUTOtagging(hwndDlg, 0);
						EnableWindow(GetDlgItem(hwndDlg, IDC_GENRE), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_GENRE_2), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_YEAR), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_ENCBY), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_ARTIST), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_ALBUM), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_TITLE), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_TRACK_NO), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_CHECK_USE_FLAC), TRUE);
					}
					else {
						EnableWindow(GetDlgItem(hwndDlg, IDC_CHECK_USE_FLAC), FALSE);
						CheckDlgButton(hwndDlg,IDC_CHECK_USE_FLAC,FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_FILENAME_FLAC), FALSE);
					}

					break;

				case IDC_CHECK_USE_FLAC:

					if (IsDlgButtonChecked(hwndDlg, IDC_CHECK_USE_FLAC) == BST_CHECKED) {
						EnableWindow(GetDlgItem(hwndDlg, IDC_FILENAME_FLAC), TRUE);
						CheckDlgButton(hwndDlg,IDC_RENAME_FILE_ON_TAG,FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_FILENAME), FALSE);
					}
					else
						EnableWindow(GetDlgItem(hwndDlg, IDC_FILENAME_FLAC), FALSE);
					break;

				case IDC_RENAME_FILE_ON_TAG:

					if (IsDlgButtonChecked(hwndDlg, IDC_RENAME_FILE_ON_TAG) == BST_CHECKED) {
						EnableWindow(GetDlgItem(hwndDlg, IDC_FILENAME), TRUE);
						CheckDlgButton(hwndDlg,IDC_CHECK_USE_FLAC,FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_FILENAME_FLAC), FALSE);
					}
					else
						EnableWindow(GetDlgItem(hwndDlg, IDC_FILENAME), FALSE);
					break;

				case IDC_CHECK_PLAYLIST:

					if (IsDlgButtonChecked(hwndDlg, IDC_CHECK_PLAYLIST) == BST_CHECKED)
						EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT_PLAYLIST), TRUE);
					else
						EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT_PLAYLIST), FALSE);
					break;

				case IDC_TAG_HELP:

					if (IsDlgButtonChecked(hwndDlg, IDC_CHECK_USE_FLAC) == BST_CHECKED)
						DialogBox(hinst, MAKEINTRESOURCE(IDD_FLAC_HELP), hwndDlg, FLACHelpProc);
					else
						DialogBox(hinst, MAKEINTRESOURCE(IDD_TAG_HELP), hwndDlg, TagHelpProc);

					break;

				default:
					break;
			}
			break;
	}
return FALSE; 
}

/*
 *  Tag parameters dialog procedures.
 */

BOOL CALLBACK TagHelpProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{

	switch (message) { 

		case WM_INITDIALOG: {

			HICON hBgIcon = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON1));
			PostMessage(hwndDlg, WM_SETICON, ICON_BIG, (LPARAM)hBgIcon);

		}
		return TRUE;

		case WM_CLOSE:

			// Closing the Dialog behaves the same as Cancel               
			PostMessage(hwndDlg, WM_COMMAND, IDCANCEL, 0);
			break; // End of WM_CLOSE                                      

		case WM_COMMAND: 

			switch (LOWORD(wParam)) {
				case IDCANCEL:
					EndDialog(hwndDlg, FALSE);
					break;
			}
			break;
	}
return FALSE; 
}

/*
 *  Tag parameters dialog procedures.
 */

BOOL CALLBACK FLACHelpProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) { 

		case WM_INITDIALOG: {

			HICON hBgIcon = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON1));
			PostMessage(hwndDlg, WM_SETICON, ICON_BIG, (LPARAM)hBgIcon);

		}
		return TRUE;

		case WM_CLOSE:

			// Closing the Dialog behaves the same as Cancel               
			PostMessage(hwndDlg, WM_COMMAND, IDCANCEL, 0);
			break; // End of WM_CLOSE                                      

		case WM_COMMAND: 

			switch (LOWORD(wParam)) {
				case IDCANCEL:
					EndDialog(hwndDlg, FALSE);
					break;
			}
			break;
	}
return FALSE; 
}

 
/*
 *  Directory parameters dialog procedures.
 */

BOOL CALLBACK DirProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
		case WM_INITDIALOG: {

			HICON hBgIcon = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON1));
			PostMessage(hwndDlg, WM_SETICON, ICON_BIG, (LPARAM)hBgIcon);

			if (iniSettings.out_dir_mode == 1) {
				CheckDlgButton(hwndDlg,IDC_USEOTHERDEF,TRUE);
				SetDlgItemText(hwndDlg, IDC_EDITDIR, config_OGGoutdir);
			}
			else {
				CheckDlgButton(hwndDlg,IDC_USESAMEDIR,TRUE);
//				EnableWindow(GetDlgItem(hwndDlg, IDC_EDITDIR), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_SETDIR), FALSE);
				SetDlgItemText(hwndDlg, IDC_EDITDIR, "");
			}
		}
		return TRUE;

		case WM_CLOSE:

			// Closing the Dialog behaves the same as Cancel               
			PostMessage(hwndDlg, WM_COMMAND, IDCANCEL, 0);
			break; // End of WM_CLOSE                                      

		case WM_COMMAND: 

			switch (LOWORD(wParam)) {
				case IDC_BUTTON4: {

					char     szTemp_2[MAX_PATH];
					unsigned int i = 0;
					unsigned int j = 0;
					char     sub[MAX_PATH];

					if (IsDlgButtonChecked(hwndDlg, IDC_USESAMEDIR) == BST_CHECKED) {
						strcpy(config_OGGoutdir, "");
						set_out_dir("");
						set_out_dir_mode(0);
					}
					else {
						GetDlgItemText(hwndDlg, IDC_EDITDIR, szTemp_2, sizeof(szTemp_2));
						if (!szTemp_2[0]) {
							set_out_dir_mode(0);
							strcpy(config_OGGoutdir, "");
						}
						else {
							j = strlen(szTemp_2) - 1;
							if (szTemp_2[j] != '\\') {
								szTemp_2[j+1] = '\\';
								szTemp_2[j+2] = '\0';
							}
/*							for (i = 0; i < strlen(szTemp_2); i++) {
								if (szTemp_2[i] == '\\') {
									sub[i] = szTemp_2[i];
									sub[i+1] = '\0';
								}
								else
									sub[i] = szTemp_2[i];
							}
							sub[i-1] = '\0';
							strcpy(config_OGGoutdir, sub); */
							strcpy(config_OGGoutdir, szTemp_2);
						}
						set_out_dir_mode(IsDlgButtonChecked(hwndDlg, IDC_USEOTHERDEF) == BST_CHECKED ? 1 : 0);
						set_out_dir(config_OGGoutdir);
					}

					EndDialog(hwndDlg, FALSE);
				}
				break;

				case IDCANCEL:

					EndDialog(hwndDlg, FALSE);
					break;

				case IDC_USESAMEDIR:

					SetDlgItemText(hwndDlg, IDC_EDITDIR, "");
//					EnableWindow(GetDlgItem(hwndDlg, IDC_EDITDIR), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_SETDIR), FALSE);
					break;
									
				case IDC_USEOTHERDIR:

//					EnableWindow(GetDlgItem(hwndDlg, IDC_EDITDIR), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_SETDIR), TRUE);
					break;

				case IDC_USEOTHERDEF:

//					EnableWindow(GetDlgItem(hwndDlg, IDC_EDITDIR), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_SETDIR), TRUE);
					break;

									
				case IDC_SETDIR: {
					char        szTemp[MAX_PATH];
					char        name[MAX_PATH];
					BROWSEINFO  bi;
					ITEMIDLIST  *idlist;

					bi.hwndOwner = hwndDlg;
					bi.pidlRoot = 0;
					bi.pszDisplayName = name;
					bi.lpszTitle = "Select a directory for output:";
//					bi.ulFlags = BIF_RETURNONLYFSDIRS;
					bi.ulFlags = BIF_NEWDIALOGSTYLE;
					bi.lpfn = BrowseCallbackProc;
					bi.lParam = 0;
					bi.iImage = 0;
					OleInitialize(NULL);
					idlist = SHBrowseForFolder( &bi );

					if (idlist) 
						SHGetPathFromIDList( idlist, szTemp );
					else
						strcpy(szTemp,"");

					SetDlgItemText(hwndDlg, IDC_EDITDIR, szTemp);
				}
				break;

				default:
					break;
			}
			break;
	}
return FALSE; 
}

/*
 *  Temporary Directory parameters dialog procedures.
 */

BOOL CALLBACK TempProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
		case WM_INITDIALOG: {
 
			HICON hBgIcon = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON1));
			PostMessage(hwndDlg, WM_SETICON, ICON_BIG, (LPARAM)hBgIcon);

			if (iniSettings.temp_dir[0]) {
				SetDlgItemText(hwndDlg, IDC_EDITEMP, config_TEMPdir);
			}
			else {
				SetDlgItemText(hwndDlg, IDC_EDITEMP, "");
			}
		}
		return TRUE;

		case WM_CLOSE:

			// Closing the Dialog behaves the same as Cancel               
			PostMessage(hwndDlg, WM_COMMAND, IDCANCEL, 0);
			break; // End of WM_CLOSE                                      

		case WM_COMMAND: 
			switch (LOWORD(wParam)) {
									
				case IDC_BUTTON3: {

					char     szTemp[MAX_PATH];
					GetDlgItemText(hwndDlg, IDC_EDITEMP, szTemp, sizeof(szTemp));
					strcpy(config_TEMPdir, szTemp);
					if (!szTemp[0])
						strcpy(config_TEMPdir, "");
					set_temp_dir(config_TEMPdir);

					EndDialog(hwndDlg, FALSE);
				}
				break;

				case IDCANCEL:

					EndDialog(hwndDlg, FALSE);
					break;

				case IDC_SETEMP: {

					char        szTemp[MAX_PATH];
					char        name[MAX_PATH];
					BROWSEINFO  bi;
					ITEMIDLIST  *idlist;

					bi.hwndOwner = hwndDlg;
					bi.pidlRoot = 0;
					bi.pszDisplayName = name;
					bi.lpszTitle = "Select a temporary directory for lossless decoding:";
//					bi.ulFlags = BIF_RETURNONLYFSDIRS;
					bi.ulFlags = BIF_NEWDIALOGSTYLE;
					bi.lpfn = BrowseCallbackProcTemp;
					bi.lParam = 0;
					bi.iImage = 0;
					OleInitialize(NULL);
					idlist = SHBrowseForFolder( &bi );

					if (idlist) 
						SHGetPathFromIDList( idlist, szTemp );
					else
						strcpy(szTemp,"");

					SetDlgItemText(hwndDlg, IDC_EDITEMP, szTemp);
				}
				break;
				default:
					break;
			}
			break;
	}
return FALSE; 
}

/*
 *  Show Log parameters dialog procedures.
 */

BOOL CALLBACK LogProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
		case WM_INITDIALOG: {

			char    *log_data;
			FILE*   fp = NULL;
			char    PathAndName[MAX_PATH];
			char    buff[MAX_PATH+18];
			int     len;

			HICON hBgIcon = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON1));
			PostMessage(hwndDlg, WM_SETICON, ICON_BIG, (LPARAM)hBgIcon);

			memset(buff, '\0', sizeof(buff));
			// Read Log File

			strcpy(PathAndName, CmdDir);
			strcat(PathAndName, LOG_FILE);
			if ((fp = fopen(PathAndName, "r")) == NULL) {
				MessageBox(NULL, "No Log File available", NULL, MB_OK | MB_ICONEXCLAMATION);
				EndDialog(hwndDlg, TRUE);
				return TRUE;
			}
			if (fgets(buff, sizeof(buff), fp) != NULL) {
				len = strlen(buff);
				if (len > 0) {
					if (buff[len-1] == '\n') {
						buff[len-1] = '\r';
						buff[len]   = '\n';
						buff[len+1] = '\0';
					}
					log_data = malloc(strlen(buff) + 1);
					if (!log_data) {
						MessageBox(NULL, "Unable to allocate memory to display log file", NULL, MB_OK | MB_ICONEXCLAMATION);
						EndDialog(hwndDlg, TRUE);
						return TRUE;
					}
					else
						memset(log_data, '\0', strlen(buff));
					strcpy(log_data, buff);
					while (fgets(buff, sizeof(buff), fp) != NULL) {
						len = strlen(buff);
						if (buff[len-1] == '\n') {
							buff[len-1] = '\r';
							buff[len]   = '\n';
							buff[len+1] = '\0';
						}
						log_data = realloc(log_data, strlen(log_data) + strlen(buff) + 1);
						if (!log_data) {
							MessageBox(NULL, "Unable to re-allocate memory to display log file", NULL, MB_OK | MB_ICONEXCLAMATION);
							break;
						}
						strcat(log_data, buff);
					}
				}
			}
			else {
				log_data = malloc(sizeof(buff));
				if (!log_data) {
					MessageBox(NULL, "Unable to allocate memory to display log file", NULL, MB_OK | MB_ICONEXCLAMATION);
					EndDialog(hwndDlg, TRUE);
					return TRUE;
				}
				else
					memset(log_data, '\0', sizeof(buff));
				strcpy(log_data, "");
			}

			SetDlgItemText(hwndDlg, IDC_SHOW_LOG, log_data);

			if (log_data)
				free(log_data);
			fclose (fp);
		}
		return TRUE;

		case WM_CLOSE:

			// Closing the Dialog behaves the same as Cancel               
			PostMessage(hwndDlg, WM_COMMAND, IDCANCEL, 0);
			break; // End of WM_CLOSE                                      

		case WM_COMMAND: 

			switch (LOWORD(wParam)) {
				case IDCANCEL:

					EndDialog(hwndDlg, FALSE);
					break;

				case IDC_CLEAR_LOG: {

					int     result;
					FILE*   fp = NULL;
					char    PathAndName[MAX_PATH];
					strcpy(PathAndName, CmdDir);
					strcat(PathAndName, LOG_FILE);
					result = MessageBox(NULL, "Clear Log File, are you sure?", "Clear Log File?", MB_YESNO | MB_TOPMOST | MB_ICONEXCLAMATION);
					switch (result) {
						case IDYES:
							SetDlgItemText(hwndDlg, IDC_SHOW_LOG, "");
							if ((fp = fopen(PathAndName, "w")) == NULL)
								MessageBox(NULL, "Unable to create new Log File", NULL, MB_OK | MB_TOPMOST | MB_ICONEXCLAMATION);
							fclose (fp);
							break;

						case IDNO:
							break;
					}
				}
					break;
			}
			break;
	}
return FALSE; 
}

/*
 *  About parameters dialog procedures.
 */

BOOL CALLBACK AboutProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
		case WM_INITDIALOG: {
 
			HICON hBgIcon = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON1));
			PostMessage(hwndDlg, WM_SETICON, ICON_BIG, (LPARAM)hBgIcon);

			SetDlgItemText(hwndDlg, IDC_COMPILE_DATE, "Compiled: " __DATE__);
		}
		return TRUE;

		case WM_CLOSE:

			// Closing the Dialog behaves the same as Cancel               
			PostMessage(hwndDlg, WM_COMMAND, IDCANCEL, 0);
			break; // End of WM_CLOSE                                      

		case WM_COMMAND: 

			switch (LOWORD(wParam)) {
				case IDCANCEL:
					EndDialog(hwndDlg, FALSE);
					break;
			}
			break;
	}
return FALSE; 
}

/*
 * end of main.c
 */

